! In this example, a dataset is created with one table and one
! array.
! The generic subroutine displayBlock, which operates on the
! BlockT base type. The blockType() function operates on objects
! of type BlockT.
! The example also shows blocks being retrieved from the dataset
! both by name and by number.
subroutine displayBlock( thisBlock )
  use dal

  implicit none

  type(BlockT) thisBlock

  write(*,*) "The block with name ", name( thisBlock )

  if( blockType( thisBlock ) .eq. ARRAY_BLOCK ) then
    write(*,*) " is an array."
  end if

  if( blockType( thisBlock ) .eq. TABLE_BLOCK ) then
    write(*,*) " is a table."
  end if

end subroutine displayBlock

subroutine displayBlocks( thisSet )
  use dal

  implicit none

  type(DataSetT) thisSet
  integer i

  interface
    subroutine displayBlock( blk )
      use dal
      implicit none
      type(BlockT), intent(in) :: blk

    end subroutine displayBlock
  end interface
  call foreachblock( thisSet, displayBlock )

end subroutine displayBlocks

program example_foreachblock
   
  use dal

  implicit none
   
  type(DataSetT) set
  type(TableT) tab
  type(ArrayT) arr
  integer, dimension(3), parameter :: s = (/ 3,4,2 /)

  set = dataSet("test.dat",CREATE)
  tab = addTable(set,"table",10);
  arr = addArray(set, "array", INTEGER32, dimensions=s )

  call displayBlock( block( tab ) )
  call displayBlock( block( arr ) )
  call displayBlock( block( set, "table", READ ) )
  call displayBlock( block( set, "array", READ ) )
  call displayBlocks( set )

  call release(set)
   
end program example_foreachblock
