

module Dal

	use ErrorHandling
	use types
	use Daltypes

        implicit none

	private


	! =======
	! Publics
	! =======

	! ====================
	! From module Daltypes
	! ====================

	public :: ArrayT
	public :: BlockT
	public :: ColumnT
	public :: DataSetT
	public :: TableT
        public :: SubTableT
        public :: RowT
	public :: AttributeT
	public :: AttributableT
	public :: LabelledT
	public :: DataComponentT
	public :: NullableT
	public :: TableSeekerT


	! =================
	! From module types
	! =================

	public :: BOOL
	public :: INT8
	public :: INT16
	public :: INT32
	public :: SINGLE
	public :: DOUBLE


	! ===========
	! This module
	! ===========

	integer, public, parameter :: BOOLEAN = 1	! int
	integer, public, parameter :: INTEGER8 = 2	! char
	integer, public, parameter :: UINTEGER16 = 3	! unsigned short
	integer, public, parameter :: INTEGER16 = 4	! short
	integer, public, parameter :: UINTEGER32 = 5	! unsigned int/long
	integer, public, parameter :: INTEGER32 = 6	! int/long
	integer, public, parameter :: REAL32 = 7	! float
	integer, public, parameter :: REAL64 = 8	! double
	integer, public, parameter :: STRING = 9	! char[]

	integer, public, parameter :: INTEGER_NULL = 1	! integer null value
	integer, public, parameter :: REAL_NULL = 2	! real null value
	integer, public, parameter :: STRING_NULL = 3	! string null value
	integer, public, parameter :: UNDEFINED_NULL = 4! undefined null value
	integer, public, parameter :: BOOLEAN_ATTRIBUTE = 1	! boolean attribute value
	integer, public, parameter :: INTEGER_ATTRIBUTE = 2	! integer attribute value
	integer, public, parameter :: REAL_ATTRIBUTE = 3	! real attribute value
	integer, public, parameter :: STRING_ATTRIBUTE = 4	! string attribute value
	integer, public, parameter :: READ = 1		! Read
	integer, public, parameter :: CREATE = 2	! Create and write
	integer, public, parameter :: WRITE = 2		! Create and write
	integer, public, parameter :: MODIFY = 3	! Read and Write
	integer, public, parameter :: TEMP = 4		! Temporary/discard
	integer, public, parameter :: AS_PARENT = 5	! Use access mode of parent

	integer, public, parameter :: TABLE_BLOCK = 1	! table block type
	integer, public, parameter :: ARRAY_BLOCK = 2	! array block type

	integer, public, parameter :: HIGH_MEMORY = 1	! High memory model
	integer, public, parameter :: HIGH_LOW_MEMORY = 2! Low memory model
	integer, public, parameter :: LOW_MEMORY = 3	! Low memory model
	integer, public, parameter :: USE_ENVIRONMENT = 4! As specified in env.

	public :: addArray
	public :: addAttributes
	public :: addColumn
	public :: addComment
	public :: addHistory
	public :: addTable
	public :: array
	public :: arrayDataType
	! public :: arrayUnits       ! obsolete; use units
	public :: attributable
	public :: attribute
	! public :: attributeComment ! obsolete: use label
	! public :: attributeUnits   ! obsolete: use units
	public :: block
	public :: blockNumber
	public :: blockType
	public :: booleanAttribute
	public :: boolArray2Data
	public :: boolArray3Data
	public :: boolArray4Data
	public :: boolData
	public :: boolVectorData
	public :: cleanupClient
        public :: client
	public :: clobber
	public :: clone
	public :: column
	public :: columnDataType
	public :: columnNumber
	! public :: columnUnits       ! obsolete; use units
	! public :: copyAttributes    ! obsolete; use setAttributes/addAttributes
	public :: copyDataSet
	public :: copyBlock
	public :: copyColumn
	public :: copyRows
	public :: count
	public :: dataComponent
	public :: dataSet
	public :: dataType
	public :: deleteAttribute
	public :: deleteBlock
	public :: deleteColumn
	public :: deleteDataSet
	public :: deleteRows
	public :: dimensions
	public :: discardDataSet
	public :: forEachBlock
	public :: forEachColumn
	public :: forEachSubTable
	public :: forEachRow
	public :: from
	public :: hasAttribute
	public :: hasBlock
	public :: hasColumn
	public :: hasNulls
	public :: hasScaling
	public :: insertRows
	public :: int8Array2Data
	public :: int8Array3Data
	public :: int8Array4Data
	public :: int8Attribute
	public :: int8Data
	public :: int8VectorData
	public :: int16Array2Data
	public :: int16Array3Data
	public :: int16Array4Data
	public :: int16Attribute
	public :: int16Data
	public :: int16VectorData
	public :: int32Array2Data
	public :: int32Array3Data
	public :: int32Array4Data
	public :: int32Attribute
	public :: int32Data
	public :: int32VectorData
	public :: intNullValue
	public :: isNotNull
	public :: isNull
	public :: label
	public :: labelled
	public :: keepDataSet
	public :: mode
	public :: name
	public :: newTableSeeker
	public :: next
	public :: nullable
	public :: nullDefined
	public :: nullType
	public :: numberOfAttributes
	public :: numberOfBlocks
	public :: numberOfColumns
	public :: numberOfDimensions
	public :: numberOfElements
	public :: numberOfRows
	public :: parent
	public :: qualifiedName
	public :: real32Array2Data
	public :: real32Array3Data
	public :: real32Array4Data
	public :: real32Attribute
	public :: real32Data
	public :: real32VectorData
	public :: real64Array2Data
	public :: real64Array3Data
	public :: real64Array4Data
	public :: real64Attribute
	public :: real64Data
	public :: real64VectorData
	public :: relabel
	public :: release
	public :: rename
	public :: reset
	public :: scale
	public :: seek
	public :: setAttribute
	public :: setAttributes
        public :: setClient
	public :: setData
	public :: setExists
	public :: setNullValue
	public :: setScaling
	public :: setStringCell
	public :: setToNull
	public :: setUnits
	public :: stringAttribute
	public :: stringCell
	public :: subTable
	public :: table
	public :: uint16Array2Data
	public :: uint16Array3Data
	public :: uint16Array4Data
	public :: uint16Data
	public :: uint16VectorData
	public :: uint32Array2Data
	public :: uint32Array3Data
	public :: uint32Array4Data
	public :: uint32Data
	public :: uint32VectorData
        public :: units
	public :: unsetScaling
	public :: zero

! Convert derived type to base type

	interface attributable
		module procedure datasetAttributable
		module procedure blockAttributable
		module procedure tableAttributable
		module procedure arrayAttributable
		module procedure columnAttributable
	end interface

	interface block
		module procedure tableBlock
		module procedure arrayBlock
	end interface

	interface blockType
		module procedure blockType
		module procedure blockTypeOfBlockWithNumber
		module procedure blockTypeOfBlockWithName
	end interface

	interface labelled
		module procedure attributableLabelled
		module procedure datasetLabelled
		module procedure blockLabelled
		module procedure tableLabelled
		module procedure arrayLabelled
		module procedure columnLabelled
		module procedure attributeLabelled
	end interface

	interface dataComponent
		module procedure arrayDataComponent
		module procedure columnDataComponent
	end interface

	interface nullable
		module procedure arrayNullable
		module procedure columnNullable
		module procedure dataComponentNullable
	end interface

	interface dataType
		module procedure columnDataType
		module procedure arrayDataType
		module procedure dataComponentDataType
		module procedure attributeDataType
		module procedure arrayAttributeDataTypeN        ! by name
		module procedure arrayAttributeDataTypeR        ! by number
		module procedure attributableAttributeDataTypeN ! by name
		module procedure attributableAttributeDataTypeR ! by number
		module procedure blockAttributeDataTypeN        ! by name
		module procedure blockAttributeDataTypeR        ! by number
		module procedure columnAttributeDataTypeN       ! by name
		module procedure columnAttributeDataTypeR       ! by number
		module procedure dataSetAttributeDataTypeN      ! by name
		module procedure dataSetAttributeDataTypeR      ! by number
		module procedure tableAttributeDataTypeN        ! by name
		module procedure tableAttributeDataTypeR        ! by number
	end interface

	interface parent
		module procedure parentDataSetOfTable
		module procedure parentDataSetOfArray
		module procedure parentDataSetOfBlock
		module procedure parentTable
		module procedure parentAttributable
	end interface

	interface deleteBlock
		module procedure deleteBlockWithName
		module procedure deleteBlockWithNumber
	end interface

	interface deleteColumn
		module procedure deleteColumnWithNumber
		module procedure deleteColumnWithName
	end interface

	interface name
		module procedure nameOfAttribute
		module procedure nameOfAttributable
		module procedure nameOfDataSet
		module procedure nameOfColumn
		module procedure nameOfTable
		module procedure nameOfArray
		module procedure nameOfBlock
		module procedure nameOfLabelled
	end interface

	interface qualifiedName
		module procedure qualifiedNameOfAttribute
		module procedure qualifiedNameOfAttributable
		module procedure qualifiedNameOfDataSet
		module procedure qualifiedNameOfColumn
		module procedure qualifiedNameOfLabelled
		module procedure qualifiedNameOfTable
		module procedure qualifiedNameOfArray
		module procedure qualifiedNameOfBlock
	end interface

	interface numberOfRows
		module procedure numberOfRowsInTable
		module procedure numberOfRowsInColumn
	end interface

	interface columnNumber
		module procedure columnNumberOfTableColumn
		module procedure columnNumberOfColumn
	end interface

	interface release
		module procedure releaseDataSet
		module procedure releaseTable
		module procedure releaseColumn
		module procedure releaseBlock
		module procedure releaseArray
		module procedure releaseTableSeeker
	end interface

	! Obsolete: use set/addAttributes
	!interface copyAttributes
	!	module procedure datasetCopyAttributes
	!	module procedure tableCopyAttributes
	!	module procedure arrayCopyAttributes
	!	module procedure columnCopyAttributes
	!end interface

	interface boolData
		module procedure boolColumnData
		module procedure boolColumnDataElement
		module procedure boolArrayData
	end interface

	interface int8Data
		module procedure int8ColumnData
		module procedure int8ColumnDataElement
		module procedure int8ArrayData
	end interface

	interface int16Data
		module procedure int16ColumnData
		module procedure int16ColumnDataElement
		module procedure int16ArrayData
	end interface

	interface uint16Data		! Warning: returns signed data
		module procedure uint16ColumnData
		module procedure uint16ColumnDataElement
		module procedure uint16ArrayData
	end interface

	interface uint32Data		! Warning: returns signed data
		module procedure uint32ColumnData
		module procedure uint32ColumnDataElement
		module procedure uint32ArrayData
	end interface

	interface int32Data
		module procedure int32ColumnData
		module procedure int32ColumnDataElement
		module procedure int32ArrayData
	end interface

	interface real32Data
		module procedure real32ColumnData
		module procedure real32ColumnDataElement
		module procedure real32ArrayData
	end interface

	interface real64Data
		module procedure real64ColumnData
		module procedure real64ColumnDataElement
		module procedure real64ArrayData
	end interface

	interface boolVectorData
		module procedure boolColumnVectorData
		module procedure boolColumnVectorDataElement
		module procedure boolArrayVectorData
	end interface

	interface int8VectorData
		module procedure int8ColumnVectorData
		module procedure int8ColumnVectorDataElement
		module procedure int8ArrayVectorData
	end interface

	interface int16VectorData
		module procedure int16ColumnVectorData
		module procedure int16ColumnVectorDataElement
		module procedure int16ArrayVectorData
	end interface

	interface uint16VectorData		! Warning: returns signed data
		module procedure uint16ColumnVectorData
		module procedure uint16ColumnVectorDataElement
		module procedure uint16ArrayVectorData
	end interface

	interface uint32VectorData		! Warning: returns signed data
		module procedure uint32ColumnVectorData
		module procedure uint32ColumnVectorDataElement
		module procedure uint32ArrayVectorData
	end interface

	interface int32VectorData
		module procedure int32ColumnVectorData
		module procedure int32ColumnVectorDataElement
		module procedure int32ArrayVectorData
	end interface

	interface real32VectorData
		module procedure real32ColumnVectorData
		module procedure real32ColumnVectorDataElement
		module procedure real32ArrayVectorData
	end interface

	interface real64VectorData
		module procedure real64ColumnVectorData
		module procedure real64ColumnVectorDataElement
		module procedure real64ArrayVectorData
	end interface

	interface boolArray2Data
		module procedure boolColumnArray2Data
		module procedure boolColumnArray2DataElement
		module procedure boolArrayArray2Data
	end interface

	interface int8Array2Data
		module procedure int8ColumnArray2Data
		module procedure int8ColumnArray2DataElement
		module procedure int8ArrayArray2Data
	end interface

	interface int16Array2Data
		module procedure int16ColumnArray2Data
		module procedure int16ColumnArray2DataElement
		module procedure int16ArrayArray2Data
	end interface

	interface uint16Array2Data		! Warning: returns signed data
		module procedure uint16ColumnArray2Data
		module procedure uint16ColumnArray2DataElement
		module procedure uint16ArrayArray2Data
	end interface

	interface uint32Array2Data		! Warning: returns signed data
		module procedure uint32ColumnArray2Data
		module procedure uint32ColumnArray2DataElement
		module procedure uint32ArrayArray2Data
	end interface

	interface int32Array2Data
		module procedure int32ColumnArray2Data
		module procedure int32ColumnArray2DataElement
		module procedure int32ArrayArray2Data
	end interface

	interface real32Array2Data
		module procedure real32ColumnArray2Data
		module procedure real32ColumnArray2DataElement
		module procedure real32ArrayArray2Data
	end interface

	interface real64Array2Data
		module procedure real64ColumnArray2Data
		module procedure real64ColumnArray2DataElement
		module procedure real64ArrayArray2Data
	end interface

	interface boolArray3Data
		module procedure boolColumnArray3Data
		module procedure boolColumnArray3DataElement
		module procedure boolArrayArray3Data
	end interface

	interface int8Array3Data
		module procedure int8ColumnArray3Data
		module procedure int8ColumnArray3DataElement
		module procedure int8ArrayArray3Data
	end interface

	interface int16Array3Data
		module procedure int16ColumnArray3Data
		module procedure int16ColumnArray3DataElement
		module procedure int16ArrayArray3Data
	end interface

	interface uint16Array3Data		! Warning: returns signed data
		module procedure uint16ColumnArray3Data
		module procedure uint16ColumnArray3DataElement
		module procedure uint16ArrayArray3Data
	end interface

	interface uint32Array3Data		! Warning: returns signed data
		module procedure uint32ColumnArray3Data
		module procedure uint32ColumnArray3DataElement
		module procedure uint32ArrayArray3Data
	end interface

	interface int32Array3Data
		module procedure int32ColumnArray3Data
		module procedure int32ColumnArray3DataElement
		module procedure int32ArrayArray3Data
	end interface

	interface real32Array3Data
		module procedure real32ColumnArray3Data
		module procedure real32ColumnArray3DataElement
		module procedure real32ArrayArray3Data
	end interface

	interface real64Array3Data
		module procedure real64ColumnArray3Data
		module procedure real64ColumnArray3DataElement
		module procedure real64ArrayArray3Data
	end interface

	interface boolArray4Data
		module procedure boolColumnArray4Data
		module procedure boolColumnArray4DataElement
	end interface

	interface int8Array4Data
		module procedure int8ColumnArray4Data
		module procedure int8ColumnArray4DataElement
	end interface

	interface int16Array4Data
		module procedure int16ColumnArray4Data
		module procedure int16ColumnArray4DataElement
	end interface

	interface uint16Array4Data		! Warning: returns signed data
		module procedure uint16ColumnArray4Data
		module procedure uint16ColumnArray4DataElement
	end interface

	interface uint32Array4Data		! Warning: returns signed data
		module procedure uint32ColumnArray4Data
		module procedure uint32ColumnArray4DataElement
	end interface

	interface int32Array4Data
		module procedure int32ColumnArray4Data
		module procedure int32ColumnArray4DataElement
	end interface

	interface real32Array4Data
		module procedure real32ColumnArray4Data
		module procedure real32ColumnArray4DataElement
	end interface

	interface real64Array4Data
		module procedure real64ColumnArray4Data
		module procedure real64ColumnArray4DataElement
	end interface

	interface units
		module procedure arrayUnits
		module procedure columnUnits
		module procedure dataComponentUnits
		module procedure unitsOfAttribute
		module procedure attributableAttributeUnits
		module procedure dataSetAttributeUnits
		module procedure blockAttributeUnits
		module procedure arrayAttributeUnits
		module procedure columnAttributeUnits
		module procedure tableAttributeUnits
	end interface

	interface dimensions
		module procedure dimensionsOfArray
		module procedure dimensionsOfColumn
	end interface

	interface numberOfAttributes
		module procedure numberOfAttributesOfAttrib
		module procedure numberOfAttributesOfDataSet
		module procedure numberOfAttributesOfBlock
		module procedure numberOfAttributesOfArray
		module procedure numberOfAttributesOfTable
		module procedure numberOfAttributesOfColumn
	end interface

	interface numberOfDimensions
		module procedure numberOfDimensionsOfArray
		module procedure numberOfDimensionsOfColumn
	end interface

	interface numberOfElements
		module procedure numberOfElementsOfArray
		module procedure numberOfElementsOfColumn
                module procedure numberOfElementsOfCell
	end interface

	interface column
		module procedure columnWithName
		module procedure columnWithNumber
	end interface

	interface array
		module procedure arrayWithNumber
		module procedure arrayWithName
	end interface

	interface block
		module procedure blockWithNumber
		module procedure blockWithName
	end interface

	interface table
		module procedure tableWithNumber
		module procedure tableWithName
	end interface

	interface setData
		module procedure setInt8Cell
		module procedure setInt16Cell
		module procedure setInt32Cell
		module procedure setReal32Cell
		module procedure setReal64Cell
		module procedure setBoolCell
		module procedure setStringVariableCell
		module procedure setDataxxx   !PV 31-7-2000
	end interface

	interface setScaling
		module procedure setScalingOfColumn
		module procedure setScalingOfArray
	end interface

	interface zero
		module procedure zeroOfColumn
		module procedure zeroOfArray
	end interface

	interface scale
		module procedure scaleOfColumn
		module procedure scaleOfArray
	end interface

	interface hasScaling
		module procedure hasScalingOfColumn
		module procedure hasScalingOfArray
	end interface

	interface reset
		module procedure resetTableSeeker
	end interface

	interface unsetScaling
		module procedure unsetScalingOfColumn
		module procedure unsetScalingOfArray
	end interface

	interface intNullValue
		module procedure intNullValueArray
		module procedure intNullValueColumn
		module procedure intNullValueDataComponent
		module procedure intNullValueNullable
	end interface

	interface setToNull
		module procedure setToNullArray
		module procedure setToNullCell
		module procedure setToNullColumn
	end interface

	interface nullDefined
		module procedure nullDefinedArray
		module procedure nullDefinedColumn
		module procedure nullDefinedDataComponent
		module procedure nullDefinedNullable
	end interface

	interface nullType
		module procedure nullTypeArray
		module procedure nullTypeColumn
		module procedure nullTypeDataComponent
		module procedure nullTypeNullable
	end interface

	interface hasNulls
		module procedure hasNullsArray
		module procedure hasNullsColumn
	end interface

	interface isNotNull
		module procedure isNotNullArray
		module procedure isNotNullColumn
	end interface

	interface isNull
		module procedure isNullArray
		module procedure isNullCell
		module procedure isNullColumn
	end interface

	interface setNullValue
		module procedure setNullValueArray
		module procedure setNullValueColumn
		module procedure setNullValueDataComponent
		module procedure setNullValueNullable
	end interface

	interface addComment
		module procedure addCommentToAttributable
		module procedure addCommentToDataSet
		module procedure addCommentToBlock
		module procedure addCommentToArray
		module procedure addCommentToTable
	end interface

	interface addHistory
		module procedure addHistoryToAttributable
		module procedure addHistoryToDataSet
		module procedure addHistoryToBlock
		module procedure addHistoryToArray
		module procedure addHistoryToTable
	end interface

	interface hasAttribute
		module procedure attributableHasAttribute
		module procedure dataSetHasAttribute
		module procedure blockHasAttribute
		module procedure arrayHasAttribute
		module procedure columnHasAttribute
		module procedure tableHasAttribute
	end interface

	interface int8Attribute
		module procedure int8Attribute
		module procedure int8AttributableAttribute
		module procedure int8DataSetAttribute
		module procedure int8BlockAttribute
		module procedure int8ArrayAttribute
		module procedure int8ColumnAttribute
		module procedure int8TableAttribute
	end interface

	interface int16Attribute
		module procedure int16Attribute
		module procedure int16AttributableAttribute
		module procedure int16DataSetAttribute
		module procedure int16BlockAttribute
		module procedure int16ArrayAttribute
		module procedure int16ColumnAttribute
		module procedure int16TableAttribute
	end interface

	interface int32Attribute
		module procedure int32Attribute
		module procedure int32AttributableAttribute
		module procedure int32DataSetAttribute
		module procedure int32BlockAttribute
		module procedure int32ArrayAttribute
		module procedure int32ColumnAttribute
		module procedure int32TableAttribute
	end interface

	interface real32Attribute
		module procedure real32Attribute
		module procedure real32AttributableAttribute
		module procedure real32DataSetAttribute
		module procedure real32BlockAttribute
		module procedure real32ArrayAttribute
		module procedure real32ColumnAttribute
		module procedure real32TableAttribute
	end interface

	interface real64Attribute
		module procedure real64Attribute
		module procedure real64AttributableAttribute
		module procedure real64DataSetAttribute
		module procedure real64BlockAttribute
		module procedure real64ArrayAttribute
		module procedure real64ColumnAttribute
		module procedure real64TableAttribute
	end interface

	interface stringAttribute
		module procedure stringAttribute
		module procedure stringAttributableAttribute
		module procedure stringDataSetAttribute
		module procedure stringBlockAttribute
		module procedure stringArrayAttribute
		module procedure stringColumnAttribute
		module procedure stringTableAttribute
	end interface

	interface booleanAttribute
		module procedure booleanAttribute
		module procedure booleanAttributableAttribute
		module procedure booleanDataSetAttribute
		module procedure booleanBlockAttribute
		module procedure booleanArrayAttribute
		module procedure booleanColumnAttribute
		module procedure booleanTableAttribute
	end interface

	interface setAttribute
		module procedure attributableSetAttribute
		module procedure datasetSetAttribute
		module procedure blockSetAttribute
		module procedure arraySetAttribute
		module procedure columnSetAttribute
		module procedure tableSetAttribute
		module procedure setInt8Attribute
		module procedure setInt8AttributableAttribute
		module procedure setInt8DataSetAttribute
		module procedure setInt8BlockAttribute
		module procedure setInt8ArrayAttribute
		module procedure setInt8ColumnAttribute
		module procedure setInt8TableAttribute
		module procedure setInt16Attribute
		module procedure setInt16AttributableAttribute
		module procedure setInt16DataSetAttribute
		module procedure setInt16BlockAttribute
		module procedure setInt16ArrayAttribute
		module procedure setInt16ColumnAttribute
		module procedure setInt16TableAttribute
		module procedure setInt32Attribute
		module procedure setInt32AttributableAttribute
		module procedure setInt32DataSetAttribute
		module procedure setInt32BlockAttribute
		module procedure setInt32ArrayAttribute
		module procedure setInt32ColumnAttribute
		module procedure setInt32TableAttribute
		module procedure setReal32Attribute
		module procedure setReal32AttributableAttribute
		module procedure setReal32DataSetAttribute
		module procedure setReal32BlockAttribute
		module procedure setReal32ArrayAttribute
		module procedure setReal32ColumnAttribute
		module procedure setReal32TableAttribute
		module procedure setReal64Attribute
		module procedure setReal64AttributableAttribute
		module procedure setReal64DataSetAttribute
		module procedure setReal64BlockAttribute
		module procedure setReal64ArrayAttribute
		module procedure setReal64ColumnAttribute
		module procedure setReal64TableAttribute
		module procedure setStringAttribute
		module procedure setStringAttributableAttribute
		module procedure setStringDataSetAttribute
		module procedure setStringBlockAttribute
		module procedure setStringArrayAttribute
		module procedure setStringColumnAttribute
		module procedure setStringTableAttribute
		module procedure setBooleanAttribute
		module procedure setBooleanAttributableAttribute
		module procedure setBooleanDataSetAttribute
		module procedure setBooleanBlockAttribute
		module procedure setBooleanArrayAttribute
		module procedure setBooleanColumnAttribute
		module procedure setBooleanTableAttribute
	end interface

	interface attribute
		module procedure attributableAttributeWithName
		module procedure attributableAttributeWithNumber
		module procedure arrayAttributeWithName
		module procedure arrayAttributeWithNumber
		module procedure blockAttributeWithName
		module procedure blockAttributeWithNumber
		module procedure columnAttributeWithName
		module procedure columnAttributeWithNumber
		module procedure dataSetAttributeWithName
		module procedure dataSetAttributeWithNumber
		module procedure tableAttributeWithName
		module procedure tableAttributeWithNumber
	end interface

	interface deleteAttribute
		module procedure deleteAttribute
		module procedure deleteAttribAttributeWithName
		module procedure deleteAttribAttributeWithNumber
		module procedure deleteArrayAttributeWithNumber
		module procedure deleteArrayAttributeWithName
		module procedure deleteBlockAttributeWithName
		module procedure deleteBlockAttributeWithNumber
		module procedure deleteColumnAttributeWithName
		module procedure deleteColumnAttributeWithNumber
		module procedure deleteDataSetAttributeWithName
		module procedure deleteDatSetAttributeWithNumber
		module procedure deleteTableAttributeWithName
		module procedure deleteTableAttributeWithNumber
	end interface

	! obsolete: use units
	!interface attributeUnits
	!	module procedure unitsOfAttribute
	!	module procedure attributableAttributeUnits
	!	module procedure dataSetAttributeUnits
	!	module procedure blockAttributeUnits
	!	module procedure arrayAttributeUnits
	!	module procedure columnAttributeUnits
	!	module procedure tableAttributeUnits
	!end interface

	interface setUnits
		module procedure setArrayUnits
		module procedure setColumnUnits
		module procedure setAttributeUnits
		module procedure setAttributableAttributeUnits
		module procedure setDataSetAttributeUnits
		module procedure setBlockAttributeUnits
		module procedure setArrayAttributeUnits
		module procedure setColumnAttributeUnits
		module procedure setTableAttributeUnits
	end interface

	! obsolete: use label
	!interface attributeComment
	!	module procedure labelOfAttribute
	!	module procedure attributableAttributeComment
	!	module procedure dataSetAttributeComment
	!	module procedure blockAttributeComment
	!	module procedure arrayAttributeComment
	!	module procedure columnAttributeComment
	!	module procedure tableAttributeComment
	!end interface

	interface label
		module procedure labelOfArray
		module procedure labelOfAttributable
		module procedure labelOfAttribute
		module procedure labelOfBlock
		module procedure labelOfColumn
		module procedure labelOfDataSet
		module procedure labelOfTable
		module procedure labelOfLabelled
		module procedure attributableAttributeComment
		module procedure dataSetAttributeComment
		module procedure blockAttributeComment
		module procedure arrayAttributeComment
		module procedure columnAttributeComment
		module procedure tableAttributeComment
	end interface

	interface relabel
		module procedure relabelArray
		module procedure relabelAttribute
		module procedure relabelAttributable
		module procedure relabelBlock
		module procedure relabelColumn
		module procedure relabelDataSet
		module procedure relabelTable
		module procedure relabelLabelled
	end interface

	interface rename
		module procedure renameArray
		module procedure renameAttribute
		module procedure renameAttributable
		module procedure renameBlock
		module procedure renameColumn
		module procedure renameDataSet
		module procedure renameTable
		module procedure renameLabelled
	end interface

	interface next
		module procedure subTableNext
		module procedure tableSeekerNext
	end interface

	interface count
		module procedure countTable
		module procedure countColumn
		module procedure countTableSeeker
	end interface

	interface from
		module procedure fromTable
		module procedure fromColumn
		module procedure fromTableSeeker
	end interface

	interface seek
		module procedure seekTable
		module procedure seekColumn
	end interface

	! ========
	! Privates
	! ========
	character(len=*), parameter :: errorMessage  = "Invalid Dal library"
	integer, parameter :: identifierLength = 32
	integer, parameter :: nameLength = 256
	integer, parameter :: qualifiedNameLength = 256
	integer, parameter :: stringAttributeLength = 128
	integer, parameter :: columnStringCellLength = 128


contains

	function clobber()

		logical :: clobber

 		call error( "", errorMessage )

	end function clobber

! Type conversions: convert a subclass of Attributable to Attributable

	function datasetAttributable( set )
		type(DataSetT), intent(in) :: set
		type(AttributableT) :: datasetAttributable

 		call error( "", errorMessage )

	end function

	function blockAttributable( block )
		type(BlockT), intent(in) :: block
		type(AttributableT) :: blockAttributable

 		call error( "", errorMessage )

	end function

	function arrayAttributable( arr )
		type(ArrayT), intent(in) :: arr
		type(AttributableT) :: arrayAttributable

 		call error( "", errorMessage )

	end function

	function tableAttributable( tab )
		type(TableT), intent(in) :: tab
		type(AttributableT) :: tableAttributable

 		call error( "", errorMessage )

	end function

	function columnAttributable( col )
		type(ColumnT), intent(in) :: col
		type(AttributableT) :: columnAttributable

 		call error( "", errorMessage )

	end function

! Type conversions: convert a subclass of a Block to a Block

	function arrayBlock( array )
		type(ArrayT), intent(in) :: array
		type(BlockT) :: arrayBlock

 		call error( "", errorMessage )

	end function

	function tableBlock( table )
		type(TableT), intent(in) :: table
		type(BlockT) :: tableBlock

 		call error( "", errorMessage )

	end function


! Type conversions: convert a subclass of Labelled to Labelled

	function attributableLabelled( obj ) result(labelled)
		type(AttributableT), intent(in) :: obj
		type(LabelledT) :: labelled

 		call error( "", errorMessage )

	end function attributableLabelled

	function datasetLabelled( obj ) result(labelled)
		type(DataSetT), intent(in) :: obj
		type(LabelledT) :: labelled

 		call error( "", errorMessage )

	end function datasetLabelled

	function blockLabelled( obj ) result(labelled)
		type(BlockT), intent(in) :: obj
		type(LabelledT) :: labelled

 		call error( "", errorMessage )

	end function blockLabelled

	function tableLabelled( obj ) result( labelled )
		type(TableT), intent(in) :: obj
		type(LabelledT) :: labelled

 		call error( "", errorMessage )

	end function tableLabelled

	function arrayLabelled( obj ) result( labelled )
		type(ArrayT), intent(in) :: obj
		type(LabelledT) :: labelled

 		call error( "", errorMessage )

	end function arrayLabelled

	function columnLabelled( obj ) result( labelled )
		type(ColumnT), intent(in) :: obj
		type(LabelledT) :: labelled

 		call error( "", errorMessage )

	end function columnLabelled

	function attributeLabelled( obj ) result( labelled )
		type(AttributeT), intent(in) :: obj
		type(LabelledT) :: labelled

 		call error( "", errorMessage )

	end function attributeLabelled

! Parent functions: return the parent object of the given object

	function parentDataSetOfTable( table )
		type(TableT), intent(in) :: table
		type(DataSetT) :: parentDataSetOfTable

 		call error( "", errorMessage )

	end function parentDataSetOfTable

	function parentDataSetOfArray( array )
		type(ArrayT), intent(in) :: array
		type(DataSetT) :: parentDataSetOfArray

 		call error( "", errorMessage )

	end function parentDataSetOfArray

	function parentDataSetOfBlock( block )
		type(BlockT), intent(in) :: block
		type(DataSetT) :: parentDataSetOfBlock

 		call error( "", errorMessage )

	end function parentDataSetOfBlock

	function parentTable( column )
		type(ColumnT), intent(in) :: column
		type(TableT) :: parentTable

 		call error( "", errorMessage )

	end function parentTable

	function parentAttributable( attribute )
		type(AttributeT), intent(in) :: attribute
		type(AttributableT) :: parentAttributable

 		call error( "", errorMessage )

	end function parentAttributable

! Type concersions: convert a subclass of Nullable into Nullable

	function arrayNullable( array )
		type(ArrayT), intent(in) :: array

		type(NullableT) :: arrayNullable
		
 		call error( "", errorMessage )

	end function arrayNullable

	function columnNullable( column )
		type(ColumnT), intent(in) :: column

		type(NullableT) :: columnNullable
		
 		call error( "", errorMessage )

	end function columnNullable

	function dataComponentNullable( dataComponent )
		type(DataComponentT), intent(in) :: DataComponent

		type(NullableT) :: dataComponentNullable
		
 		call error( "", errorMessage )

	end function DataComponentNullable

! Type conversions: convert a subclass of DataComponent into DataComponent

	function columnDatacomponent( obj ) result( comp )
		type(ColumnT), intent(in) :: obj
		type(DataComponentT) :: comp

 		call error( "", errorMessage )

	end function columnDatacomponent

	function arrayDatacomponent( obj ) result( comp )
		type(ArrayT), intent(in) :: obj
		type(DatacomponentT) :: comp

 		call error( "", errorMessage )

	end function arrayDatacomponent

! Data sets

	function dataSet( dataSetName, mode, memoryModel )
		character(len=*), intent(in) :: dataSetName
		integer, intent(in) :: mode                  ! READ|CREATE|MODIFY|TEMP
		integer, intent(in), optional :: memoryModel !HIGH|HIGHLOW|LOW|USEENVIRONMENT

		type(DataSetT) :: dataSet

		call error( "", errorMessage )

	end function dataSet

	function clone( from, to, mode, memoryModel )
		character(len=*), intent(in) :: from
		character(len=*), intent(in) :: to
		integer, intent(in) :: mode                  ! MODIFY|TEMP
		integer, intent(in), optional :: memoryModel !HIGH|HIGHLOW|LOW|USEENVIRONMENT

		type(DataSetT) :: clone

		call error( "", errorMessage )

	end function clone

	subroutine deleteDataSet( dataSetName )
		character(len=*), intent(in) :: dataSetName

		call error( "", errorMessage )

	end subroutine deleteDataSet

	subroutine releaseDataSet( dataSet )
		type(DataSetT), intent(in) :: dataSet

		call error( "", errorMessage )

	end subroutine releaseDataSet

	subroutine cleanupClient( clientName )
		character(len=*), intent(in) :: clientName

		call error( "", errorMessage )

	end subroutine cleanupClient

	subroutine discardDataSet( dataSetName )
		character(len=*), intent(in) :: dataSetName

		call error( "", errorMessage )

	end subroutine discardDataSet

	subroutine keepDataSet( dataSetName )
		character(len=*), intent(in) :: dataSetName

		call error( "", errorMessage )

	end subroutine keepDataSet

	function setExists( setName )
		character(len=*), intent(in) :: setName

		logical setExists

		call error( "", errorMessage )

	end function setExists

	subroutine copyDataSet( sourceName, destName )
		character(len=*), intent(in) :: destName, sourceName

		call error( "", errorMessage )
	end subroutine copyDataSet

	subroutine releaseTable( table )
		type(TableT), intent(in) :: table

		call error( "", errorMessage )

	end subroutine releaseTable

	subroutine releaseColumn( column )
		type(ColumnT), intent(in) :: column

		call error( "", errorMessage )

	end subroutine releaseColumn

	subroutine releaseBlock( block )
		type(BlockT), intent(in) :: block

		call error( "", errorMessage )

	end subroutine releaseBlock

	subroutine releaseArray( array )
		type(ArrayT), intent(in) :: array

		call error( "", errorMessage )

	end subroutine releaseArray

! Names

	function nameOfAttributable( attributable )
		type(AttributableT) :: attributable

		character(len=identifierLength) :: nameOfAttributable

		call error( "", errorMessage )

	end function nameOfAttributable

	function nameOfDataSet( dataSet )
		type(DataSetT) :: dataSet

		character(len=nameLength) :: nameOfDataSet

		call error( "", errorMessage )

	end function nameOfDataSet

	function nameOfLabelled( labelled )
		type(LabelledT) :: labelled

		character(len=nameLength) :: nameOfLabelled

		call error( "", errorMessage )

	end function nameOfLabelled

	function nameOfColumn( column )
		type(ColumnT) :: column

		character(len=nameLength) :: nameOfColumn

		call error( "", errorMessage )

	end function nameOfColumn

	function nameOfTable( table )
		type(TableT) :: table

		character(len=nameLength) :: nameOfTable

		call error( "", errorMessage )

	end function nameOfTable

	function nameOfArray( array )
		type(ArrayT) :: array

		character(len=nameLength) :: nameOfArray

		call error( "", errorMessage )

	end function nameOfArray

	function nameOfAttribute( attribute )
		type(AttributeT) :: attribute

		character(len=nameLength) :: nameOfAttribute

		call error( "", errorMessage )

	end function nameOfAttribute

	function nameOfBlock( block )
		type(BlockT) :: block

		character(len=nameLength) :: nameOfBlock

		call error( "", errorMessage )

	end function nameOfBlock

! Qualified Names

	function qualifiedNameOfAttributable( attributable )
		type(AttributableT) :: attributable

		character(len=qualifiedNameLength) :: qualifiedNameOfAttributable

		call error( "", errorMessage )

	end function qualifiedNameOfAttributable

	function qualifiedNameOfDataSet( dataSet )
		type(DataSetT) :: dataSet

		character(len=qualifiedNameLength) :: qualifiedNameOfDataSet

		call error( "", errorMessage )

	end function qualifiedNameOfDataSet

	function qualifiedNameOfColumn( column )
		type(ColumnT) :: column

		character(len=qualifiedNameLength) :: qualifiedNameOfColumn

		call error( "", errorMessage )

	end function qualifiedNameOfColumn

	function qualifiedNameOfLabelled( labelled )
		type(LabelledT) :: labelled

		character(len=qualifiedNameLength) :: qualifiedNameOfLabelled

		call error( "", errorMessage )

	end function qualifiedNameOfLabelled

	function qualifiedNameOfTable( table )
		type(TableT) :: table

		character(len=qualifiedNameLength) :: qualifiedNameOfTable

		call error( "", errorMessage )

	end function qualifiedNameOfTable

	function qualifiedNameOfArray( array )
		type(ArrayT) :: array

		character(len=qualifiedNameLength) :: qualifiedNameOfArray

		call error( "", errorMessage )

	end function qualifiedNameOfArray

	function qualifiedNameOfAttribute( attribute )
		type(AttributeT) :: attribute

		character(len=qualifiedNameLength) :: qualifiedNameOfAttribute

		call error( "", errorMessage )

	end function qualifiedNameOfAttribute

	function qualifiedNameOfBlock( block )
		type(BlockT) :: block

		character(len=qualifiedNameLength) :: qualifiedNameOfBlock

		call error( "", errorMessage )

	end function qualifiedNameOfBlock

	function mode( dataSet )
		type(DataSetT) :: dataSet

		integer :: mode		! READ|CREATE|MODIFY|TEMP

		call error( "", errorMessage )

	end function mode


	function hasBlock( set, name )
		type(DataSetT), intent(in) :: set
		character(len=*), intent(in) :: name

		logical hasBlock

		call error( "", errorMessage )

	end function hasBlock


	subroutine copyBlock( set, block, newName )
		type(DataSetT), intent(in) :: set
		type(BlockT), intent(in) :: block
		character(len=*), intent(in), optional :: newName
		type(BlockT) :: addBlock

		call error( "", errorMessage )

	end subroutine copyBlock

 	function addTable( dataSet, name, numberOfRows, comment, position )
 		type(DataSetT), intent(in) :: dataSet
 		character(len=*), intent(in) :: name
 		integer, intent(in) :: numberOfRows
 		character(len=*), intent(in), optional :: comment
 		integer, intent(in), optional :: position
 
 		type(TableT) :: addTable
 
 		call error( "", errorMessage )
 
 	end function addTable
 
	function addArray( dataSet, name, type,	dimensions, units, comment, position )
	 	type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name
		integer, intent(in) :: type
		integer(KIND=INT32), dimension(:) :: dimensions
		character(len=*), intent(in), optional :: units
		character(len=*), intent(in), optional :: comment
	 	integer, intent(in), optional :: position
 
 		type( ArrayT ) :: addArray
 
 		call error( "", errorMessage )
 
 	end function addArray

	subroutine deleteBlockWithNumber( dataSet, blockNumber )
		type(DataSetT), intent(in) :: dataSet
		integer, intent(in) :: blockNumber

		call error( "", errorMessage )

	end subroutine deleteBlockWithNumber

	subroutine deleteBlockWithName( dataSet, blockName )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: blockName

		call error( "", errorMessage )

	end subroutine deleteBlockWithName

	function numberOfBlocks( dataSet )
		type(DataSetT), intent(in) :: dataSet

		integer :: numberOfBlocks

		call error( "", errorMessage )

	end function numberOfBlocks

	function blockType( block )
		type(BlockT), intent(in) :: block

		integer :: blockType	! ARRAY_BLOCK|TABLE_BLOCK

		call error( "", errorMessage )

	end function blockType

	function blockTypeOfBlockWithNumber( dataSet, blockNumber )
		type(DataSetT), intent(in) :: dataSet
		integer, intent(in) :: blockNumber

		integer :: blockTypeOfBlockWithNumber	! ARRAY_BLOCK|TABLE_BLOCK

		call error( "", errorMessage )

	end function blockTypeOfBlockWithNumber

	function blockTypeOfBlockWithName( dataSet, blockName )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: blockName

		integer :: blockTypeOfBlockWithName	! ARRAY_BLOCK|TABLE_BLOCK

		call error( "", errorMessage )

	end function blockTypeOfBlockWithName

	function blockNumber( dataSet, blockName )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: blockName

		integer :: blockNumber

		call error( "", errorMessage )

	end function blockNumber


! Tables

	function tableWithNumber( dataSet, tableNumber, mode )
		type(DataSetT), intent(in) :: dataSet
		integer, intent(in) :: tableNumber
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(TableT) :: tableWithNumber

		call error( "", errorMessage )

	end function tableWithNumber

	function tableWithName( dataSet, tableName, mode )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: tableName
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(TableT) :: tableWithName

		call error( "", errorMessage )

	end function tableWithName

	function numberOfRowsInTable( table )
		type( TableT ), intent(in) :: table

		integer :: numberOfRowsInTable

		call error( "", errorMessage )

	end function numberOfRowsInTable

	function hasColumn( tab, name )
		type(TableT), intent(in) :: tab
		character(len=*), intent(in) :: name

		logical :: hasColumn

		call error( "", errorMessage )

	end function hasColumn


	function addColumn( table, columnName, dataKind, units, dimensions, comment, position )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: columnName
		integer, intent(in) :: dataKind
		character(len=*), intent(in), optional :: units
		character(len=*), intent(in), optional :: comment
		integer, dimension(:), optional :: dimensions
		integer, intent(in), optional :: position

		type(ColumnT)	addColumn

		call error( "", errorMessage )

	end function addColumn


	subroutine copyColumn( table, column, newName )
		type(TableT), intent(in) :: table
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in), optional :: newName
		type(ColumnT) :: addColumnFromColumn

		call error( "", errorMessage )

	end subroutine copyColumn


	subroutine deleteColumnWithNumber( table, columnNumber )
		type( TableT ), intent(in) :: table
		integer, intent(in) :: columnNumber

		call error( "", errorMessage )

	end subroutine deleteColumnWithNumber

	subroutine deleteColumnWithName( table, columnName )
		type( TableT ), intent(in) :: table
		character(len=*), intent(in) :: columnName

		call error( "", errorMessage )

	end subroutine deleteColumnWithName

	function columnNumberOfTableColumn( table, columnName )
		type(TableT), intent(in) :: table
 		character(len=*), intent(in) :: columnName

		integer :: columnNumberOfTableColumn
 
 		call error( "", errorMessage )
 
	end function columnNumberOfTableColumn

	function columnNumberOfColumn( column )
		type(ColumnT), intent(in) :: column

		integer :: columnNumberOfColumn
 
 		call error( "", errorMessage )
 
	end function columnNumberOfColumn

	function numberOfColumns( table )
		type( TableT ), intent( in ) :: table

		integer :: numberOfColumns

		call error( "", errorMessage )

	end function numberOfColumns

	function columnWithNumber( table, columnNumber, mode )
		type(TableT), intent(in) :: table
		integer, intent(in) :: columnNumber
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(ColumnT) :: columnWithNumber

		call error( "", errorMessage )

	end function columnWithNumber

	function columnWithName( table, columnName, mode )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: columnName
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(ColumnT) :: columnWithName

		call error( "", errorMessage )

	end function columnWithName

	subroutine copyRows( table, from, to, count )
		type(TableT), intent(in) :: table
		integer, intent(in) :: from, to
		integer, intent(in), optional :: count

		call error( "", errorMessage )

	end subroutine

	subroutine insertRows( table, position, count )
		type(TableT), intent(in) :: table
		integer, intent(in) :: position
		integer, intent(in), optional :: count

		call error( "", errorMessage )

	end subroutine

	subroutine deleteRows( table, from, count )
		type(TableT), intent(in) :: table
		integer, intent(in) :: from
		integer, intent(in), optional :: count

		call error( "", errorMessage )

	end subroutine

! DataComponent

	function dataComponentDataType( dataComponent )
		type(DataComponentT), intent(in) :: dataComponent

		integer :: dataComponentDataType

		call error( "", errorMessage )
	
	end function dataComponentDataType

	function dataComponentUnits( dataComponent )
		type(DataComponentT), intent(in) :: dataComponent

		character(len=IdentifierLength) :: dataComponentUnits

		call error( "", errorMessage )

	end function dataComponentUnits

! Columns

	function columnDataType( column )
		type(ColumnT), intent(in) :: column

		integer :: columnDataType

		call error( "", errorMessage )
	
	end function columnDataType

	function numberOfRowsInColumn( column )
		type( ColumnT ), intent(in) :: column

		integer :: numberOfRowsInColumn

		call error( "", errorMessage )

	end function numberOfRowsInColumn

	function columnUnits( column )
		type(ColumnT), intent(in) :: column

		character(len=IdentifierLength) :: columnUnits

		call error( "", errorMessage )
	
	end function columnUnits

! STRING COLUMNS : FIXED AND VARIABLE LENGTH

	subroutine setStringCell( column, row, value )
		type(ColumnT), intent(in) :: column
		integer(kind=INT32), intent(in) :: row
		character(len=*) :: value

		call error( "", errorMessage )

	end subroutine setStringCell

	function stringCell( column, row )
		type(ColumnT), intent(in) :: column
		integer(kind=INT32), intent(in) :: row

		character(len=columnStringCellLength) :: stringCell

		call error( "", errorMessage )

	end function stringCell


! SETTING VARIABLE LENGTH COLUMNS

	subroutine setInt8Cell( column, row, value )
		type(ColumnT), intent(in) :: column
		integer(kind=INT32), intent(in) :: row
		integer(kind=INT8), dimension(:), intent(in) :: value

		call error( "", errorMessage )

	end subroutine setInt8Cell

	subroutine setInt16Cell( column, row, value )
		type(ColumnT), intent(in) :: column
		integer(kind=INT32), intent(in) :: row
		integer(kind=INT16), dimension(:), intent(in) :: value

		call error( "", errorMessage )

	end subroutine setInt16Cell

	subroutine setInt32Cell( column, row, value )
		type(ColumnT), intent(in) :: column
		integer(kind=INT32), intent(in) :: row
		integer(kind=INT32), dimension(:), intent(in) :: value

		call error( "", errorMessage )

	end subroutine setInt32Cell

	subroutine setReal32Cell( column, row, value )
		type(ColumnT), intent(in) :: column
		integer(kind=INT32), intent(in) :: row
		real(kind=SINGLE), dimension(:), intent(in) :: value

		call error( "", errorMessage )

	end subroutine setReal32Cell

	subroutine setReal64Cell( column, row, value )
		type(ColumnT), intent(in) :: column
		integer(kind=INT32), intent(in) :: row
		real(kind=DOUBLE), dimension(:), intent(in) :: value

		call error( "", errorMessage )

	end subroutine setReal64Cell

	subroutine setBoolCell( column, row, value )
		type(ColumnT), intent(in) :: column
		integer(kind=INT32), intent(in) :: row
		logical(kind=BOOL), dimension(:), intent(in) :: value

		call error( "", errorMessage )

	end subroutine setBoolCell

	subroutine setStringVariableCell( column, row, value )
		type(ColumnT), intent(in) :: column
		integer(kind=INT32), intent(in) :: row
		character(len=*) :: value

		call error( "", errorMessage )

	end subroutine setStringVariableCell


! SCALAR COLUMNS - ACCESS ELEMENT

	function boolColumnDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		logical(kind=bool), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function boolColumnDataElement

	function int8ColumnDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT8), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int8ColumnDataElement

	function int16ColumnDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT16), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int16ColumnDataElement

	function uint16ColumnDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT16), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint16ColumnDataElement

	function int32ColumnDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT32), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int32ColumnDataElement

	function uint32ColumnDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT32), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint32ColumnDataElement

	function real32ColumnDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		real(kind=SINGLE), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real32ColumnDataElement

	function real64ColumnDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		real(kind=DOUBLE), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real64ColumnDataElement

! SCALAR COLUMNS - ACCESS ALL DATA

	function boolColumnData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		logical(kind=bool), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function boolColumnData

	function int8ColumnData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT8), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int8ColumnData

	function int16ColumnData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT16), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int16ColumnData

	function uint16ColumnData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT16), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint16ColumnData

	function int32ColumnData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT32), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int32ColumnData

	function uint32ColumnData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT32), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint32ColumnData

	function real32ColumnData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		real(kind=SINGLE), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real32ColumnData

	function real64ColumnData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		real(kind=DOUBLE), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real64ColumnData

! FIXED VECTOR COLUMNS - ACCESS ELEMENT

	function boolColumnVectorDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer(kind=INT32), intent(in) :: row

		logical(kind=bool), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function boolColumnVectorDataElement

	function int8ColumnVectorDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer(kind=INT32), intent(in) :: row

		integer(kind=INT8), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int8ColumnVectorDataElement

	function int16ColumnVectorDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT16), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int16ColumnVectorDataElement

	function uint16ColumnVectorDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT16), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint16ColumnVectorDataElement

	function int32ColumnVectorDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT32), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int32ColumnVectorDataElement

	function uint32ColumnVectorDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT32), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint32ColumnVectorDataElement

	function real32ColumnVectorDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		real(kind=SINGLE), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real32ColumnVectorDataElement

	function real64ColumnVectorDataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		real(kind=DOUBLE), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real64ColumnVectorDataElement

! FIXED VECTOR COLUMNS - ACCESS ALL DATA

	function boolColumnVectorData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		logical(kind=bool), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function boolColumnVectorData

	function int8ColumnVectorData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT8), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int8ColumnVectorData

	function int16ColumnVectorData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT16), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int16ColumnVectorData

	function uint16ColumnVectorData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT16), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint16ColumnVectorData

	function int32ColumnVectorData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT32), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int32ColumnVectorData

	function uint32ColumnVectorData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT32), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint32ColumnVectorData

	function real32ColumnVectorData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		real(kind=SINGLE), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real32ColumnVectorData

	function real64ColumnVectorData( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		real(kind=DOUBLE), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real64ColumnVectorData

! FIXED 2D ARRAYS COLUMNS - ACCESS ELEMENT

	function boolColumnArray2DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		logical(kind=bool), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function boolColumnArray2DataElement

	function int8ColumnArray2DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT8), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int8ColumnArray2DataElement

	function int16ColumnArray2DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT16), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int16ColumnArray2DataElement

	function uint16ColumnArray2DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT16), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint16ColumnArray2DataElement

	function int32ColumnArray2DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT32), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int32ColumnArray2DataElement

	function uint32ColumnArray2DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT32), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint32ColumnArray2DataElement

	function real32ColumnArray2DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		real(kind=SINGLE), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real32ColumnArray2DataElement

	function real64ColumnArray2DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		real(kind=DOUBLE), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real64ColumnArray2DataElement

! FIXED 2D ARRAYS COLUMNS - ACCESS ALL DATA

	function boolColumnArray2Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		logical(kind=bool), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function boolColumnArray2Data

	function int8ColumnArray2Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT8), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int8ColumnArray2Data

	function int16ColumnArray2Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT16), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int16ColumnArray2Data

	function uint16ColumnArray2Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT16), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint16ColumnArray2Data

	function int32ColumnArray2Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT32), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int32ColumnArray2Data

	function uint32ColumnArray2Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT32), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint32ColumnArray2Data

	function real32ColumnArray2Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		real(kind=SINGLE), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real32ColumnArray2Data

	function real64ColumnArray2Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		real(kind=DOUBLE), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real64ColumnArray2Data

! FIXED 3D ARRAYS COLUMNS - ACCESS ELEMENT

	function boolColumnArray3DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		logical(kind=bool), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function boolColumnArray3DataElement

	function int8ColumnArray3DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT8), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int8ColumnArray3DataElement

	function int16ColumnArray3DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT16), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int16ColumnArray3DataElement

	function uint16ColumnArray3DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT16), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint16ColumnArray3DataElement

	function int32ColumnArray3DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT32), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int32ColumnArray3DataElement

	function uint32ColumnArray3DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT32), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint32ColumnArray3DataElement

	function real32ColumnArray3DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		real(kind=SINGLE), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real32ColumnArray3DataElement

	function real64ColumnArray3DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		real(kind=DOUBLE), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real64ColumnArray3DataElement

! FIXED 3D ARRAYS COLUMNS - ACCESS ALL DATA

	function boolColumnArray3Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		logical(kind=bool), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function boolColumnArray3Data

	function int8ColumnArray3Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT8), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int8ColumnArray3Data

	function int16ColumnArray3Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT16), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int16ColumnArray3Data

	function uint16ColumnArray3Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT16), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint16ColumnArray3Data

	function int32ColumnArray3Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT32), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int32ColumnArray3Data

	function uint32ColumnArray3Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT32), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint32ColumnArray3Data

	function real32ColumnArray3Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		real(kind=SINGLE), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real32ColumnArray3Data

	function real64ColumnArray3Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		real(kind=DOUBLE), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real64ColumnArray3Data

! FIXED 4D ARRAYS COLUMNS - ACCESS ELEMENT

	function boolColumnArray4DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		logical(kind=bool), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function boolColumnArray4DataElement

	function int8ColumnArray4DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT8), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int8ColumnArray4DataElement

	function int16ColumnArray4DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT16), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int16ColumnArray4DataElement

	function uint16ColumnArray4DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT16), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint16ColumnArray4DataElement

	function int32ColumnArray4DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT32), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int32ColumnArray4DataElement

	function uint32ColumnArray4DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		integer(kind=INT32), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint32ColumnArray4DataElement

	function real32ColumnArray4DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		real(kind=SINGLE), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real32ColumnArray4DataElement

	function real64ColumnArray4DataElement( column, row ) result( ptr )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: row

		real(kind=DOUBLE), dimension(:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real64ColumnArray4DataElement

! FIXED 4D ARRAYS COLUMNS - ACCESS ALL DATA

	function boolColumnArray4Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		logical(kind=bool), dimension(:,:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function boolColumnArray4Data

	function int8ColumnArray4Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT8), dimension(:,:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int8ColumnArray4Data

	function int16ColumnArray4Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT16), dimension(:,:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int16ColumnArray4Data

	function uint16ColumnArray4Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT16), dimension(:,:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint16ColumnArray4Data

	function int32ColumnArray4Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT32), dimension(:,:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int32ColumnArray4Data

	function uint32ColumnArray4Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		integer(kind=INT32), dimension(:,:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint32ColumnArray4Data

	function real32ColumnArray4Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		real(kind=SINGLE), dimension(:,:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real32ColumnArray4Data

	function real64ColumnArray4Data( column ) result( ptr )
		type(ColumnT), intent(in) :: column

		real(kind=DOUBLE), dimension(:,:,:,:,:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real64ColumnArray4Data

! VARIABLE VECTOR COLUMNS

	! TO BE SPECIFIED

! VARIABLE 2D ARRAY COLUMNS

	! TO BE SPECIFIED

! VARIABLE 3D ARRAY COLUMNS

	! TO BE SPECIFIED

! VARIABLE 4D ARRAY COLUMNS

	! TO BE SPECIFIED

! Columns

	function dimensionsOfColumn( column )
		type( ColumnT ), intent(in) :: column

		integer, dimension(:), pointer :: dimensionsOfColumn
 
 		call error( "", errorMessage )

	end function dimensionsOfColumn

	function numberOfDimensionsOfColumn( column )
		type( ColumnT ), intent(in) :: column

		integer :: numberOfDimensionsOfColumn
 
 		call error( "", errorMessage )

	end function numberOfDimensionsOfColumn

	function numberOfElementsOfColumn( column )
		type( ColumnT ), intent(in) :: column
		integer :: numberOfElementsOfColumn

 		call error( "", errorMessage )

	end function numberOfElementsOfColumn

	function numberOfElementsOfCell( column , row)
		type( ColumnT ), intent(in) :: column
                integer, intent(in) :: row
		integer :: numberOfElementsOfCell

 		call error( "", errorMessage )

        end function numberOfElementsOfCell

! Arrays

	function arrayUnits( array )
		type(ArrayT), intent(in) :: array

		character(len=IdentifierLength) :: arrayUnits

		call error( "", errorMessage )
	
	end function arrayUnits

	function boolArrayData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		logical(kind=bool), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function boolArrayData

	function int8ArrayData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		integer(kind=INT8), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int8ArrayData

	function int16ArrayData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		integer(kind=INT16), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int16ArrayData

	function uint16ArrayData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		integer(kind=INT16), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint16ArrayData

	function int32ArrayData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		integer(kind=INT32), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int32ArrayData

	function uint32ArrayData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		integer(kind=INT32), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint32ArrayData

	function real32ArrayData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		real(kind=SINGLE), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real32ArrayData

	function real64ArrayData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		real(kind=DOUBLE), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real64ArrayData

	function boolArrayVectorData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		logical(kind=bool), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function boolArrayVectorData

	function int8ArrayVectorData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		integer(kind=INT8), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int8ArrayVectorData

	function int16ArrayVectorData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		integer(kind=INT16), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int16ArrayVectorData

	function uint16ArrayVectorData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		integer(kind=INT16), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint16ArrayVectorData

	function int32ArrayVectorData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		integer(kind=INT32), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function int32ArrayVectorData

	function uint32ArrayVectorData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		integer(kind=INT32), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function uint32ArrayVectorData

	function real32ArrayVectorData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		real(kind=SINGLE), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real32ArrayVectorData

	function real64ArrayVectorData( array ) result( ptr )
		type(ArrayT), intent(in) :: array

		real(kind=DOUBLE), dimension(:), pointer :: ptr

		call error( "", errorMessage )
	
	end function real64ArrayVectorData

	function boolArrayArray2Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		logical(kind=bool), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )

	end function boolArrayArray2Data

	function int8ArrayArray2Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		integer(kind=INT8), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )

	end function int8ArrayArray2Data

	function int16ArrayArray2Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		integer(kind=INT16), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )

	end function int16ArrayArray2Data

	function uint16ArrayArray2Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		integer(kind=INT16), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )

	end function uint16ArrayArray2Data

	function int32ArrayArray2Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		integer(kind=INT32), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )

	end function int32ArrayArray2Data

	function uint32ArrayArray2Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		integer(kind=INT32), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )

	end function uint32ArrayArray2Data

	function real32ArrayArray2Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		real(kind=single), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )

	end function real32ArrayArray2Data

	function real64ArrayArray2Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		real(kind=double), dimension(:,:), pointer :: ptr

		call error( "", errorMessage )

	end function real64ArrayArray2Data

	function boolArrayArray3Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		logical(kind=bool), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )

	end function boolArrayArray3Data

	function int8ArrayArray3Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		integer(kind=INT8), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )

	end function int8ArrayArray3Data

	function int16ArrayArray3Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		integer(kind=INT16), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )

	end function int16ArrayArray3Data

	function uint16ArrayArray3Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		integer(kind=INT16), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )

	end function uint16ArrayArray3Data

	function int32ArrayArray3Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		integer(kind=INT32), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )

	end function int32ArrayArray3Data

	function uint32ArrayArray3Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		integer(kind=INT32), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )

	end function uint32ArrayArray3Data

	function real32ArrayArray3Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		real(kind=single), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )

	end function real32ArrayArray3Data

	function real64ArrayArray3Data( array ) result(ptr)
		type(ArrayT), intent(in) :: array

		real(kind=double), dimension(:,:,:), pointer :: ptr

		call error( "", errorMessage )

	end function real64ArrayArray3Data

	function dimensionsOfArray( array )
		type( ArrayT ), intent(in) :: array

		integer, dimension(:), pointer :: dimensionsOfArray
 
 		call error( "", errorMessage )

	end function dimensionsOfArray

	function numberOfDimensionsOfArray( array )
		type( ArrayT ), intent(in) :: array

		integer :: numberOfDimensionsOfArray
 
 		call error( "", errorMessage )

	end function numberOfDimensionsOfArray

	function numberOfElementsOfArray( array )
		type( ArrayT ), intent(in) :: array

		integer :: numberOfElementsOfArray

 		call error( "", errorMessage )

	end function numberOfElementsOfArray

! Attributes

	function attributeDataType( attribute )
		type(AttributeT), intent(in) :: attribute

		integer :: attributeDataType

		call error( "", errorMessage )
	
	end function attributeDataType

	function arrayAttributeDataTypeN( array, name )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name

		integer :: arrayAttributeDataTypeN

		call error( "", errorMessage )
	
	end function arrayAttributeDataTypeN

	function arrayAttributeDataTypeR( array, number )
		type(ArrayT), intent(in) :: array
		integer, intent(in) :: number

		integer :: arrayAttributeDataTypeR

		call error( "", errorMessage )
	
	end function arrayAttributeDataTypeR

	function attributableAttributeDataTypeN( attributable, name )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name

		integer ::  attributableAttributeDataTypeN

		call error( "", errorMessage )
	
	end function  attributableAttributeDataTypeN

	function attributableAttributeDataTypeR( attributable, number )
		type(AttributableT), intent(in) :: attributable
		integer, intent(in) :: number

		integer ::  attributableAttributeDataTypeR

		call error( "", errorMessage )
	
	end function  attributableAttributeDataTypeR

	function blockAttributeDataTypeN( block, name )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name

		integer :: blockAttributeDataTypeN

		call error( "", errorMessage )
	
	end function blockAttributeDataTypeN

	function blockAttributeDataTypeR( block, number )
		type(BlockT), intent(in) :: block
		integer, intent(in) :: number

		integer :: blockAttributeDataTypeR

		call error( "", errorMessage )
	
	end function blockAttributeDataTypeR

	function columnAttributeDataTypeN( column, name )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name

		integer :: columnAttributeDataTypeN

		call error( "", errorMessage )
	
	end function columnAttributeDataTypeN

	function columnAttributeDataTypeR( column, number )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: number

		integer :: columnAttributeDataTypeR

		call error( "", errorMessage )
	
	end function columnAttributeDataTypeR

	function dataSetAttributeDataTypeN( dataSet, name )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name

		integer :: dataSetAttributeDataTypeN

		call error( "", errorMessage )
	
	end function dataSetAttributeDataTypeN

	function dataSetAttributeDataTypeR( dataSet, number )
		type(DataSetT), intent(in) :: dataSet
		integer, intent(in) :: number

		integer :: dataSetAttributeDataTypeR

		call error( "", errorMessage )
	
	end function dataSetAttributeDataTypeR

	function tableAttributeDataTypeN( table, name )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name

		integer :: tableAttributeDataTypeN

		call error( "", errorMessage )
	
	end function tableAttributeDataTypeN

	function tableAttributeDataTypeR( table, number )
		type(TableT), intent(in) :: table
		integer, intent(in) :: number

		integer :: tableAttributeDataTypeR

		call error( "", errorMessage )
	
	end function tableAttributeDataTypeR

	function numberOfAttributesOfAttrib( attributable )
		type( AttributableT ), intent(in) :: attributable

		integer :: numberOfAttributesOfAttrib
 
 		call error( "", errorMessage )

	end function numberOfAttributesOfAttrib

	function numberOfAttributesOfDataSet( dataSet )
		type( DataSetT ), intent(in) :: dataSet

		integer :: numberOfAttributesOfDataSet
 
 		call error( "", errorMessage )

	end function numberOfAttributesOfDataSet

	function numberOfAttributesOfArray( array )
		type( ArrayT ), intent(in) :: array

		integer :: numberOfAttributesOfArray
 
 		call error( "", errorMessage )

	end function numberOfAttributesOfArray

	function numberOfAttributesOfTable( table )
		type( TableT ), intent(in) :: table

		integer :: numberOfAttributesOfTable
 
 		call error( "", errorMessage )

	end function numberOfAttributesOfTable

	function numberOfAttributesOfBlock( block )
		type( BlockT ), intent(in) :: block

		integer :: numberOfAttributesOfBlock
 
 		call error( "", errorMessage )

	end function numberOfAttributesOfBlock

	function numberOfAttributesOfColumn( column)
		type( ColumnT ), intent(in) :: column

		integer :: numberOfAttributesOfColumn
 
 		call error( "", errorMessage )

	end function numberOfAttributesOfColumn

	subroutine attributableSetAttribute( attributable, attribute )
		type(AttributableT), intent(in) :: attributable
		type(AttributeT), intent(in) :: attribute

		call error( "", errorMessage )

	end subroutine attributableSetAttribute

	subroutine datasetSetAttribute( dataset, attribute )
		type(DataSetT), intent(in) :: dataset
		type(AttributeT), intent(in) :: attribute

		call error( "", errorMessage )

	end subroutine datasetSetAttribute

	subroutine blockSetAttribute( block, attribute )
		type(BlockT), intent(in) :: block
		type(AttributeT), intent(in) :: attribute

		call error( "", errorMessage )

	end subroutine blockSetAttribute

	subroutine arraySetAttribute( array, attribute )
		type(ArrayT), intent(in) :: array
		type(AttributeT), intent(in) :: attribute

		call error( "", errorMessage )

	end subroutine arraySetAttribute

	subroutine tableSetAttribute( table, attribute )
		type(TableT), intent(in) :: table
		type(AttributeT), intent(in) :: attribute

		call error( "", errorMessage )

	end subroutine tableSetAttribute

	subroutine columnSetAttribute( column, attribute )
		type(ColumnT), intent(in) :: column
		type(AttributeT), intent(in) :: attribute

		call error( "", errorMessage )

	end subroutine columnSetAttribute

	subroutine setAttributes( attributable, origin )
		type(AttributableT), intent(in) :: attributable, origin

		call error( "", errorMessage )

	end subroutine setAttributes

	subroutine addAttributes( attributable, origin )
		type(AttributableT), intent(in) :: attributable, origin

		call error( "", errorMessage )

	end subroutine addAttributes

	! Obsolete; use set/addAttributes
	!subroutine datasetCopyAttributes( set, origin )
	!	type(DataSetT), intent(in) :: set, origin

	!	call error( "", errorMessage )

	!end subroutine

	! Obsolete; use set/addAttributes
	!subroutine tableCopyAttributes( table, origin )
	!	type(TableT), intent(in) :: table, origin

	!	call error( "", errorMessage )

	!end subroutine

	! Obsolete; use set/addAttributes
	!subroutine arrayCopyAttributes( array, origin )
	!	type(ArrayT), intent(in) :: array, origin

	!	call error( "", errorMessage )

	!end subroutine

	! Obsolete; use set/addAttributes
	!subroutine columnCopyAttributes( col, origin )
	!	type(ColumnT), intent(in) :: col, origin

	!	call error( "", errorMessage )

	!end subroutine

! Blocks

	function blockWithNumber( dataSet, blockNumber, mode )
		type(DataSetT), intent(in) :: dataSet
		integer, intent(in) :: blockNumber
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(BlockT) :: blockWithNumber

		call error( "", errorMessage )

	end function blockWithNumber

	function blockWithName( dataSet, blockName, mode )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: blockName
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(BlockT) :: blockWithName

		call error( "", errorMessage )

	end function blockWithName


! Arrays

	function arrayWithNumber( dataSet, blockNumber, mode )
		type(DataSetT), intent(in) :: dataSet
		integer, intent(in) :: blockNumber
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(ArrayT) :: arrayWithNumber

		call error( "", errorMessage )

	end function arrayWithNumber

	function arrayWithName( dataSet, blockName, mode )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: blockName
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(ArrayT) :: arrayWithName

		call error( "", errorMessage )

	end function arrayWithName

	function arrayDataType( array )
		type(ArrayT), intent(in) :: array

		integer :: arrayDataType

		call error( "", errorMessage )

	end function arrayDataType


! Scaling

	subroutine setScalingOfColumn( column, zero, scale, toType )
		type(ColumnT), intent(in) :: column
		real(kind=DOUBLE), intent(in) :: zero
		real(kind=DOUBLE), intent(in) :: scale
		integer, intent(in) :: toType

		call error( "", errorMessage )

	end subroutine setScalingOfColumn

	subroutine setScalingOfArray( array, zero, scale, toType )
		type(ArrayT), intent(in) :: array
		real(kind=DOUBLE), intent(in) :: zero
		real(kind=DOUBLE), intent(in) :: scale
		integer, intent(in) :: toType

		call error( "", errorMessage )

	end subroutine setScalingOfArray

	function zeroOfColumn( column )
		type(ColumnT), intent(in) :: column

		real(kind=DOUBLE) :: zeroOfColumn

		call error( "", errorMessage )

	end function zeroOfColumn

	function zeroOfArray( array )
		type(ArrayT), intent(in) :: array

		real(kind=DOUBLE) :: zeroOfArray

		call error( "", errorMessage )

	end function zeroOfArray

	function scaleOfColumn( column )
		type(ColumnT), intent(in) :: column

		real(kind=DOUBLE) :: scaleOfColumn

		call error( "", errorMessage )

	end function scaleOfColumn

	function scaleOfArray( array )
		type(ArrayT), intent(in) :: array

		real(kind=DOUBLE) :: scaleOfArray

		call error( "", errorMessage )

	end function scaleOfArray

	function hasScalingOfColumn( column )
		type(ColumnT), intent(in) :: column

		logical :: hasScalingOfColumn

		call error( "", errorMessage )

	end function hasScalingOfColumn

	function hasScalingOfArray( array )
		type(ArrayT), intent(in) :: array

		logical :: hasScalingOfArray

		call error( "", errorMessage )

	end function hasScalingOfArray

	subroutine unsetScalingOfColumn( column, toType )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: toType

		call error( "", errorMessage )

	end subroutine unsetScalingOfColumn

	subroutine unsetScalingOfArray( array, toType )
		type(ArrayT), intent(in) :: array
		integer, intent(in) :: toType

		call error( "", errorMessage )

	end subroutine unsetScalingOfArray


! Null Values

	function intNullValueArray( array )
		type(ArrayT), intent(in) :: array

		integer :: intNullValueArray

		call error( "", errorMessage )

	end function intNullValueArray

	function intNullValueColumn( column )
		type(ColumnT), intent(in) :: column

		integer :: intNullValueColumn

		call error( "", errorMessage )

	end function intNullValueColumn

	function intNullValueDataComponent( dataComponent )
		type(DataComponentT), intent(in) :: dataComponent

		integer :: intNullValueDataComponent

		call error( "", errorMessage )

	end function intNullValueDataComponent

	function intNullValueNullable( nullable )
		type(NullableT), intent(in) :: nullable

		integer :: intNullValueNullable

		call error( "", errorMessage )

	end function intNullValueNullable

	function isNotNullArray( array )
		type(ArrayT), intent(in) :: array

		logical, dimension(:), pointer :: isNotNullArray

		call error( "", errorMessage )

	end function isNotNullArray

	function isNotNullColumn( column )
		type(ColumnT), intent(in) :: column

		logical, dimension(:), pointer :: isNotNullColumn

		call error( "", errorMessage )

	end function isNotNullColumn

	subroutine setNullValueArray( array, value )
		type(ArrayT), intent(in) :: array
		integer, intent(in) :: value

		call error( "", errorMessage )

	end subroutine setNullValueArray

	subroutine setNullValueColumn( column, value )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: value

		call error( "", errorMessage )

	end subroutine setNullValueColumn

	subroutine setNullValueDataComponent( dataComponent, value )
		type(DataComponentT), intent(in) :: dataComponent
		integer, intent(in) :: value

		call error( "", errorMessage )

	end subroutine setNullValueDataComponent

	subroutine setNullValueNullable( nullable, value )
		type(NullableT), intent(in) :: nullable
		integer, intent(in) :: value

		call error( "", errorMessage )

	end subroutine setNullValueNullable

	function nullDefinedArray( array )
		type(ArrayT), intent(in) :: array

		logical :: nullDefinedArray

		call error( "", errorMessage )

	end function nullDefinedArray

	function nullDefinedColumn( column )
		type(ColumnT), intent(in) :: column

		logical :: nullDefinedColumn

		call error( "", errorMessage )

	end function nullDefinedColumn

	function nullDefinedDataComponent( dataComponent )
		type(DataComponentT), intent(in) :: dataComponent

		logical :: nullDefinedDataComponent

		call error( "", errorMessage )

	end function nullDefinedDataComponent

	function nullDefinedNullable( nullable )
		type(NullableT), intent(in) :: nullable

		logical :: nullDefinedNullable

		call error( "", errorMessage )

	end function nullDefinedNullable

	function nullTypeArray( array )
		type(ArrayT), intent(in) :: array

		integer :: nullTypeArray

		call error( "", errorMessage )

	end function nullTypeArray

	function nullTypeColumn( column )
		type(ColumnT), intent(in) :: column

		integer :: nullTypeColumn

		call error( "", errorMessage )

	end function nullTypeColumn

	function nullTypeDataComponent( dataComponent )
		type(DataComponentT), intent(in) :: dataComponent

		integer :: nullTypeDataComponent

		call error( "", errorMessage )

	end function nullTypeDataComponent

	function nullTypeNullable( nullable )
		type(NullableT), intent(in) :: nullable

		integer :: nullTypeNullable

		call error( "", errorMessage )

	end function nullTypeNullable

	function hasNullsArray( array )
		type(ArrayT), intent(in) :: array

		logical :: hasNullsArray

		call error( "", errorMessage )

	end function hasNullsArray

	function hasNullsColumn( column )
		type(ColumnT), intent(in) :: column

		logical :: hasNullsColumn

		call error( "", errorMessage )

	end function hasNullsColumn

	subroutine setToNullArray( array, position )
		type(ArrayT), intent(in) :: array
 		integer, intent(in) :: position

		call error( "", errorMessage )

	end subroutine setToNullArray

	subroutine setToNullCell( column, row, position )
		type(ColumnT), intent(in) :: column
 		integer, intent(in) :: row
 		integer, intent(in) :: position

		call error( "", errorMessage )

	end subroutine setToNullCell

	subroutine setToNullColumn( column, row )
		type(ColumnT), intent(in) :: column
 		integer, intent(in) :: row

		call error( "", errorMessage )

	end subroutine setToNullColumn

	function isNullArray( array, position )
		type(ArrayT), intent(in) :: array
 		integer, intent(in) :: position

		logical :: isNullArray

		call error( "", errorMessage )

	end function isNullArray

	function isNullCell( column, row, position )
		type(ColumnT), intent(in) :: column
 		integer, intent(in) :: row
 		integer, intent(in) :: position

		logical :: isNullCell

		call error( "", errorMessage )

	end function isNullCell

	function isNullColumn( column, row )
		type(ColumnT), intent(in) :: column
 		integer, intent(in) :: row

		logical :: isNullColumn

		call error( "", errorMessage )

	end function isNullColumn

! Attributes

	function attributableAttributeWithName( attributable, name, mode )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(AttributeT) :: attributableAttributeWithName

		call error( "", errorMessage )

	end function attributableAttributeWithName

	function attributableAttributeWithNumber( attributable, number, mode )
		type(AttributableT), intent(in) :: attributable
		integer, intent(in) :: number
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(AttributeT) :: attributableAttributeWithNumber

		call error( "", errorMessage )

	end function attributableAttributeWithNumber

	function arrayAttributeWithName( array, name, mode )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(AttributeT) :: arrayAttributeWithName

		call error( "", errorMessage )

	end function arrayAttributeWithName

	function arrayAttributeWithNumber( array, number, mode )
		type(ArrayT), intent(in) :: array
		integer, intent(in) :: number
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(AttributeT) :: arrayAttributeWithNumber

		call error( "", errorMessage )

	end function arrayAttributeWithNumber

	function blockAttributeWithName( block, name, mode )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(AttributeT) :: blockAttributeWithName

		call error( "", errorMessage )

	end function blockAttributeWithName

	function blockAttributeWithNumber( block, number, mode )
		type(BlockT), intent(in) :: block
		integer, intent(in) :: number
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(AttributeT) :: blockAttributeWithNumber

		call error( "", errorMessage )

	end function blockAttributeWithNumber

	function columnAttributeWithName( column, name, mode )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(AttributeT) :: columnAttributeWithName

		call error( "", errorMessage )

	end function columnAttributeWithName

	function columnAttributeWithNumber( column, number, mode )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: number
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(AttributeT) :: columnAttributeWithNumber

		call error( "", errorMessage )

	end function columnAttributeWithNumber

	function dataSetAttributeWithName( dataSet, name, mode )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(AttributeT) :: dataSetAttributeWithName

		call error( "", errorMessage )

	end function dataSetAttributeWithName

	function dataSetAttributeWithNumber( dataSet, number, mode )
		type(DataSetT), intent(in) :: dataSet
		integer, intent(in) :: number
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(AttributeT) :: dataSetAttributeWithNumber

		call error( "", errorMessage )

	end function dataSetAttributeWithNumber

	function tableAttributeWithName( table, name, mode )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(AttributeT) :: tableAttributeWithName

		call error( "", errorMessage )

	end function tableAttributeWithName

	function tableAttributeWithNumber( table, number, mode )
		type(TableT), intent(in) :: table
		integer, intent(in) :: number
		integer, intent(in), optional :: mode !READ|WRITE|MODIFY|AS_PARENT

		type(AttributeT) :: tableAttributeWithNumber

		call error( "", errorMessage )

	end function tableAttributeWithNumber

	subroutine addCommentToAttributable( attributable, comment )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: comment

		call error( "", errorMessage )

	end subroutine addCommentToAttributable

	subroutine addCommentToDataSet( dataSet, comment )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: comment

		call error( "", errorMessage )

	end subroutine addCommentToDataSet

	subroutine addCommentToBlock( block, comment )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: comment

		call error( "", errorMessage )

	end subroutine addCommentToBlock

	subroutine addCommentToArray( array, comment )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: comment

		call error( "", errorMessage )

	end subroutine addCommentToArray

	subroutine addCommentToTable( table, comment )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: comment

		call error( "", errorMessage )

	end subroutine addCommentToTable

	subroutine addHistoryToAttributable( attributable, history )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: history

		call error( "", errorMessage )

	end subroutine addHistoryToAttributable

	subroutine addHistoryToDataSet( dataSet, history )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: history

		call error( "", errorMessage )

	end subroutine addHistoryToDataSet

	subroutine addHistoryToBlock( block, history )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: history

		call error( "", errorMessage )

	end subroutine addHistoryToBlock

	subroutine addHistoryToArray( array, history )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: history

		call error( "", errorMessage )

	end subroutine addHistoryToArray

	subroutine addHistoryToTable( table, history )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: history

		call error( "", errorMessage )

	end subroutine addHistoryToTable

	function attributableHasAttribute( attributable, name )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name

		logical :: attributableHasAttribute

		call error( "", errorMessage )

	end function attributableHasAttribute

	function dataSetHasAttribute( dataSet, name )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name

		logical :: dataSetHasAttribute

		call error( "", errorMessage )

	end function dataSetHasAttribute

	function blockHasAttribute( block, name )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name

		logical :: blockHasAttribute

		call error( "", errorMessage )

	end function blockHasAttribute

	function arrayHasAttribute( array, name )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name

		logical :: arrayHasAttribute

		call error( "", errorMessage )

	end function arrayHasAttribute

	function columnHasAttribute( column, name )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name

		logical :: columnHasAttribute

		call error( "", errorMessage )

	end function columnHasAttribute

	function tableHasAttribute( table, name )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name

		logical :: tableHasAttribute

		call error( "", errorMessage )

	end function tableHasAttribute

	function int8Attribute( attribute )
		type(AttributeT), intent(in) :: attribute

		integer(kind=INT8) :: int8Attribute 

		call error( "", errorMessage )

	end function int8Attribute 

	function int16Attribute( attribute )
		type(AttributeT), intent(in) :: attribute

		integer(kind=INT16) :: int16Attribute 

		call error( "", errorMessage )

	end function int16Attribute 

	function int32Attribute( attribute )
		type(AttributeT), intent(in) :: attribute

		integer(kind=INT32) :: int32Attribute 

		call error( "", errorMessage )

	end function int32Attribute 

	function real32Attribute( attribute )
		type(AttributeT), intent(in) :: attribute

		real(kind=SINGLE) :: real32Attribute 

		call error( "", errorMessage )

	end function real32Attribute 

	function real64Attribute( attribute )
		type(AttributeT), intent(in) :: attribute

		real(kind=DOUBLE) :: real64Attribute 

		call error( "", errorMessage )

	end function real64Attribute 

	function stringAttribute( attribute )
		type(AttributeT), intent(in) :: attribute

		character(len=stringAttributeLength) :: stringAttribute 

		call error( "", errorMessage )

	end function stringAttribute 

	function booleanAttribute( attribute )
		type(AttributeT), intent(in) :: attribute

		logical :: booleanAttribute 

		call error( "", errorMessage )

	end function booleanAttribute 

	function int8AttributableAttribute( attributable, name )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name

		integer(kind=INT8) :: int8AttributableAttribute 

		call error( "", errorMessage )

	end function int8AttributableAttribute 

	function int8DataSetAttribute( dataSet, name )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name

		integer(kind=INT8) :: int8DataSetAttribute 

		call error( "", errorMessage )

	end function int8DataSetAttribute 

	function int8BlockAttribute( Block, name )
		type(BlockT), intent(in) :: block 
		character(len=*), intent(in) :: name

		integer(kind=INT8) :: int8BlockAttribute 

		call error( "", errorMessage )

	end function int8BlockAttribute 

	function int8ArrayAttribute( array, name )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name

		integer(kind=INT8) :: int8ArrayAttribute 

		call error( "", errorMessage )

	end function int8ArrayAttribute 

	function int8ColumnAttribute( column, name )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name

		integer(kind=INT8) :: int8ColumnAttribute 

		call error( "", errorMessage )

	end function int8ColumnAttribute 

	function int8TableAttribute( table, name )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name

		integer(kind=INT8) :: int8TableAttribute 

		call error( "", errorMessage )

	end function int8TableAttribute

	function int16AttributableAttribute( attributable, name )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name

		integer(kind=INT16) :: int16AttributableAttribute 

		call error( "", errorMessage )

	end function int16AttributableAttribute 

	function int16DataSetAttribute( dataSet, name )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name

		integer(kind=INT16) :: int16DataSetAttribute 

		call error( "", errorMessage )

	end function int16DataSetAttribute 

	function int16BlockAttribute( Block, name )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name

		integer(kind=INT16) :: int16BlockAttribute 

		call error( "", errorMessage )

	end function int16BlockAttribute 

	function int16ArrayAttribute( array, name )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name

		integer(kind=INT16) :: int16ArrayAttribute 

		call error( "", errorMessage )

	end function int16ArrayAttribute 

	function int16ColumnAttribute( column, name )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name

		integer(kind=INT16) :: int16ColumnAttribute 

		call error( "", errorMessage )

	end function int16ColumnAttribute 

	function int16TableAttribute( table, name )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name

		integer(kind=INT16) :: int16TableAttribute 

		call error( "", errorMessage )

	end function int16TableAttribute 

	function int32AttributableAttribute( attributable, name )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name

		integer(kind=INT32) :: int32AttributableAttribute 

		call error( "", errorMessage )

	end function int32AttributableAttribute 

	function int32DataSetAttribute( dataSet, name )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name

		integer(kind=INT32) :: int32DataSetAttribute 

		call error( "", errorMessage )

	end function int32DataSetAttribute 

	function int32BlockAttribute( block, name )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name

		integer(kind=INT32) :: int32BlockAttribute 

		call error( "", errorMessage )

	end function int32BlockAttribute 

	function int32ArrayAttribute( array, name )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name

		integer(kind=INT32) :: int32ArrayAttribute 

		call error( "", errorMessage )

	end function int32ArrayAttribute 

	function int32ColumnAttribute( column, name )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name

		integer(kind=INT32) :: int32ColumnAttribute 

		call error( "", errorMessage )

	end function int32ColumnAttribute 

	function int32TableAttribute( table, name )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name

		integer(kind=INT32) :: int32TableAttribute 

		call error( "", errorMessage )

	end function int32TableAttribute 

	function real32AttributableAttribute( attributable, name )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name

		real(kind=SINGLE) :: real32AttributableAttribute 

		call error( "", errorMessage )

	end function real32AttributableAttribute 

	function real32DataSetAttribute( dataSet, name )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name

		real(kind=SINGLE) :: real32DataSetAttribute 

		call error( "", errorMessage )

	end function real32DataSetAttribute 

	function real32BlockAttribute( Block, name )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name

		real(kind=SINGLE) :: real32BlockAttribute 

		call error( "", errorMessage )

	end function real32BlockAttribute 

	function real32ArrayAttribute( array, name )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name

		real(kind=SINGLE) :: real32ArrayAttribute 

		call error( "", errorMessage )

	end function real32ArrayAttribute 

	function real32ColumnAttribute( column, name )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name

		real(kind=SINGLE) :: real32ColumnAttribute 

		call error( "", errorMessage )

	end function real32ColumnAttribute 

	function real32TableAttribute( table, name )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name

		real(kind=SINGLE) :: real32TableAttribute 

		call error( "", errorMessage )

	end function real32TableAttribute 

	function real64AttributableAttribute( attributable, name )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name

		real(kind=DOUBLE) :: real64AttributableAttribute 

		call error( "", errorMessage )

	end function real64AttributableAttribute 

	function real64DataSetAttribute( dataSet, name )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name

		real(kind=DOUBLE) :: real64DataSetAttribute 

		call error( "", errorMessage )

	end function real64DataSetAttribute 

	function real64BlockAttribute( Block, name )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name

		real(kind=DOUBLE) :: real64BlockAttribute 

		call error( "", errorMessage )

	end function real64BlockAttribute 

	function real64ArrayAttribute( array, name )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name

		real(kind=DOUBLE) :: real64ArrayAttribute 

		call error( "", errorMessage )

	end function real64ArrayAttribute 

	function real64ColumnAttribute( column, name )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name

		real(kind=DOUBLE) :: real64ColumnAttribute 

		call error( "", errorMessage )

	end function real64ColumnAttribute 

	function real64TableAttribute( table, name )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name

		real(kind=DOUBLE) :: real64TableAttribute 

		call error( "", errorMessage )

	end function real64TableAttribute 

	function stringAttributableAttribute( attributable, name )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: stringAttributableAttribute 

		call error( "", errorMessage )

	end function stringAttributableAttribute

	function stringDataSetAttribute( dataSet, name )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: stringDataSetAttribute 

		call error( "", errorMessage )

	end function stringDataSetAttribute

	function stringBlockAttribute( Block, name )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: stringBlockAttribute 

		call error( "", errorMessage )

	end function stringBlockAttribute 

	function stringArrayAttribute( array, name )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: stringArrayAttribute 

		call error( "", errorMessage )

	end function stringArrayAttribute 

	function stringColumnAttribute( column, name )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: stringColumnAttribute 

		call error( "", errorMessage )

	end function stringColumnAttribute 

	function stringTableAttribute( table, name )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: stringTableAttribute 

		call error( "", errorMessage )

	end function stringTableAttribute 

	function booleanAttributableAttribute( attributable, name )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name

		logical :: booleanAttributableAttribute 

		call error( "", errorMessage )

	end function booleanAttributableAttribute 

	function booleanDataSetAttribute( dataSet, name )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name

		logical :: booleanDataSetAttribute 

		call error( "", errorMessage )

	end function booleanDataSetAttribute 

	function booleanBlockAttribute( block, name )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name

		logical :: booleanBlockAttribute 

		call error( "", errorMessage )

	end function booleanBlockAttribute 

	function booleanArrayAttribute( array, name )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name

		logical :: booleanArrayAttribute 

		call error( "", errorMessage )

	end function booleanArrayAttribute 

	function booleanColumnAttribute( column, name )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name

		logical :: booleanColumnAttribute 

		call error( "", errorMessage )

	end function booleanColumnAttribute 

	function booleanTableAttribute( table, name )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name

		logical :: booleanTableAttribute 

		call error( "", errorMessage )

	end function booleanTableAttribute 

	subroutine setInt8Attribute( attribute, value, units, comment )
		type(AttributeT), intent(in) :: attribute
		integer(kind=INT8), intent(in) :: value
		character(len=*), intent(in), optional :: units
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setInt8Attribute

	subroutine setInt16Attribute( attribute, value, units, comment )
		type(AttributeT), intent(in) :: attribute
		integer(kind=INT16), intent(in) :: value
		character(len=*), intent(in), optional :: units
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setInt16Attribute

	subroutine setInt32Attribute( attribute, value, units, comment )
		type(AttributeT), intent(in) :: attribute
		integer(kind=INT32), intent(in) :: value
		character(len=*), intent(in), optional :: units
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setInt32Attribute

	subroutine setReal32Attribute( attribute, value, units, comment )
		type(AttributeT), intent(in) :: attribute
		real(kind=SINGLE), intent(in) :: value
		character(len=*), intent(in), optional :: units
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setReal32Attribute

	subroutine setReal64Attribute( attribute, value, units, comment )
		type(AttributeT), intent(in) :: attribute
		real(kind=DOUBLE), intent(in) :: value
		character(len=*), intent(in), optional :: units
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setReal64Attribute

	subroutine setStringAttribute( attribute, value, comment )
		type(AttributeT), intent(in) :: attribute
		character(len=*), intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setStringAttribute

	subroutine setBooleanAttribute( attribute, value, comment )
		type(AttributeT), intent(in) :: attribute
		logical, intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setBooleanAttribute

	subroutine setInt8AttributableAttribute( attributable, name, value, units, comment )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name
		integer(kind=INT8), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt8AttributableAttribute

	subroutine setInt8DataSetAttribute( dataSet, name, value, units, comment )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name
		integer(kind=INT8), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt8DataSetAttribute

	subroutine setInt8BlockAttribute( block, name, value, units, comment )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name
		integer(kind=INT8), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt8BlockAttribute

	subroutine setInt8ArrayAttribute( array, name, value, units, comment )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name
		integer(kind=INT8), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt8ArrayAttribute

	subroutine setInt8ColumnAttribute( column, name, value, units, comment )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name
		integer(kind=INT8), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt8ColumnAttribute

	subroutine setInt8TableAttribute( table, name, value, units, comment )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name
		integer(kind=INT8), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt8TableAttribute

	subroutine setInt16AttributableAttribute( attributable, name, value, units, comment )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name
		integer(kind=INT16), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt16AttributableAttribute

	subroutine setInt16DataSetAttribute( dataSet, name, value, units, comment )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name
		integer(kind=INT16), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt16DataSetAttribute

	subroutine setInt16BlockAttribute( block, name, value, units, comment )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name
		integer(kind=INT16), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt16BlockAttribute

	subroutine setInt16ArrayAttribute( array, name, value, units, comment )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name
		integer(kind=INT16), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt16ArrayAttribute

	subroutine setInt16ColumnAttribute( column, name, value, units, comment )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name
		integer(kind=INT16), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt16ColumnAttribute

	subroutine setInt16TableAttribute( table, name, value, units, comment )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name
		integer(kind=INT16), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt16TableAttribute

	subroutine setInt32AttributableAttribute( attributable, name, value, units, comment )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name
		integer(kind=INT32), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt32AttributableAttribute

	subroutine setInt32DataSetAttribute( dataSet, name, value, units, comment )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name
		integer(kind=INT32), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt32DataSetAttribute

	subroutine setInt32BlockAttribute( block, name, value, units, comment )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name
		integer(kind=INT32), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt32BlockAttribute

	subroutine setInt32ArrayAttribute( array, name, value, units, comment )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name
		integer(kind=INT32), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt32ArrayAttribute

	subroutine setInt32ColumnAttribute( column, name, value, units, comment )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name
		integer(kind=INT32), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt32ColumnAttribute

	subroutine setInt32TableAttribute( table, name, value, units, comment )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name
		integer(kind=INT32), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setInt32TableAttribute

	subroutine setReal32AttributableAttribute( attributable, name, value, units, comment )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name
		real(kind=SINGLE), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setReal32AttributableAttribute

	subroutine setReal32DataSetAttribute( dataSet, name, value, units, comment )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name
		real(kind=SINGLE), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setReal32DataSetAttribute

	subroutine setReal32BlockAttribute( block, name, value, units, comment )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name
		real(kind=SINGLE), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setReal32BlockAttribute

	subroutine setReal32ArrayAttribute( array, name, value, units, comment )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name
		real(kind=SINGLE), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setReal32ArrayAttribute

	subroutine setReal32ColumnAttribute( column, name, value, units, comment )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name
		real(kind=SINGLE), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setReal32ColumnAttribute

	subroutine setReal32TableAttribute( table, name, value, units, comment )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name
		real(kind=SINGLE), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setReal32TableAttribute

	subroutine setReal64AttributableAttribute( attributable, name, value, units, comment )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name
		real(kind=DOUBLE), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setReal64AttributableAttribute

	subroutine setReal64DataSetAttribute( dataSet, name, value, units, comment )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name
		real(kind=DOUBLE), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setReal64DataSetAttribute

	subroutine setReal64BlockAttribute( block, name, value, units, comment )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name
		real(kind=DOUBLE), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setReal64BlockAttribute

	subroutine setReal64ArrayAttribute( array, name, value, units, comment )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name
		real(kind=DOUBLE), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setReal64ArrayAttribute

	subroutine setReal64ColumnAttribute( column, name, value, units, comment )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name
		real(kind=DOUBLE), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setReal64ColumnAttribute

	subroutine setReal64TableAttribute( table, name, value, units, comment )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name
		real(kind=DOUBLE), intent(in) :: value
		character(len=*), intent(in), optional :: units, comment

		call error( "", errorMessage )

	end subroutine setReal64TableAttribute

	subroutine setStringAttributableAttribute( attributable, name, value, comment )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name
		character(len=*), intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setStringAttributableAttribute

	subroutine setStringDataSetAttribute( dataSet, name, value, comment )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name
		character(len=*), intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setStringDataSetAttribute

	subroutine setStringBlockAttribute( block, name, value, comment )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name
		character(len=*), intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setStringBlockAttribute

	subroutine setStringArrayAttribute( array, name, value, comment )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name
		character(len=*), intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setStringArrayAttribute

	subroutine setStringColumnAttribute( column, name, value, comment )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name
		character(len=*), intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setStringColumnAttribute

	subroutine setStringTableAttribute( table, name, value, comment )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name
		character(len=*), intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setStringTableAttribute

	subroutine setBooleanAttributableAttribute( attributable, name, value, comment )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name
		logical, intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setBooleanAttributableAttribute

	subroutine setBooleanDataSetAttribute( dataSet, name, value, comment )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name
		logical, intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setBooleanDataSetAttribute

	subroutine setBooleanBlockAttribute( block, name, value, comment )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name
		logical, intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setBooleanBlockAttribute

	subroutine setBooleanArrayAttribute( array, name, value, comment )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name
		logical, intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setBooleanArrayAttribute

	subroutine setBooleanColumnAttribute( column, name, value, comment )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name
		logical, intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setBooleanColumnAttribute

	subroutine setBooleanTableAttribute( table, name, value, comment )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name
		logical, intent(in) :: value
		character(len=*), intent(in), optional :: comment

		call error( "", errorMessage )

	end subroutine setBooleanTableAttribute

	subroutine deleteAttribute( attribute )
		type(AttributeT), intent(in) :: attribute

		call error( "", errorMessage )

	end subroutine deleteAttribute

	subroutine deleteAttribAttributeWithNumber( attributable, attributeNumber )
		type(AttributableT), intent(in) :: attributable
		integer, intent(in) :: attributeNumber

		call error( "", errorMessage )

	end subroutine deleteAttribAttributeWithNumber

	subroutine deleteAttribAttributeWithName( attributable, name )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name

		call error( "", errorMessage )

	end subroutine deleteAttribAttributeWithName

	subroutine deleteArrayAttributeWithNumber( array, attributeNumber )
		type(ArrayT), intent(in) :: array
		integer, intent(in) :: attributeNumber

		call error( "", errorMessage )

	end subroutine deleteArrayAttributeWithNumber

	subroutine deleteArrayAttributeWithName( array, name )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name

		call error( "", errorMessage )

	end subroutine deleteArrayAttributeWithName

	subroutine deleteBlockAttributeWithNumber( block, attributeNumber )
		type(BlockT), intent(in) :: block
		integer, intent(in) :: attributeNumber

		call error( "", errorMessage )

	end subroutine deleteBlockAttributeWithNumber

	subroutine deleteBlockAttributeWithName( block, name )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name

		call error( "", errorMessage )

	end subroutine deleteBlockAttributeWithName

	subroutine deleteColumnAttributeWithNumber( column, attributeNumber )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: attributeNumber

		call error( "", errorMessage )

	end subroutine deleteColumnAttributeWithNumber

	subroutine deleteColumnAttributeWithName( column, name )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name

		call error( "", errorMessage )

	end subroutine deleteColumnAttributeWithName

	subroutine deleteDatSetAttributeWithNumber( dataSet, attributeNumber )
		type(DataSetT), intent(in) :: dataSet
		integer, intent(in) :: attributeNumber

		call error( "", errorMessage )

	end subroutine deleteDatSetAttributeWithNumber

	subroutine deleteDataSetAttributeWithName( dataSet, name )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name

		call error( "", errorMessage )

	end subroutine deleteDataSetAttributeWithName

	subroutine deleteTableAttributeWithNumber( table, attributeNumber )
		type(TableT), intent(in) :: table
		integer, intent(in) :: attributeNumber

		call error( "", errorMessage )

	end subroutine deleteTableAttributeWithNumber

	subroutine deleteTableAttributeWithName( table, name )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name

		call error( "", errorMessage )

	end subroutine deleteTableAttributeWithName

	function unitsOfAttribute( attribute )
		type(AttributeT), intent(in) :: attribute

		character(len=stringAttributeLength) :: unitsOfAttribute

		call error( "", errorMessage )

	end function unitsOfAttribute

	function attributableAttributeUnits( attributable, name )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: attributableAttributeUnits

		call error( "", errorMessage )

	end function attributableAttributeUnits

	function dataSetAttributeUnits( dataSet, name )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: dataSetAttributeUnits

		call error( "", errorMessage )

	end function dataSetAttributeUnits

	function blockAttributeUnits( block, name )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: blockAttributeUnits

		call error( "", errorMessage )

	end function blockAttributeUnits

	function arrayAttributeUnits( array, name )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: arrayAttributeUnits

		call error( "", errorMessage )

	end function arrayAttributeUnits

	function columnAttributeUnits( column, name )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: columnAttributeUnits

		call error( "", errorMessage )

	end function columnAttributeUnits

	function tableAttributeUnits( table, name )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: tableAttributeUnits

		call error( "", errorMessage )

	end function tableAttributeUnits

	function attributableAttributeComment( attributable, name )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: attributableAttributeComment

		call error( "", errorMessage )

	end function attributableAttributeComment

	function dataSetAttributeComment( dataSet, name )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: dataSetAttributeComment

		call error( "", errorMessage )

	end function dataSetAttributeComment

	function blockAttributeComment( block, name )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: blockAttributeComment

		call error( "", errorMessage )

	end function blockAttributeComment

	function arrayAttributeComment( array, name )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: arrayAttributeComment

		call error( "", errorMessage )

	end function arrayAttributeComment

	function columnAttributeComment( column, name )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: columnAttributeComment

		call error( "", errorMessage )

	end function columnAttributeComment

	function tableAttributeComment( table, name )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: name

		character(len=stringAttributeLength) :: tableAttributeComment

		call error( "", errorMessage )

	end function tableAttributeComment

! Setting units

	subroutine setArrayUnits( array, units )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: units

		call error( "", errorMessage )

	end subroutine setArrayUnits

	subroutine setColumnUnits( column, units )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: units

		call error( "", errorMessage )

	end subroutine setColumnUnits

	subroutine setAttributeUnits( attribute, units )
		type(AttributeT), intent(in) :: attribute
		character(len=*), intent(in):: units

		call error( "", errorMessage )

	end subroutine setAttributeUnits

 	subroutine setAttributableAttributeUnits( attributable, attributeName, units )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: attributeName
		character(len=*), intent(in) :: units

		call error( "", errorMessage )

	end subroutine

 	subroutine setDataSetAttributeUnits( dataSet, attributeName, units )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: attributeName
		character(len=*), intent(in) :: units

		call error( "", errorMessage )

	end subroutine setDataSetAttributeUnits

	subroutine setBlockAttributeUnits( block, attributeName, units )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: attributeName
		character(len=*), intent(in) :: units

		call error( "", errorMessage )

	end subroutine setBlockAttributeUnits

	subroutine setArrayAttributeUnits( array, attributeName, units )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: attributeName
		character(len=*), intent(in) :: units

		call error( "", errorMessage )

	end subroutine setArrayAttributeUnits

	subroutine setColumnAttributeUnits( column, attributeName, units )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: attributeName
		character(len=*), intent(in) :: units

		call error( "", errorMessage )

	end subroutine setColumnAttributeUnits

	subroutine setTableAttributeUnits( table, attributeName, units )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: attributeName
		character(len=*), intent(in) :: units

		call error( "", errorMessage )

	end subroutine setTableAttributeUnits

! Names and Labels

	function labelOfAttributable( attributable )
		type(AttributableT), intent(in) :: attributable

		character(len=IdentifierLength) :: labelOfAttributable

		call error( "", errorMessage )

	end function labelOfAttributable

	function labelOfArray( array )
		type(ArrayT), intent(in) :: array

		character(len=IdentifierLength) :: labelOfArray

		call error( "", errorMessage )

	end function labelOfArray

	function labelOfAttribute( attribute )
		type(AttributeT), intent(in) :: attribute

		character(len=stringAttributeLength) :: labelOfAttribute

		call error( "", errorMessage )

	end function labelOfAttribute

	function labelOfBlock( block )
		type(BlockT), intent(in) :: block

		character(len=IdentifierLength) :: labelOfBlock

		call error( "", errorMessage )

	end function labelOfBlock

	function labelOfLabelled( labelled )
		type(LabelledT), intent(in) :: labelled

		character(len=IdentifierLength) :: labelOfLabelled

		call error( "", errorMessage )

	end function labelOfLabelled

	function labelOfColumn( column )
		type(ColumnT), intent(in) :: column

		character(len=IdentifierLength) :: labelOfColumn

		call error( "", errorMessage )

	end function labelOfColumn

	function labelOfDataSet( dataSet )
		type(DataSetT), intent(in) :: dataSet

		character(len=IdentifierLength) :: labelOfDataSet

		call error( "", errorMessage )

	end function labelOfDataSet

	function labelOfTable( table )
		type(TableT) :: table

		character(len=IdentifierLength) :: labelOfTable

		call error( "", errorMessage )

	end function labelOfTable

	subroutine relabelAttributable( attributable, newLabel )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: newLabel

		call error( "", errorMessage )

	end subroutine relabelAttributable

	subroutine relabelArray( array, newLabel )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: newLabel

		call error( "", errorMessage )

	end subroutine relabelArray

	subroutine relabelAttribute( attribute, newLabel )
		type(AttributeT), intent(in) :: attribute
		character(len=*), intent(in) :: newLabel

		call error( "", errorMessage )

	end subroutine relabelAttribute

	subroutine relabelBlock( block, newLabel )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: newLabel

		call error( "", errorMessage )

	end subroutine relabelBlock

	subroutine relabelLabelled( labelled, newLabel )
		type(LabelledT), intent(in) :: labelled
		character(len=*), intent(in) :: newLabel

		call error( "", errorMessage )

	end subroutine relabelLabelled

	subroutine relabelColumn( column, newLabel )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: newLabel

		call error( "", errorMessage )

	end subroutine relabelColumn

	subroutine relabelDataSet( dataSet, newLabel )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: newLabel

		call error( "", errorMessage )

	end subroutine relabelDataSet

	subroutine relabelTable( table, newLabel )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: newLabel

		call error( "", errorMessage )

	end subroutine relabelTable

	subroutine renameAttribute( attribute, newName )
		type(AttributeT), intent(in) :: attribute
		character(len=*), intent(in) :: newName

		call error( "", errorMessage )

	end subroutine renameAttribute

	subroutine renameArray( array, newName )
		type(ArrayT), intent(in) :: array
		character(len=*), intent(in) :: newName

		call error( "", errorMessage )

	end subroutine renameArray

	subroutine renameBlock( block, newName )
		type(BlockT), intent(in) :: block
		character(len=*), intent(in) :: newName

		call error( "", errorMessage )

	end subroutine renameBlock

	subroutine renameLabelled( labelled, newName )
		type(LabelledT), intent(in) :: labelled
		character(len=*), intent(in) :: newName

		call error( "", errorMessage )

	end subroutine renameLabelled

	subroutine renameColumn( column, newName )
		type(ColumnT), intent(in) :: column
		character(len=*), intent(in) :: newName

		call error( "", errorMessage )

	end subroutine renameColumn

	subroutine renameAttributable( attributable, newName )
		type(AttributableT), intent(in) :: attributable
		character(len=*), intent(in) :: newName

		call error( "", errorMessage )

	end subroutine renameAttributable

	subroutine renameDataSet( dataSet, newName )
		type(DataSetT), intent(in) :: dataSet
		character(len=*), intent(in) :: newName

		call error( "", errorMessage )

	end subroutine renameDataSet

	subroutine renameTable( table, newName )
		type(TableT), intent(in) :: table
		character(len=*), intent(in) :: newName

		call error( "", errorMessage )

	end subroutine renameTable

! Extended DAL

	subroutine forEachColumn( table, callThisFunction )
		type(TableT), intent(in) :: table
		interface
			subroutine callThisFunction( column )
				use Daltypes

				type(ColumnT), intent(in) :: column
			end subroutine callThisFunction
		end interface

		call error( "", errorMessage )

	end subroutine forEachColumn

	subroutine forEachBlock( dataSet, callThisFunction )
		type(DataSetT), intent(in) :: dataSet
		interface
			subroutine callThisFunction( block )

				use Daltypes

				type(BlockT), intent(in) :: block
			end subroutine callThisFunction
		end interface

		call error( "", errorMessage )

	end subroutine forEachBlock

	subroutine forEachSubTable( table, callThisFunction )
		type(TableT), intent(in) :: table
		interface
			subroutine callThisFunction( subTable )
				use Daltypes
				type(TableT), intent(in) :: subTable
			end subroutine callThisFunction
		end interface

		call error( "", errorMessage )

	end subroutine forEachSubTable

        subroutine forEachRow( table, fn )
		type(TableT), intent(in) :: table
		interface
			subroutine fn( r )
				use DalTypes
				type(TableT), intent(in) :: r
			end subroutine
		end interface

		call error( "", errorMessage )

	end subroutine forEachRow

	function subTable( table, from, to )
		type(TableT), intent(in) :: table
		integer, intent(in), optional :: from
		integer, intent(in), optional :: to

		type(SubTableT) :: subTable

		call error( "", errorMessage )

	end function subTable

	function subTableNext( subTable )
		type(SubTableT), intent(in) :: subTable

		logical subTableNext

		call error( "", errorMessage )

	end function subTableNext

        subroutine subTableSeek( table, from, count )
		type(SubTableT), intent(in) :: table
		integer, intent(in) :: from, count

		call error( "", errorMessage )

	end subroutine subTableSeek

	function countTable( table )
		type(TableT), intent(in) :: table

		integer :: countTable

		call error( "", errorMessage )

	end function countTable

	function countColumn( column )
		type(ColumnT), intent(in) :: column

		integer :: countColumn

		call error( "", errorMessage )

	end function countColumn

	function fromTable( table )
		type(TableT), intent(in) :: table

		integer :: fromTable

		call error( "", errorMessage )

	end function fromTable

	function fromColumn( column )
		type(ColumnT), intent(in) :: column

		integer :: fromColumn

		call error( "", errorMessage )

	end function fromColumn

	subroutine seekTable( table, from, count )
		type(TableT), intent(in) :: table
		integer, intent(in) :: from
		integer, intent(in) :: count

		call error( "", errorMessage )

	end subroutine seekTable

	subroutine seekColumn( column, from, count )
		type(ColumnT), intent(in) :: column
		integer, intent(in) :: from
		integer, intent(in) :: count

		call error( "", errorMessage )

	end subroutine seekColumn

	! Functions/Subroutines related to the TableSeeker class
	function newTableSeeker( table )
		type(TableT), intent(in) :: table

		type(TableSeekerT) :: newTableSeeker

		call error( "", errorMessage )

	end function newTableSeeker

	subroutine releaseTableSeeker( tableSeeker )
		type(TableSeekerT), intent(in) :: tableSeeker

		call error( "", errorMessage )

	end subroutine releaseTableSeeker

	function tableSeekerNext( tableSeeker )
		type(TableSeekerT), intent(in) :: tableSeeker
		
		logical :: tableSeekerNext

		call error( "", errorMessage )

	end function tableSeekerNext

	function fromTableSeeker( tableSeeker )
		type(TableSeekerT), intent(in) :: tableSeeker

		integer :: fromTableSeeker

		call error( "", errorMessage )

	end function fromTableSeeker

	function countTableSeeker( tableSeeker )
		type(TableSeekerT), intent(in) :: tableSeeker

		integer :: countTableSeeker

		call error( "", errorMessage )

	end function countTableSeeker

	subroutine resetTableSeeker( tableSeeker )
		type(TableSeekerT), intent(in) :: tableSeeker

		call error( "", errorMessage )

	end subroutine resetTableSeeker

	subroutine setDataxxx(toColumn, fromColumn)   !PV 31-7-2000
    		type(ColumnT), intent(in) :: fromcolumn
    		type(ColumnT) :: toColumn

		call error( "", errorMessage )

  	end subroutine 

! client
        
        function client()
          character(len=identifierLength) :: client
          call error("", errorMessage)
        end function client

        subroutine setClient(nameOfClient)
          character(len=*) :: nameOfClient
          call error("", errorMessage)
        end subroutine setClient

end module Dal



