using namespace std;
/*************************************************************** 
* 
* DalArrayData.cc           & XMM Science Analysis System (SAS) 
* 
* Class representing DAL Array Data.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: DalArrayData.cc,v 1.1.1.1 2002/08/30 13:06:29 gvacanti Exp $
* 
*****************************************************************/ 

#include <Tracer.h>
#include <vectorutil.h>
#include <errstr.h>


#include "DalArrayData.h"


DalArrayData::DalArrayData(
	const Array *			parent,
	const longVec&			from,
	const longVec&			count )
	:
		DalSeekable<longVec>( from, count ),
		DalChild<Array>( parent )
{
	PROC
	ENDPROC
}


static bool
singular( const longVec& pos )
{
	PROC
	for( unsigned int i = 0; i < pos.size(); ++i )
	{
		if( pos[ i ] )
		{
			ALTRETURNV( false );
		} // if
	} // for
	RETURN true;
}


unsigned long
DalArrayData::position( const longVec& pos ) const
{
	PROC

	unsigned dim = this -> DalChild<Array>::parent()->size().size();

	if (pos.size() != dim)
		errstr << "the position into array " << this -> DalChild<Array>::parent()->name() << " has dimension " << pos.size() << " whereas the data has dimension " << dim << Sas::error("invalidPosition");

	// TEMP SOLUTION TO SPR-1166
	if( singular( pos ) )
	{
		ALTRETURNV( 0 );
	} // if

	unsigned long idx = 0, mult = 1;

	const longVec& fromVec = from(), countVec = count();

	for (unsigned i = dim, ix = dim-1; i; --i,--ix) {

		unsigned long from = ( fromVec == zero ? 0 : fromVec[ix] );
		unsigned long count = ( countVec == zero ? size()[ix] - from : countVec[ix] );
		
		SAS_DEBUG( clog << pos[ix] << ' ' << from << ' ' << count << endl; )

		if (pos[ix] < from || pos[ix] >= from + count)
			errstr << "cannot access position " << pos <<" since the array data starts at " << fromVec << " and has length " << countVec << Sas::error("invalidPosition");

		unsigned long dist = pos[ix] - from;

		idx += dist*mult;

		if (ix) mult *= count;
	}

	RETURN idx;
}



