/*************************************************************** 
* 
* DalAttributable.h          & XMM Science Analysis System (SAS) 
* 
* Class representing a set of attributes.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: DalAttributable.h,v 1.1.1.1 2002/08/30 13:06:29 gvacanti Exp $
* 
*****************************************************************/ 
#ifndef DAL_ATTRIBUTABLE_H
#define DAL_ATTRIBUTABLE_H

#include "Dal.h"
#include "DalLabelled.h"
#include "DalDal.h"

class DalAttribute;

class DalAttributable : public virtual Attributable, public DalLabelled, public DalDal
{
	public :

		DalAttributable(
				const std::string &		name,
				const std::string &		label,
				AccessMode		accessMode,
				const DalDal *		parentAccessMode );

		virtual ~DalAttributable();

		virtual Attributable& operator=(const Attributable& );

		Attribute * addAttribute(
					const std::string &		name,
					const std::string &		units = "",
					const std::string &		comment = "" );

		Attribute* addAttribute( const Attribute*  attribute );

		void addAttributes( const Attributable* );

		bool hasAttribute( const std::string & attributeName ) const;
		const Attribute * attribute(
					const std::string&	attributeName,
					AccessMode	accessMode=AsParent ) const;
		Attribute * attribute(
					const std::string&	attributeName,
					AccessMode	accessMode=AsParent );
		const Attribute * attribute(
					unsigned int	number,
					AccessMode	accessMode=AsParent ) const;
		Attribute * attribute(
					unsigned int	number,
					AccessMode	accessMode=AsParent );
		unsigned int attributeNumber(
					const std::string&	attributeName ) const;

		void deleteAttribute( const std::string & name );
		void deleteAttribute( unsigned int number );
		unsigned int attributes() const
			{ PROC RETURN _attributes.size(); }

		void addComment( const std::string &	comment )
			{ PROC _comments.push_back( comment ); ENDPROC }
		unsigned int comments() const
			{ PROC RETURN _comments.size(); }
		const std::string & comment( unsigned int number ) const;

		void addHistory( const std::string & history )
			{ PROC _history.push_back( history ); ENDPROC }
		unsigned int historys() const
			{ PROC RETURN _history.size(); }
		const std::string & history ( unsigned int number ) const;

		void printOn( ostream & os ) const;

		virtual std::string parseName( const std::string & compoundName ) const;

	private :

		typedef std::vector<DalAttribute*>	AttributeList;
		typedef std::vector<std::string>		HistoryList;
		typedef std::vector<std::string>		CommentList;
		AttributeList			_attributes;
		HistoryList			_history;
		CommentList			_comments;

		// Returns the key associated with the name (which is
                // the capitalized 8 char version of the name).
		std::string key(const std::string& name) const;

		void checkValidAttributeNumber( unsigned int attributeNumber ) const;
};

#endif




