/*************************************************************** 
* 
* DalAttribute.h             & XMM Science Analysis System (SAS) 
* 
* Class representing an Attribute. 
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA, and Marco Beijersbergen, CRI
* 
* $Id: DalAttribute.h,v 1.1.1.1 2002/08/30 13:06:29 gvacanti Exp $
* 
*****************************************************************/ 
#ifndef DAL_ATTRIBUTE_H
#define DAL_ATTRIBUTE_H


#include <stringutil.h>

#include "machine_specific_types.h"
#include "Dal.h"
#include "DalChild.h"
#include "DalLabelled.h"
#include "DalDal.h"
#include "DalAttributable.h"


class Value;


class DalAttribute : public virtual Attribute, public virtual DalLabelled, public DalDal, public DalChild<Attributable>
{
	public :

		// Constructors
		DalAttribute(
			const Attribute*	org,
			const DalAttributable*	parent );

		DalAttribute(
			const std::string &		name,
			const std::string &		units,
			const std::string &		comment,
			const DalAttributable *	parent );

		virtual ~DalAttribute();

		virtual Attribute & operator=( int		value );
		virtual Attribute & operator=( double		value );
		virtual Attribute & operator=( bool 		value );
		virtual Attribute & operator=( const char *	value );
		virtual Attribute & operator=( const std::string &	value );
		virtual Attribute & operator=( const Attribute& );

		int asInt() const;
		double asReal() const;
		std::string asString() const;
		bool asBool() const;

		const std::string & units() const;
		void units(const std::string& u);

		DataType dataType() const;

		void dataType(DataType type);

		char* addressOfValue() const;

		unsigned int dataSize() const;

		unsigned int attributeNumber() const;

		void printOn( ostream & os ) const;

		std::string parseName( const std::string& compoundName ) const;

		std::string qualifiedName() const;

		void release();

	private :

		Value*		_value;
		std::string		_units;

		void checkValue() const;
};

#endif















