/*************************************************************** 
* 
* DalDataComponent.h & XMM Science Analysis System (SAS) 
* 
* Class representing DAL DataComponent.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: DalDataComponent.h,v 1.1.1.1 2002/08/30 13:06:29 gvacanti Exp $
* 
*****************************************************************/ 
#ifndef DAL_DATA_COMPONENT_H
#define DAL_DATA_COMPONENT_H


#include <Dal.h>
#include "DalNullable.h"


class DalDataComponent : public virtual DataComponent, public DalNullable
{
	public :
		DalDataComponent(
			DataType			dataType,
			const longVec&			size,
			const std::string &			units,
			const Labelled*			labelled );

		virtual ~DalDataComponent();
		DalDataComponent& operator=( const Column& c );

		const std::string& units() const;
		void units(const std::string& units);

		void scaling( double zero, double scale );

		void scale( bool onoff );

		bool scaled() const;
		
		double scaleFactor() const;

		double scaleZero() const;

		DataType dataType() const;
		const longVec& size() const;
		unsigned int dimensions() const;
		void removeDataObject( Data* thisDataObject );
		void addDataObject( Data* thisDataObject );
		void release();

	protected :

		mutable vector<Data*>	_dataObjects;
		void checkUnreleasedData();

	private :

		// Disallow
		DalDataComponent( const DataComponent& );
		DalDataComponent& operator=( const DataComponent& c );

		// *****************************************************************
		// The _labelled pointer will be used to get the name of the
		// object to which the 'data' objects belong. It will be used in the
		// destructor to raise warnings for any 'data' objects which
		// have not been released.
		// There is thus a restriction on the order of the base classes
		// Attributable and DataComponent. Refer to the comments in the
		// destructor definition for further information.
		// *****************************************************************
		const Labelled*		_labelled;
		DataType		_dataType;
		longVec			_size;
		std::string			_units;
		bool			_scaled;
		double 			_scaleZero;
		double 			_scaleFactor;
};

#endif

