/*************************************************************** 
* 
* DalDataSet.h               & XMM Science Analysis System (SAS) 
* 
* Class representing a DAL Data Set.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: DalDataSet.h,v 1.1.1.1 2002/08/30 13:06:29 gvacanti Exp $ 
* 
*****************************************************************/ 
#ifndef DAL_DATA_SET_H
#define DAL_DATA_SET_H


#include "Dal.h"
#include "DataSetReaderWriter.h"
#include "DalAttributable.h"
#include "DalChild.h"
#include "DalDataSetServer.h"


class DalBlock;
class DalTable;
class DalArray;


class DalDataSet : public virtual DataSet, public DalAttributable, public DalChild<DataSetServer>, public DalChild<DalDataSetServer>
{
	public :

		// Constructors
		DalDataSet(
				const std::string &			name,
				const std::string &			label,
				AccessMode			openMode,
				DataSetServer::MemoryModel	memoryModel,
				DalDataSetServer *		server );

		virtual ~DalDataSet(){}
		virtual DataSet& 			operator=( const DataSet& );

		std::string 				parseName( const std::string & compoundName ) const;
		std::string 				qualifiedName() const;
		//void releaseAll();

		DataSetReaderWriter* dataSetReaderWriter() const;

		Table * 			addTable(
							const std::string &		name,
							unsigned long		rows,
							const std::string &		label = "",
							int			position = -1 );

		Array *	 			addArray(
							const std::string &			name,
							DataType			dataType,
							const longVec &			size,
							const std::string &			units = "",
							const std::string &			label = "",
							int				position = -1 );

		Block * 			add(
							const Block *			block,
							const std::string&			newName = "",
							int				position = -1 );

		Table *				table( unsigned int blockNumber, AccessMode=AsParent );
		const Table *			table( unsigned int blockNumber, AccessMode=AsParent ) const;
		Table *				table( const std::string & blockName, AccessMode=AsParent );
		const Table *			table( const std::string & blockName, AccessMode=AsParent ) const;
		Array *				array( unsigned int blockNumber, AccessMode=AsParent );
		const Array *			array( unsigned int blockNumber, AccessMode=AsParent ) const;
		Array *				array( const std::string & blockName, AccessMode=AsParent );
		const Array *			array( const std::string & blockName, AccessMode=AsParent ) const;
		Block *				block( unsigned int blockNumber, AccessMode=AsParent );
		const Block *			block( unsigned int blockNumber, AccessMode=AsParent ) const;
		Block *				block( const std::string & blockName, AccessMode=AsParent );
		const Block *			block( const std::string & blockName, AccessMode=AsParent ) const;
		unsigned int			blockNumber( const std::string & name ) const;
		void				deleteBlock( unsigned int blockNumber );
		void				deleteBlock( const std::string & name );
		unsigned int			blocks() const;
		Block::Type			blockType( unsigned int blockNumber ) const;
		Block::Type			blockType( const std::string & blockName ) const;
		bool				hasBlock( const std::string & blockName ) const;
		void				rename( const std::string& newName );
		void				release();
		DataSetServer::MemoryModel	memoryModel() const;

	protected :

		typedef vector<DalBlock*>	BlockList;

		BlockList		_blocks;
		DataSetReaderWriter *	_dataSetReaderWriter;

		void		insert( DalBlock * block, int position );

		void addDebug() const;

	private :

		DataSetServer::MemoryModel		_memoryModel;

		virtual DalTable * makeTable(
				const std::string &		name,
				const std::string &		label,
				unsigned long		rows,
				DalDataSet *		parent ) = 0;

		virtual DalArray * makeArray(
				const std::string &			name,
				DataType			dataType,
				const longVec&			size,
				const std::string &			units,
				const std::string &			label,
				DalDataSet *			parent ) = 0;

		Block * add(
				const Table *			block,
				const std::string&			newName = "",
				int				position = -1 );

		Block * add(
				const Array *			block,
				const std::string&			newName = "",
				int				position = -1 );

		void checkValidBlockNumber( unsigned int blockNumber ) const;

		void checkIfValidBlock( 
				const Block*		blk,
				Block::Type		blockType ) const;


};


#endif
