/*************************************************************** 
* 
* DalDataSetServer.h         & XMM Science Analysis System (SAS) 
* 
* Class representing the DAL Data Set Server.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: DalDataSetServer.h,v 1.1.1.1 2002/08/30 13:06:29 gvacanti Exp $ 
* 
*****************************************************************/ 

#ifndef DAL_DATA_SET_SERVER_H
#define DAL_DATA_SET_SERVER_H

#include <map>

#include "Dal.h"
#include "DataSetReaderWriter.h"
#include "Tracer.h"
#include "Model.h"


class DalDataSetServer : public virtual DataSetServer
{
	public :

		DalDataSetServer();

		virtual ~DalDataSetServer();

		void client( const std::string& name );
		const std::string& client() const;

		void process( const std::string& processDescription );
		const std::string& process() const;

		void process(
				const DataSet*		dataSet,
				const std::string& processDescription );

		const std::string& process(
				const DataSet*		dataSet,
				unsigned int		processNumber ) const;

		unsigned int processes( const DataSet* dataSet ) const;

		void debug(
				const DataSet*		dataSet,
				const std::string&		debugDescription );

		const std::string& debug(
				const DataSet*		dataSet,
				unsigned int		debugNumber ) const;

		unsigned int debugs( const DataSet* dataSet ) const;

		void creator(
				const DataSet*		dataSet,
				const std::string&		creator );

		const std::string& creator( const DataSet*	dataSet ) const;

		void date(
				const DataSet*		dataSet,
				const std::string		date );

		const std::string& date( const DataSet*	dataSet ) const;

		void clobber( bool onoff );

		bool clobber() const;

		virtual DataSet * open(
				const std::string &		setName,
				Dal::AccessMode		openMode,
				MemoryModel		model,
				DataSetReaderWriter*	readerWriter );

		DataSet * clone(
				const std::string &		from,
				const std::string &		to,
				Dal::AccessMode		openMode,
				MemoryModel		model,
				DataSetReaderWriter*	readerWriter );

		void keep(
				const std::string&		setName );
		void discard(
				const std::string&		setName );

		virtual void close( DataSet * dataSet );

		void remove( const std::string& setName );

		void cleanupClient( const std::string& clientName );

		void clobberDataSet( const std::string& setName );

		virtual bool exists( const std::string & setName ) const;

		void copy( const std::string& from, const std::string& to );

		void printOn(ostream& os) const;

		void renameDataSet(
				const std::string& 		oldName,
				const std::string&		newName );

	protected :

		std::string				_client;
		Model				_defaultModel;

	private :

		std::string parseName( const std::string & compoundName ) const;
		//bool existsInMemory( const std::string & setName ) const;
		//bool existsOnDisk( const std::string & setName ) const;

		bool					_clobber;
		DataSetReaderWriter::FileType		_outputFileFormat;

		class DataSetRecord {
			public :
			DataSetRecord(
				const std::string& name,
				const std::string& clientName );
			void dataSet( DataSet* dataSet );
			DataSet* dataSet();
			void incRef();
			void decRefs();
			unsigned int refs();
			bool mustKeep() const;
			void keep();
			void discard();
			bool inMemory() const;
			bool onDisk() const;
			bool exists() const;
			void remove();
			void close();
			//void name( const std::string& name );
			const std::string& name() const;
			void client( const std::string& client );
			const std::string& client() const;
			private :
			DataSet* 	_dataSet;
			bool		_keep;
			unsigned int	_refs;
			std::string		_client;
			std::string		_name;
		};
		DataSetRecord*& getRecord( const std::string& name ) const;
		typedef std::map<std::string, DataSetRecord*> Records;
		typedef Records::iterator RecordIterator;
		typedef Records::const_iterator ConstRecordIterator;
		mutable Records _records;

		std::string					_process;
		typedef std::map<const DataSet*, std::vector<std::string> > ProcessHistory;
		ProcessHistory				_processHistory;

		typedef std::map<const DataSet*, std::vector<std::string> > DebugHistory;
		DebugHistory				_debugHistory;

		typedef std::map<const DataSet*, std::string> DateMap;
		DateMap					_dateOf;

		typedef std::map<const DataSet*, std::string> CreatorMap;
		CreatorMap				_creatorOf;

		unsigned int numberOfSetsInMemory() const;
};




#endif
