using namespace std;
/*************************************************************** 
* 
* HiLowCellData.cc            & XMM Science Analysis System (SAS) 
* 
* Class representing the Data in a Column Cell. 
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: HiLowCellData.cc,v 1.1.1.1 2002/08/30 13:06:29 gvacanti Exp $ 
* 
*****************************************************************/ 

#include "errstr.h"
#include "Tracer.h"
#include "vectorutil.h"
#include "HiLowCellData.h"

#include "DalTable.h"
#include "DalColumn.h"
#include "DalDataSet.h"
#include "Storage.h"


HiLowCellData::HiLowCellData(
			HiLowDataStorage *		data,
			const DalColumn *		parent,
			unsigned long			rowNumber,
			unsigned long 			from,
			unsigned long 			count,
			AccessMode			accessMode )
		:	DalCellData( parent, rowNumber, from, count ),
			HiLowData<DalColumn>( data, 0, 0, parent, accessMode )
{
	PROC

	seek( from, count );

	ENDPROC
}


HiLowCellData::~HiLowCellData()
{
	PROC

	((Storage<HiLowCellData>&)*dataStorage()).parent( this );

	DalColumn * column = this -> DalChild<DalColumn>::parent();
	dataStorage() -> release(
				DalSeekable<unsigned long>::from(),
				DalSeekable<unsigned long>::count() );
	column -> removeDataObject( this );
	ENDPROC
}


void
HiLowCellData::size( unsigned long size )
{
	PROC

	dataStorage() -> size( size ); 
	seek( 0, size );

	// *************************************************
	// Set the size of the variable length column in the
	// underlying dataset.
	// *************************************************
	const DalColumn * column = DalChild<DalColumn>::parent();
	const DalTable * table = ((DalChild<DalTable>&)(*column)).parent();
	const DalDataSet * dataSet = ((DalChild<DalDataSet>&)(*table)).parent();
	dataSet -> dataSetReaderWriter() -> setVariableColumnSize( *this );

	ENDPROC
}


unsigned long
HiLowCellData::size() const
{
	PROC
	RETURN dataStorage() -> size();
}


unsigned long
HiLowCellData::row() const
{
	PROC
	RETURN _rowNumber;
}


unsigned long
HiLowCellData::elements() const
{
	PROC
	RETURN dataStorage() -> size();
}


void
HiLowCellData::seek(
	unsigned long	from,
	unsigned long	count )
{
	PROC

	//dataStorage() -> release( seekable().from(), seekable().count() );

	unsigned long	_from;
	unsigned long	_count;

	_from = from;

	if (count == 0)
		_count = dataStorage() -> size();

	else
		_count = count;

	DalSeekable< unsigned long >::seek( _from, _count );

	assert( DalSeekable<unsigned long>::from() == _from );
	assert(	DalSeekable<unsigned long>::count() == _count );

	DalSeekable<unsigned long> oldSeek = seekable();

	seekable().seek( _from, _count );

	//dataStorage() -> initialise( seekable().from(), seekable().count() );
	dataStorage() -> initialise( oldSeek, seekable() );


	// ************************************
	// TODO: Implement properly.
	// In this version seek the entire row.
	// ************************************
	//DalSeekable< unsigned long >::seek(
	//			_rowNumber,
	//			dataStorage() -> size() );

	//seekable().seek(
	//	DalSeekable<unsigned long>::from(),
	//	DalSeekable<unsigned long>::count() );

	ENDPROC
}


template<class T>
static void dumpData(
	const HiLowCellData *	cellData,
	T			values,
	unsigned long 		from,
	unsigned long 		count )
{
	PROC

	const unsigned long rowNumber = cellData -> row();
	const DalColumn * column = cellData -> DalChild<DalColumn>::parent();

	if( cellData -> mode() == Dal::Read )
	{
		// ***************
		// General Message
		// ***************
		if( errHandler.verbosity(Msg::App) >= Msg::Verbose )
		{
			errstr << "Cell[" << rowNumber << "] data for column with qualified name " << column -> qualifiedName() << " (column number is " << column -> columnNumber() << ") from " << from << " to " << from + count - 1 << " is read-only, and will not be written back to disk." << Sas::message( Msg::Core, Msg::Verbose );
		} // if

		ALTRETURN;
	} // if

	const DalTable * table = column -> DalChild<DalTable>::parent();
	const DalDataSet * dataSet = ((DalChild<DalDataSet>&)*table).parent();
        dataSet -> dataSetReaderWriter() -> dumpData( *column, rowNumber, values, from, count );
	ENDPROC
}


template<class T>
static void loadData(
	const HiLowCellData *	cellData,
	T			values,
	unsigned long 		from,
	unsigned long 		count )
{
	PROC
	const unsigned long rowNumber = cellData -> row();
	const DalColumn * column = cellData -> DalChild<DalColumn>::parent();
	const DalTable * table = column -> DalChild<DalTable>::parent();
	const DalDataSet * dataSet = ((DalChild<DalDataSet>&)*table).parent();
        dataSet -> dataSetReaderWriter() -> loadData( *column, rowNumber, values, from, count );
	ENDPROC
}


void
HiLowCellData::dumpData( char ** values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::dumpData( bool8 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::dumpData( int8 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::dumpData( uint16 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::dumpData( int16 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::dumpData( uint32 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::dumpData( int32 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::dumpData( real32 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::dumpData( real64 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::loadData( char ** values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::loadData( bool8 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::loadData( int8 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::loadData( uint16 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::loadData( int16 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::loadData( uint32 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::loadData( int32 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::loadData( real32 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowCellData::loadData( real64 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}

void
HiLowCellData::copyData( unsigned long , unsigned long , unsigned long  ) const
//  HiLowCellData::copyData( unsigned long from, unsigned long to, unsigned long count ) const
{
	PROC
	errstr << "HiLowCellData::copyData" << Sas::error( "notImplemented" );
	ENDPROC
}


int8 *
HiLowCellData::int8Data() const
{
	PROC
	((Storage<HiLowCellData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> int8Data( seekable().from(), seekable().count() );
}


uint16 *
HiLowCellData::uint16Data() const
{
	PROC
	((Storage<HiLowCellData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> uint16Data( seekable().from(), seekable().count() );
}


int16 *
HiLowCellData::int16Data() const
{
	PROC
	((Storage<HiLowCellData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> int16Data( seekable().from(), seekable().count() );
}


uint32 *
HiLowCellData::uint32Data() const
{
	PROC
	((Storage<HiLowCellData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> uint32Data( seekable().from(), seekable().count() );
}


int32 *
HiLowCellData::int32Data() const
{
	PROC
	((Storage<HiLowCellData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> int32Data( seekable().from(), seekable().count() );
}


real32 *
HiLowCellData::real32Data() const
{
	PROC
	((Storage<HiLowCellData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> real32Data( seekable().from(), seekable().count() );
}


real64 *
HiLowCellData::real64Data() const
{
	PROC
	((Storage<HiLowCellData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> real64Data( seekable().from(), seekable().count() );
}


bool8 *
HiLowCellData::boolData() const
{
	PROC
	((Storage<HiLowCellData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> boolData( seekable().from(), seekable().count() );
}


char *
HiLowCellData::stringData() const
{
	PROC
	((Storage<HiLowCellData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> stringData( seekable().from(), seekable().count() );
}
