using namespace std;


//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//						(c) 1998 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Slicer.cc implements the abstract class Slicer that represents a memory slicer manager.
//
//	DESCRIPTION:
//
//
//	USAGE EXAMPLES:
//
//	CHANGE HISTORY:
//		MT, 1998-10-01: 0.1.0: - first version
//
//	AUTHOR:
//		M.Thomas
//
//	REFERENCES:
//
//	$Header: /usr/local/cvs/sasdev/sas/packages/dal/src/highlow/Slicer.cc,v 1.1.1.1 2002/08/30 13:06:29 gvacanti Exp $
///


#include "Tracer.h"
#include "errstr.h"
#include "Slicer.h"
#include "DalSeekable.h"


// ************************************************************************************************
// Purpose: The function copyBytes( unsigned long f, unsigned long t, unsigned long c, T * values )
//		is used to copy the bytes within a given slice.
// ************************************************************************************************
template<class T>
void
copyBytes( unsigned long f, unsigned long t, unsigned long c, T * values )
{
	PROC
	// ***********************************************************
	// Check in which direction the copying should be carried out.
	// ***********************************************************
	if( f < t )
	{
		// Do the copying backwards.
		for( unsigned long i = 0; i < c; ++i )
		{
			values[ t + c - 1 - i ] = values[ f + c - 1 - i ];
		} // for
	}
	else
	{
		// Do the copying forwards.
		for( unsigned long i = 0; i < c; ++i )
		{
			values[ t + i ] = values[ f + i ];
		} // for
	} // if
	ENDPROC
}


class SliceValues : public DalChild<Slice>
{
public :
  SliceValues(
	      Slice * slice ) : DalChild<Slice>( slice ){ PROC ENDPROC }
  virtual ~SliceValues(){PROC ENDPROC}
  virtual bool8* boolValues()
    { PROC errstr << "Invalid type" << Sas::error( "InvalidType" ); RETURN (bool8*)(0); }
  virtual int8* int8Values()
    { PROC errstr << "Invalid type" << Sas::error( "InvalidType" ); RETURN (int8*)(0); }
  virtual uint16* uint16Values()
    { PROC errstr << "Invalid type" << Sas::error( "InvalidType" ); RETURN (uint16*)(0); }
  virtual int16* int16Values()
    { PROC errstr << "Invalid type" << Sas::error( "InvalidType" ); RETURN (int16*)(0); }
  virtual uint32* uint32Values()
    { PROC errstr << "Invalid type" << Sas::error( "InvalidType" ); RETURN (uint32*)(0); }
  virtual int32* int32Values()
    { PROC errstr << "Invalid type" << Sas::error( "InvalidType" ); RETURN (int32*)(0); }
  virtual real32* real32Values()
    { PROC errstr << "Invalid type" << Sas::error( "InvalidType" ); RETURN (real32*)(0); }
  virtual real64* real64Values()
    { PROC errstr << "Invalid type" << Sas::error( "InvalidType" ); RETURN (real64*)(0); }
  virtual char* stringValues()
    { PROC errstr << "Invalid type" << Sas::error( "InvalidType" ); RETURN (char*)(0); }
  virtual void copyBytes( Slice * slice ) = 0;
  virtual void copyBytes( Slice * slice, unsigned long from, unsigned long count ) = 0;
  virtual void copyBytes( SliceValues * sliceValues ) = 0;
  virtual void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count ) = 0;
  virtual void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to ) = 0;
  virtual void copyBoolBytes( bool8 * fromValues )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyBoolBytes( bool8 * fromValues, unsigned long, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyBoolBytes( bool8 * fromValues, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyInt8Bytes( int8 * fromValues )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyInt8Bytes( int8 * fromValues, unsigned long, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyInt8Bytes( int8 * fromValues, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyUint16Bytes( uint16 * fromValues )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyUint16Bytes( uint16 * fromValues, unsigned long, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyUint16Bytes( uint16 * fromValues, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyInt16Bytes( int16 * fromValues )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyInt16Bytes( int16 * fromValues, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyInt16Bytes( int16 * fromValues, unsigned long, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyUint32Bytes( uint32 * fromValues )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyUint32Bytes( uint32 * fromValues, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyUint32Bytes( uint32 * fromValues, unsigned long, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyInt32Bytes( int32 * fromValues )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyInt32Bytes( int32 * fromValues, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyInt32Bytes( int32 * fromValues, unsigned long, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyReal32Bytes( real32 * fromValues )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyReal32Bytes( real32 * fromValues, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyReal32Bytes( real32 * fromValues, unsigned long, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyReal64Bytes( real64 * fromValues )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyReal64Bytes( real64 * fromValues, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyReal64Bytes( real64 * fromValues, unsigned long, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyStringBytes( char * fromValues )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyStringBytes( char * fromValues, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyStringBytes( char * fromValues, unsigned long, unsigned long, unsigned long )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << fromValues; ENDPROC }
  virtual void copyBytes( unsigned long f, unsigned long t, unsigned long c )
    { PROC errstr << "SliceValue" << Sas::error( "NotImplemented") << f << t << c; ENDPROC }
};


// ************************************************************
// Since the compiler doesn't like s -> Child<Slicer>::parent()
// we define the following helper macros:
// ************************************************************
#define SV_TO_SLICER( v ) ( (((Child<Slicer>&)*v -> parent()).parent() ))
#define S_TO_S( s ) ( ((Child<Slice>&)*s).parent() )


class BoolSliceValues : public SliceValues
{
public :
  BoolSliceValues(
		  Slice * parent ) : SliceValues( parent ), _values(0) { PROC ENDPROC }
  ~BoolSliceValues() {
    PROC
      if( _values ) {
	SV_TO_SLICER( this ) -> parent() -> dumpData( _values, parent() -> from(), parent() -> count() );
	delete[] _values;
	_values = 0;
      } // if
    ENDPROC
      }
  bool8* boolValues() {
    PROC 
      if( !_values ) {
	_values = new bool8[ parent() -> count() ];
	SV_TO_SLICER( this ) -> parent() -> loadData( _values, parent() -> from(), parent() -> count() );
      } // if
    RETURN _values;
  }
  void copyBytes( Slice * slice ) {
    PROC
      if( _values ) {
	slice -> copyBoolBytes(  boolValues() );
      }
      else {
	slice -> copyBytes( this );
      } // if
    ENDPROC
      }
  void copyBytes( Slice * slice, unsigned long from, unsigned long count ) {
    PROC
      if( _values ) {
	slice -> copyBoolBytes(  boolValues(), from, count );
      }
      else {
	slice -> copyBytes( this, from, count );
      } // if
    ENDPROC
      }
  void copyBytes( SliceValues * sliceValues ) {
    PROC
      if( _values ) {
	sliceValues -> copyBoolBytes( boolValues() );
      }
      else {
	SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
      } // if
    ENDPROC
      } 
  void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count ) {
    PROC
      if( _values ) {
	sliceValues -> copyBoolBytes( boolValues(), from, count );
      }
      else {
	SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
      } // if
    ENDPROC
      }
  void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to ) {
    PROC
      if( _values ) {
	sliceValues -> copyBoolBytes( boolValues(), from, count, to );
      }
      else {
	SV_TO_SLICER( sliceValues ) -> parent() -> copyData( from, to, count );
      } // if
    ENDPROC
      }
  void copyBoolBytes( bool8 * fromValues ) {
    PROC
      bool8 * toValues = boolValues();
    for( unsigned long i = 0; i < parent() -> count(); ++i ) toValues[ i ] = fromValues[ i ];
    ENDPROC
      }
  virtual void copyBoolBytes( bool8 * fromValues, unsigned long from, unsigned long count ) {
    PROC
      bool8 * toValues = boolValues();
    for( unsigned long i = 0; i < count; ++i ) toValues[ i + parent() -> from() ] = fromValues[ i + from ];
    ENDPROC
      }
  void copyBoolBytes( bool8 * fromValues, unsigned long from, unsigned long count, unsigned long to ) {
    PROC
      bool8 * toValues = boolValues();
    for( unsigned long i = 0; i < count; ++i ) toValues[ i + to ] = fromValues[ i + from ];
    ENDPROC
      }
  void copyBytes( unsigned long f, unsigned long t, unsigned long c ) {
    PROC
      if( _values ) {
	::copyBytes( f, t, c, boolValues() );
      }
      else {
	SV_TO_SLICER( this ) -> parent() -> copyData( f, t, c );
      } // if
    ENDPROC
      }
private :
  bool8* 	_values;
};


class Int8SliceValues : public SliceValues
{
public :
  Int8SliceValues(
		  Slice * parent ) : SliceValues( parent ), _values(0) { PROC ENDPROC }
  ~Int8SliceValues() {
    PROC
      if( _values ) {
	SV_TO_SLICER( this ) -> parent() -> dumpData( _values, parent() -> from(), parent() -> count() );
	delete[] _values;
	_values = 0;
      } // if
    ENDPROC
      }
  int8* int8Values() {
    PROC 
      if( !_values ) {
	_values = new int8[ parent() -> count() ];
	SV_TO_SLICER( this ) -> parent() -> loadData( _values, parent() -> from(), parent() -> count() );
      } // if
    RETURN _values;
  }
  void copyBytes( Slice * slice ) {
    PROC
      if( _values ) {
	slice -> copyInt8Bytes(  int8Values() );
      }
      else {
	slice -> copyBytes( this );
      } // if
    ENDPROC
      }
  void copyBytes( Slice * slice, unsigned long from, unsigned long count ) {
    PROC
      if( _values ) {
	slice -> copyInt8Bytes(  int8Values(), from, count );
      }
      else {
	slice -> copyBytes( this, from, count );
      } // if
    ENDPROC
      }
  void copyBytes( SliceValues * sliceValues ) {
    PROC
      if( _values ) {
	sliceValues -> copyInt8Bytes( int8Values() );
      }
      else {
	SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
      } // if
    ENDPROC
      }
  void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count ) {
    PROC
      if( _values ) {
	sliceValues -> copyInt8Bytes( int8Values(), from, count );
      }
      else {
	SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
      } // if
    ENDPROC
      }
  void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to ) {
    PROC
      if( _values ) {
	sliceValues -> copyInt8Bytes( int8Values(), from, count, to );
      }
      else {
	SV_TO_SLICER( sliceValues ) -> parent() -> copyData( from, to, count );
      } // if
    ENDPROC
      }
  void copyInt8Bytes( int8 * fromValues ) {
    PROC
      int8 * toValues = int8Values();
    for( unsigned int i = 0; i < parent() -> count(); ++i ) toValues[ i ] = fromValues[ i ];
    ENDPROC
      }
  void copyInt8Bytes( int8 * fromValues, unsigned long from, unsigned long count ) {
    PROC
      int8 * toValues = int8Values();
    for( unsigned long i = 0; i < count; ++i ) toValues[ i + parent() -> from() ] = fromValues[ i + from ];
    ENDPROC
      }
  void copyInt8Bytes( int8 * fromValues, unsigned long from, unsigned long count, unsigned long to ) {
    PROC
      int8 * toValues = int8Values();
    for( unsigned long i = 0; i < count; ++i ) toValues[ i + to ] = fromValues[ i + from ];
    ENDPROC
      }
  void copyBytes( unsigned long f, unsigned long t, unsigned long c ) {
    PROC
      if( _values ) {
	::copyBytes( f, t, c, int8Values() );
      }
      else {
	SV_TO_SLICER( this ) -> parent() -> copyData( f, t, c );
      } // if
    ENDPROC
      }
  
private :
  int8* 	_values;
};


class Uint16SliceValues : public SliceValues
{
	public :
		Uint16SliceValues(
			Slice * parent ) : SliceValues( parent ), _values(0) { PROC ENDPROC }
		~Uint16SliceValues() {
		  PROC
		  if( _values ) {
		    SV_TO_SLICER( this ) -> parent() -> dumpData( _values, parent() -> from(), parent() -> count() );
		    delete[] _values;
		    _values = 0;
		  } // if
		  ENDPROC
		}
		uint16* uint16Values() {
		  PROC 
		  if( !_values ) {
		    _values = new uint16[ parent() -> count() ];
		    SV_TO_SLICER( this ) -> parent() -> loadData( _values, parent() -> from(), parent() -> count() );
		  } // if
		  RETURN _values;
		}
		void copyBytes( Slice * slice ) {
		  PROC
		  if( _values ) {
		    slice -> copyUint16Bytes(  uint16Values() );
		  }
		  else {
		    slice -> copyBytes( this );
		  } // if
		  ENDPROC
		}
		void copyBytes( Slice * slice, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    slice -> copyUint16Bytes(  uint16Values(), from, count );
		  }
		  else {
		    slice -> copyBytes( this, from, count );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyUint16Bytes( uint16Values() );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		} 
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyUint16Bytes( uint16Values(), from, count );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyUint16Bytes( uint16Values(), from, count, to );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( from, to, count );
		  } // if
		  ENDPROC
		}
		void copyUint16Bytes( uint16 * fromValues ) {
		  PROC
		  uint16 * toValues = uint16Values();
		  for( unsigned int i = 0; i < parent() -> count(); ++i ) toValues[ i ] = fromValues[ i ];
		  ENDPROC
		}
		void copyUint16Bytes( uint16 * fromValues, unsigned long from, unsigned long count ) {
		  PROC
		  uint16 * toValues = uint16Values();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + parent() -> from() ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyUint16Bytes( uint16 * fromValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  uint16 * toValues = uint16Values();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + to ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyBytes( unsigned long f, unsigned long t, unsigned long c ) {
		  PROC
		  if( _values ) {
		    ::copyBytes( f, t, c, uint16Values() );
		  }
		  else {
		    SV_TO_SLICER( this ) -> parent() -> copyData( f, t, c );
		  } // if
		  ENDPROC
		}


	private :
		uint16*	_values;
};

 
class Int16SliceValues : public SliceValues
{
	public :
		Int16SliceValues(
			Slice * parent ) : SliceValues( parent ), _values(0) { PROC ENDPROC }
		~Int16SliceValues() {
		  PROC
		  if( _values ) {
		    SV_TO_SLICER( this ) -> parent() -> dumpData( _values, parent() -> from(), parent() -> count() );
		    delete[] _values;
		    _values = 0;
		  } // if
		  ENDPROC
		}
		int16* int16Values() {
		  PROC 
		  if( !_values ) {
		    _values = new int16[ parent() -> count() ];
		    SV_TO_SLICER( this ) -> parent() -> loadData( _values, parent() -> from(), parent() -> count() );
		  } // if
		  RETURN _values;
		}
		void copyBytes( Slice * slice ) {
		  PROC
		  if( _values ) {
		    slice -> copyInt16Bytes(  int16Values() );
		  }
		  else {
		    slice -> copyBytes( this );
		  } // if
		  ENDPROC
		}
		void copyBytes( Slice * slice, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    slice -> copyInt16Bytes(  int16Values(), from, count );
		  }
		  else {
		    slice -> copyBytes( this, from, count );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyInt16Bytes( int16Values() );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		} 
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyInt16Bytes( int16Values(), from, count );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyInt16Bytes( int16Values(), from, count, to );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( from, to, count );
		  } // if
		  ENDPROC
		}
		void copyInt16Bytes( int16 * fromValues ) {
		  PROC
		  int16 * toValues = int16Values();
		  for( unsigned int i = 0; i < parent() -> count(); ++i ) toValues[ i ] = fromValues[ i ];
		  ENDPROC
		}
		void copyInt16Bytes( int16 * fromValues, unsigned long from, unsigned long count ) {
		  PROC
		  int16 * toValues = int16Values();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + parent() -> from() ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyInt16Bytes( int16 * fromValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  int16 * toValues = int16Values();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + to ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyBytes( unsigned long f, unsigned long t, unsigned long c ) {
		  PROC
		  if( _values ) {
		    ::copyBytes( f, t, c, int16Values() );
		  }
		  else {
		    SV_TO_SLICER( this ) -> parent() -> copyData( f, t, c );
		  } // if
		  ENDPROC
		}


	private :
		int16* 	_values;
};


class Uint32SliceValues : public SliceValues
{
	public :
		Uint32SliceValues(
			Slice * parent ) : SliceValues( parent ), _values(0) { PROC ENDPROC }
		~Uint32SliceValues() {
		  PROC
		  if( _values ) {
		    SV_TO_SLICER( this ) -> parent() -> dumpData( _values, parent() -> from(), parent() -> count() );
		    delete[] _values;
		    _values = 0;
		  } // if
		  ENDPROC
		}
		uint32* uint32Values() {
		  PROC 
		  if( !_values ) {
		    _values = new uint32[ parent() -> count() ];
		    SV_TO_SLICER( this ) -> parent() -> loadData( _values, parent() -> from(), parent() -> count() );
		  } // if
		  RETURN _values;
		}
		void copyBytes( Slice * slice ) {
		  PROC
		  if( _values ) {
		    slice -> copyUint32Bytes(  uint32Values() );
		  }
		  else {
		    slice -> copyBytes( this );
		  } // if
		  ENDPROC
		}
		void copyBytes( Slice * slice, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    slice -> copyUint32Bytes( uint32Values(), from, count );
		  }
		  else {
		    slice -> copyBytes( this, from, count );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyUint32Bytes( uint32Values() );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		} 
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyUint32Bytes( uint32Values(), from, count );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyUint32Bytes( uint32Values(), from, count, to );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( from, to, count );
		  } // if
		  ENDPROC
		}
		void copyUint32Bytes( uint32 * fromValues ) {
		  PROC
		  uint32 * toValues = uint32Values();
		  for( unsigned int i = 0; i < parent() -> count(); ++i ) toValues[ i ] = fromValues[ i ];
		  ENDPROC
		}
		void copyUint32Bytes( uint32 * fromValues, unsigned long from, unsigned long count ) {
		  PROC
		  uint32 * toValues = uint32Values();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + parent() -> from() ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyInt16Bytes( int16 * fromValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  int16 * toValues = int16Values();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + to ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyBytes( unsigned long f, unsigned long t, unsigned long c ) {
		  PROC
		  if( _values ) {
		    ::copyBytes( f, t, c, uint32Values() );
		  }
		  else {
		    SV_TO_SLICER( this ) -> parent() -> copyData( f, t, c );
		  } // if
		  ENDPROC
		}

	private :
		uint32* 	_values;
};


class Int32SliceValues : public SliceValues
{
	public :
		Int32SliceValues(
			Slice * parent ) : SliceValues( parent ), _values(0) { PROC ENDPROC }
		~Int32SliceValues() {
		  PROC
		  if( _values ) {
		    SV_TO_SLICER( this ) -> parent() -> dumpData( _values, parent() -> from(), parent() -> count() );
		    delete[] _values;
		    _values = 0;
		  } // if
		  ENDPROC
		}
		int32* int32Values() {
		  PROC 
		  if( !_values ) {
		    _values = new int32[ parent() -> count() ];
		    SV_TO_SLICER( this ) -> parent() -> loadData( _values, parent() -> from(), parent() -> count() );
		  } // if
		  RETURN _values;
		}
		void copyBytes( Slice * slice ) {
		  PROC
		  if( _values ) {
		    slice -> copyInt32Bytes(  int32Values() );
		  }
		  else {
		    slice -> copyBytes( this );
		  } // if
		  ENDPROC
		}
		void copyBytes( Slice * slice, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    slice -> copyInt32Bytes( int32Values(), from, count );
		  }
		  else {
		    slice -> copyBytes( this, from, count );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyInt32Bytes( int32Values() );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		} 
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyInt32Bytes( int32Values(), from, count );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyInt32Bytes( int32Values(), from, count, to );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( from, to, count );
		  } // if
		  ENDPROC
		}
		void copyInt32Bytes( int32 * fromValues ) {
		  PROC
		  int32 * toValues = int32Values();
		  for( unsigned int i = 0; i < parent() -> count(); ++i ) toValues[ i ] = fromValues[ i ];
		  ENDPROC
		}
		void copyInt32Bytes( int32 * fromValues, unsigned long from, unsigned long count ) {
		  PROC
		  int32 * toValues = int32Values();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + parent() -> from() ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyInt32Bytes( int32 * fromValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  int32 * toValues = int32Values();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + to ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyBytes( unsigned long f, unsigned long t, unsigned long c ) {
		  PROC
		  if( _values ) {
		    ::copyBytes( f, t, c, int32Values() );
		  }
		  else {
		    SV_TO_SLICER( this ) -> parent() -> copyData( f, t, c );
		  } // if
		  ENDPROC
		}

	private :
		int32* 	_values;
};


class Real32SliceValues : public SliceValues
{
	public :
		Real32SliceValues(
			Slice * parent ) : SliceValues( parent ), _values(0) { PROC ENDPROC }
		~Real32SliceValues() {
		  PROC
		  if( _values ) {
		    SV_TO_SLICER( this ) -> parent() -> dumpData( _values, parent() -> from(), parent() -> count() );
		    delete[] _values;
		    _values = 0;
		  } // if
		  ENDPROC
		}
		real32* real32Values() {
		  PROC 
		  if( !_values ) {
		    _values = new real32[ parent() -> count() ];
		    SV_TO_SLICER( this ) -> parent() -> loadData( _values, parent() -> from(), parent() -> count() );
		  } // if
		  RETURN _values;
		}
		void copyBytes( Slice * slice ) {
		  PROC
		  if( _values ) {
		    slice -> copyReal32Bytes(  real32Values() );
		  }
		  else {
		    slice -> copyBytes( this );
		  } // if
		  ENDPROC
		}
		void copyBytes( Slice * slice, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    slice -> copyReal32Bytes( real32Values(), from, count );
		  }
		  else {
		    slice -> copyBytes( this, from, count );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyReal32Bytes( real32Values() );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		} 
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyReal32Bytes( real32Values(), from, count );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyReal32Bytes( real32Values(), from, count, to );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( from, to, count );
		  } // if
		  ENDPROC
		}
		void copyReal32Bytes( real32 * fromValues ) {
		  PROC
		  real32 * toValues = real32Values();
		  for( unsigned int i = 0; i < parent() -> count(); ++i ) toValues[ i ] = fromValues[ i ];
		  ENDPROC
		}
		void copyReal32Bytes( real32 * fromValues, unsigned long from, unsigned long count ) {
		  PROC
		  real32 * toValues = real32Values();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + parent() -> from() ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyReal32Bytes( real32 * fromValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  real32 * toValues = real32Values();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + to ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyBytes( unsigned long f, unsigned long t, unsigned long c ) {
		  PROC
		  if( _values ) {
		    ::copyBytes( f, t, c, real32Values() );
		  }
		  else {
		    SV_TO_SLICER( this ) -> parent() -> copyData( f, t, c );
		  } // if
		  ENDPROC
		}


	private :
		real32*	_values;
};


class Real64SliceValues : public SliceValues
{
	public :
		Real64SliceValues(
			Slice * parent ) : SliceValues( parent ), _values(0) { PROC ENDPROC }
		~Real64SliceValues() {
		  PROC
		  if( _values ) {
		    SV_TO_SLICER( this ) -> parent() -> dumpData( _values, parent() -> from(), parent() -> count() );
		    delete[] _values;
		    _values = 0;
		  } // if
		  ENDPROC
		}
		real64* real64Values() {
		  PROC 
		  if( !_values ) {
		    _values = new real64[ parent() -> count() ];
		    SV_TO_SLICER( this ) -> parent() -> loadData( _values, parent() -> from(), parent() -> count() );
		  } // if
		  RETURN _values;
		}
		void copyBytes( Slice * slice ) {
		  PROC
		  if( _values ) {
		    slice -> copyReal64Bytes(  real64Values() );
		  }
		  else {
		    slice -> copyBytes( this );
		  } // if
		  ENDPROC
		}
		void copyBytes( Slice * slice, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    slice -> copyReal64Bytes( real64Values(), from, count );
		  }
		  else {
		    slice -> copyBytes( this, from, count );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyReal64Bytes( real64Values() );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		} 
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyReal64Bytes( real64Values(), from, count );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyReal64Bytes( real64Values(), from, count, to );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( from, to, count );
		  } // if
		  ENDPROC
		}
		void copyReal64Bytes( real64 * fromValues ) {
		  PROC
		  real64 * toValues = real64Values();
		  for( unsigned int i = 0; i < parent() -> count(); ++i ) toValues[ i ] = fromValues[ i ];
		  ENDPROC
		}
		void copyReal64Bytes( real64 * fromValues, unsigned long from, unsigned long count ) {
		  PROC
		  real64 * toValues = real64Values();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + parent() -> from() ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyReal64Bytes( real64 * fromValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  real64 * toValues = real64Values();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + to ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyBytes( unsigned long f, unsigned long t, unsigned long c ) {
		  PROC
		  if( _values ) {
		    ::copyBytes( f, t, c, real64Values() );
		  }
		  else {
		    SV_TO_SLICER( this ) -> parent() -> copyData( f, t, c );
		  } // if
		  ENDPROC
		}


	private :
		real64*	_values;
};


class StringSliceValues : public SliceValues
{
	public :
		StringSliceValues(
			Slice * parent ) : SliceValues( parent ), _values(0) { PROC ENDPROC }
		~StringSliceValues() {
		  PROC
		  if( _values ) {
		    SV_TO_SLICER( this ) -> parent() -> dumpData( &_values, parent() -> from(), parent() -> count() );
		    delete[] _values;
		    _values = 0;
		  } // if
		  ENDPROC
		}
		char* stringValues() {
		  PROC 
		  if( !_values ) {
		    _values = new char[ parent() -> count() ];
		    SV_TO_SLICER( this ) -> parent() -> loadData( &_values, parent() -> from(), parent() -> count() );
		  } // if
		  RETURN _values;
		}
		void copyBytes( Slice * slice ) {
		  PROC
		  slice -> copyStringBytes(  stringValues() );
		  if( _values ) {
		    slice -> copyStringBytes(  stringValues() );
		  }
		  else {
		    slice -> copyBytes( this );
		  } // if
		  ENDPROC
		}
		void copyBytes( Slice * slice, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    slice -> copyStringBytes( stringValues(), from, count );
		  }
		  else {
		    slice -> copyBytes( this, from, count );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyStringBytes( stringValues() );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		} 
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyStringBytes( stringValues(), from, count );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( sliceValues -> parent() -> from(), parent() -> from(), parent() -> count() );
		  } // if
		  ENDPROC
		}
		void copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  if( _values ) {
		    sliceValues -> copyStringBytes( stringValues(), from, count, to );
		  }
		  else {
		    SV_TO_SLICER( sliceValues ) -> parent() -> copyData( from, to, count );
		  } // if
		  ENDPROC
		}
		void copyStringBytes( char * fromValues ) {
		  PROC
		  char * toValues = stringValues();
		  for( unsigned int i = 0; i < parent() -> count(); ++i ) toValues[ i ] = fromValues[ i ];
		  ENDPROC
		}
		void copyStringBytes( char * fromValues, unsigned long from, unsigned long count ) {
		  PROC
		  char * toValues = stringValues();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + parent() -> from() ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyStringBytes( char * fromValues, unsigned long from, unsigned long count, unsigned long to ) {
		  PROC
		  char * toValues = stringValues();
		  for( unsigned long i = 0; i < count; ++i ) toValues[ i + to ] = fromValues[ i + from ];
		  ENDPROC
		}
		void copyBytes( unsigned long f, unsigned long t, unsigned long c ) {
		  PROC
		  if( _values ) {
		    ::copyBytes( f, t, c, stringValues() );
		  }
		  else {
		    SV_TO_SLICER( this ) -> parent() -> copyData( f, t, c );
		  } // if
		  ENDPROC
		}


	private :
		char*	_values;
};


// ------------------------- CLASS Slice here -----------------------------//

Slice::Slice() : _refs( 0 )
{
	PROC
	ENDPROC
}


void
Slice::incRefs()
{
	PROC
	_refs++;
	ENDPROC
}


void
Slice::decRefs()
{
	PROC
	_refs--;
	ENDPROC
}


unsigned long
Slice::refs() const
{
	PROC
	RETURN _refs;
}


// ------------------------- CLASS SuperSlice here ---------------------------//


class SuperSlice : public virtual Slice, public DalChild<Slicer>, public DalChild<Slice>, public DalSeekable<unsigned long>
{

	public :
		SuperSlice(
			unsigned long	from,
			unsigned long	count,
			Slicer *	parent );

		~SuperSlice();
		SliceType	type() const;
	    	int8*   	int8Values();
    		uint16*  	uint16Values();
    		int16*  	int16Values();
    		uint32*  	uint32Values();
    		int32*  	int32Values();
    		real32* 	real32Values();
    		real64* 	real64Values();
    		bool8*  	boolValues();
		char*   	stringValues();
		void		copyBytes( Slice * slice );
		void 		copyBytes( Slice * slice, unsigned long from, unsigned long count );
		void		copyBytes( SliceValues * sliceValues );
		void		copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count );
		void 		copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to );
		void		copyBytes( unsigned long from, unsigned long to, unsigned long count );
		void		copyBoolBytes( bool8 * values );
		void 		copyBoolBytes( bool8 * values, unsigned long from, unsigned long count );
		void		copyInt8Bytes( int8 * values );
		void 		copyInt8Bytes( int8 * values, unsigned long from, unsigned long count );
		void		copyUint16Bytes( uint16 * values );
		void 		copyUint16Bytes( uint16 * values, unsigned long from, unsigned long count );
		void		copyInt16Bytes( int16 * values );
		void 		copyInt16Bytes( int16* values, unsigned long from, unsigned long count );
		void		copyUint32Bytes( uint32 * values );
		void 		copyUint32Bytes( uint32 * values, unsigned long from, unsigned long count );
		void		copyInt32Bytes( int32 * values );
		void 		copyInt32Bytes( int32 * values, unsigned long from, unsigned long count );
		void		copyReal32Bytes( real32 * values );
		void 		copyReal32Bytes( real32 * values, unsigned long from, unsigned long count );
		void		copyReal64Bytes( real64 * values );
		void 		copyReal64Bytes( real64 * values, unsigned long from, unsigned long count );
		void		copyStringBytes( char * values );
		void 		copyStringBytes( char * values, unsigned long from, unsigned long count );

	private :

		// Forbid
		SuperSlice& operator=( const Slicer & );
		SuperSlice( const Slice &);

		SliceValues *	_values;

};


Slice::SliceType
SuperSlice::type() const
{
	PROC
	RETURN Slice::Super;
}


SuperSlice::SuperSlice(
	unsigned long	from,
	unsigned long	count,
	Slicer *	parent )
	:	DalChild<Slicer>( parent ),
		DalChild<Slice>( 0 ),
		DalSeekable<unsigned long>( from, count )
{
	PROC
	switch( parent -> parent() -> dataType() )
	{
		case Dal::Bool :
			_values  = new BoolSliceValues( this );
			break;
		case Dal::Int8 :
			_values = new Int8SliceValues( this );
			break;
		case Dal::Uint16 :
			_values = new Uint16SliceValues( this );
			break;
		case Dal::Int16 :
			_values = new Int16SliceValues( this );
			break;
		case Dal::Uint32 :
			_values = new Uint32SliceValues( this );
			break;
		case Dal::Int32 :
			_values = new Int32SliceValues( this );
			break;
		case Dal::Real32 :
			_values = new Real32SliceValues( this );
			break;
		case Dal::Real64 :
			_values = new Real64SliceValues( this );
			break;
		case Dal::DString :
			_values = new StringSliceValues( this );
			break;
		default :
			errstr << "Invalid Data Type" << Sas::error( "InvalidDataType" );
			break;
	} // switch
	ENDPROC
}


SuperSlice::~SuperSlice()
{
	PROC
	delete _values;
	ENDPROC
}


int8*
SuperSlice::int8Values()
{
	PROC
	RETURN _values -> int8Values();
}


uint16*
SuperSlice::uint16Values()
{
	PROC
	RETURN _values -> uint16Values();
}


int16*
SuperSlice::int16Values()
{
	PROC
	RETURN _values -> int16Values();
}


uint32*
SuperSlice::uint32Values()
{
	PROC
	RETURN _values -> uint32Values();
}


int32*
SuperSlice::int32Values()
{
	PROC
	RETURN _values -> int32Values();
}


real32*
SuperSlice::real32Values()
{
	PROC
	RETURN _values -> real32Values();
}


real64*
SuperSlice::real64Values()
{
	PROC
	RETURN _values -> real64Values();
}


bool8*
SuperSlice::boolValues()
{
	PROC
	RETURN _values -> boolValues();
}


char*
SuperSlice::stringValues()
{
	PROC
	RETURN _values -> stringValues();
}


void
SuperSlice::copyBytes( Slice * slice )
{
	PROC
	_values -> copyBytes( slice );
	ENDPROC
}


void
SuperSlice::copyBytes(
	Slice *		slice,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	_values -> copyBytes( slice, from, count );
	ENDPROC
}


void
SuperSlice::copyBytes( SliceValues * sliceValues )
{
	PROC
	_values -> copyBytes( sliceValues );
	ENDPROC
}


void
SuperSlice::copyBytes(
	SliceValues *	sliceValues,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	_values -> copyBytes( sliceValues, from, count );
	ENDPROC
}


void
SuperSlice::copyBytes(
	SliceValues * sliceValues,
	unsigned long from,
	unsigned long count, 
	unsigned long to )
{
	PROC
	_values -> copyBytes( sliceValues, from, count, to );
	ENDPROC
}


void
SuperSlice::copyBytes(
	unsigned long	from,
	unsigned long	to,
	unsigned long	count )
{
	PROC
	_values -> copyBytes( from, to, count );
	ENDPROC
}


void
SuperSlice::copyBoolBytes( bool8 * values )
{
	PROC
	_values -> copyBoolBytes( values );
	ENDPROC
}


void
SuperSlice::copyBoolBytes(
	bool8 *		values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	_values -> copyBoolBytes( values, from, count );
	ENDPROC
}


void
SuperSlice::copyInt8Bytes( int8 * values )
{
	PROC
	_values -> copyInt8Bytes( values );
	ENDPROC
}


void
SuperSlice::copyInt8Bytes(
	int8 *		values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	_values -> copyInt8Bytes( values, from, count );
	ENDPROC
}


void
SuperSlice::copyUint16Bytes( uint16 * values )
{
	PROC
	_values -> copyUint16Bytes( values );
	ENDPROC
}


void
SuperSlice::copyUint16Bytes(
	uint16 *	values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	_values -> copyUint16Bytes( values, from, count );
	ENDPROC
}


void
SuperSlice::copyInt16Bytes( int16 * values )
{
	PROC
	_values -> copyInt16Bytes( values );
	ENDPROC
}


void
SuperSlice::copyInt16Bytes(
	int16 *		values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	_values -> copyInt16Bytes( values, from, count );
	ENDPROC
}


void
SuperSlice::copyUint32Bytes( uint32 * values )
{
	PROC
	_values -> copyUint32Bytes( values );
	ENDPROC
}


void
SuperSlice::copyUint32Bytes(
	uint32 *	values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	_values -> copyUint32Bytes( values, from, count );
	ENDPROC
}


void
SuperSlice::copyInt32Bytes( int32 * values )
{
	PROC
	_values -> copyInt32Bytes( values );
	ENDPROC
}


void
SuperSlice::copyInt32Bytes(
	int32 *	values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	_values -> copyInt32Bytes( values, from, count );
	ENDPROC
}


void
SuperSlice::copyReal32Bytes( real32 * values )
{
	PROC
	_values -> copyReal32Bytes( values );
	ENDPROC
}


void
SuperSlice::copyReal32Bytes(
	real32 *	values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	_values -> copyReal32Bytes( values, from, count );
	ENDPROC
}


void
SuperSlice::copyReal64Bytes( real64 * values )
{
	PROC
	_values -> copyReal64Bytes( values );
	ENDPROC
}


void
SuperSlice::copyReal64Bytes(
	real64 *	values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	_values -> copyReal64Bytes( values, from, count );
	ENDPROC
}


void
SuperSlice::copyStringBytes( char * values )
{
	PROC
	_values -> copyStringBytes( values );
	ENDPROC
}


void
SuperSlice::copyStringBytes(
	char *		values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	_values -> copyStringBytes( values, from, count );
	ENDPROC
}


// ------------------------- CLASS SubSlice here -----------------------------//

class SubSlice : public virtual Slice, public DalChild<Slicer>, public DalChild<Slice>, public DalSeekable<unsigned long>
{

	public :
		SubSlice(
			unsigned long	from,
			unsigned long	count,
			Slice *		parentSlice );

		virtual ~SubSlice();
		SliceType	type() const;
	    	int8*   	int8Values();
    		uint16*  	uint16Values();
    		int16*  	int16Values();
    		uint32*  	uint32Values();
    		int32*  	int32Values();
    		real32* 	real32Values();
    		real64* 	real64Values();
    		bool8*  	boolValues();
		char*   	stringValues();
		void		copyBytes( Slice * slice );
		void 		copyBytes( Slice * slice, unsigned long from, unsigned long count );
		void		copyBytes( SliceValues * sliceValues );
		void		copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count );
		void 		copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to );
		void		copyBytes( unsigned long from, unsigned long to, unsigned long count );
		void		copyBoolBytes( bool8 * values );
		void 		copyBoolBytes( bool8 * values, unsigned long from, unsigned long count );
		void		copyInt8Bytes( int8 * values );
		void 		copyInt8Bytes( int8 * values, unsigned long from, unsigned long count );
		void		copyUint16Bytes( uint16 * values );
		void 		copyUint16Bytes( uint16 * values, unsigned long from, unsigned long count );
		void		copyInt16Bytes( int16 * values );
		void 		copyInt16Bytes( int16* values, unsigned long from, unsigned long count );
		void		copyUint32Bytes( uint32 * values );
		void 		copyUint32Bytes( uint32 * values, unsigned long from, unsigned long count );
		void		copyInt32Bytes( int32 * values );
		void 		copyInt32Bytes( int32 * values, unsigned long from, unsigned long count );
		void		copyReal32Bytes( real32 * values );
		void 		copyReal32Bytes( real32 * values, unsigned long from, unsigned long count );
		void		copyReal64Bytes( real64 * values );
		void 		copyReal64Bytes( real64 * values, unsigned long from, unsigned long count );
		void		copyStringBytes( char * values );
		void 		copyStringBytes( char * values, unsigned long from, unsigned long count );

	private :

		// Forbid
		SubSlice& operator=( const Slicer & );
		SubSlice( const Slice &);

};


SubSlice::SubSlice(
			unsigned long	from,
			unsigned long	count,
			Slice *		parentSlice )
		:
			DalChild<Slicer>( ((Child<Slicer>*)parentSlice) -> parent() ),
			DalChild<Slice>( parentSlice ),
			DalSeekable<unsigned long>( from, count )
{
	PROC
	ENDPROC
}


SubSlice::~SubSlice()
{
	PROC
	//Slice*	parentSlice = DalChild<Slice>::parent();
	//if( !parentSlice -> refs() )
	//{
	//	parentSlice -> incRefs();
	//	DalChild<Slicer>::parent() -> release( parentSlice -> from(), parentSlice -> count() );

	//}
	//else
	//{
	//	parentSlice -> decRefs();
	//} // if
	ENDPROC
}


Slice::SliceType
SubSlice::type() const
{
	PROC
	RETURN Slice::Sub;
}


int8*
SubSlice::int8Values()
{
	PROC
	const unsigned long offset =
			DalSeekable<unsigned long>::from() - DalChild<Slice>::parent() -> from();
	RETURN DalChild<Slice>::parent() -> int8Values() + offset;
}


uint16*
SubSlice::uint16Values()
{
	PROC
	const unsigned long offset =
			DalSeekable<unsigned long>::from() - DalChild<Slice>::parent() -> from();
	RETURN DalChild<Slice>::parent() -> uint16Values() + offset;
}


int16*
SubSlice::int16Values()
{
	PROC
	const unsigned long offset =
			DalSeekable<unsigned long>::from() - DalChild<Slice>::parent() -> from();
	RETURN DalChild<Slice>::parent() -> int16Values() + offset;
}


uint32*
SubSlice::uint32Values()
{
	PROC
	const unsigned long offset =
			DalSeekable<unsigned long>::from() - DalChild<Slice>::parent() -> from();
	RETURN DalChild<Slice>::parent() -> uint32Values() + offset;
}


int32*
SubSlice::int32Values()
{
	PROC
	const unsigned long offset =
			DalSeekable<unsigned long>::from() - DalChild<Slice>::parent() -> from();
	RETURN DalChild<Slice>::parent() -> int32Values() + offset;
}


real32*
SubSlice::real32Values()
{
	PROC
	const unsigned long offset =
			DalSeekable<unsigned long>::from() - DalChild<Slice>::parent() -> from();
	RETURN DalChild<Slice>::parent() -> real32Values() + offset;
}


real64*
SubSlice::real64Values()
{
	PROC
	const unsigned long offset =
			DalSeekable<unsigned long>::from() - DalChild<Slice>::parent() -> from();
	RETURN DalChild<Slice>::parent() -> real64Values() + offset;
}


bool8*
SubSlice::boolValues()
{
	PROC
	const unsigned long offset =
			DalSeekable<unsigned long>::from() - DalChild<Slice>::parent() -> from();
	RETURN DalChild<Slice>::parent() -> boolValues() + offset;
}


char*
SubSlice::stringValues()
{
	PROC
	const unsigned long offset =
			DalSeekable<unsigned long>::from() - DalChild<Slice>::parent() -> from();
	RETURN DalChild<Slice>::parent() -> stringValues() + offset;
}


void
SubSlice::copyBytes( Slice * slice )
{
	PROC
	DalChild<Slice>::parent() -> copyBytes( slice, from(), count() );
	ENDPROC
}


void
SubSlice::copyBytes(
	Slice *		slice,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	DalChild<Slice>::parent() -> copyBytes( slice, from, count );
	ENDPROC
}


void
SubSlice::copyBytes( SliceValues * sliceValues )
{
	PROC
	DalChild<Slice>::parent() -> copyBytes( sliceValues );
	ENDPROC
}


void
SubSlice::copyBytes(
	SliceValues *	sliceValues,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	// assert( this -> count() == count );
	DalChild<Slice>::parent() -> copyBytes( sliceValues, from, count, this -> from() );
	ENDPROC
}


void
SubSlice::copyBytes(
	SliceValues * sliceValues,
	unsigned long from,
	unsigned long count, 
	unsigned long to )
{
	PROC
	DalChild<Slice>::parent() -> copyBytes( sliceValues, from, count, to );
	ENDPROC
}


void
SubSlice::copyBytes(
	unsigned long	from,
	unsigned long	to,
	unsigned long	count )
{
	PROC
	DalChild<Slice>::parent() -> copyBytes( from, to, count );
	ENDPROC
}


void
SubSlice::copyBoolBytes( bool8 * values )
{
	PROC
	DalChild<Slice>::parent() -> copyBoolBytes( values );
	ENDPROC
}


void
SubSlice::copyBoolBytes(
	bool8 *		values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	DalChild<Slice>::parent() -> copyBoolBytes( values, from, count );
	ENDPROC
}


void
SubSlice::copyInt8Bytes( int8 * values )
{
	PROC
	DalChild<Slice>::parent() -> copyInt8Bytes( values );
	ENDPROC
}


void
SubSlice::copyInt8Bytes(
	int8 *		values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	DalChild<Slice>::parent() -> copyInt8Bytes( values, from, count );
	ENDPROC
}


void
SubSlice::copyUint16Bytes( uint16 * values )
{
	PROC
	DalChild<Slice>::parent() -> copyUint16Bytes( values );
	ENDPROC
}


void
SubSlice::copyUint16Bytes(
	uint16 *	values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	DalChild<Slice>::parent() -> copyUint16Bytes( values, from, count );
	ENDPROC
}


void
SubSlice::copyInt16Bytes( int16 * values )
{
	PROC
	DalChild<Slice>::parent() -> copyInt16Bytes( values );
	ENDPROC
}


void
SubSlice::copyInt16Bytes(
	int16 *		values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	DalChild<Slice>::parent() -> copyInt16Bytes( values, from, count );
	ENDPROC
}


void
SubSlice::copyUint32Bytes( uint32 * values )
{
	PROC
	DalChild<Slice>::parent() -> copyUint32Bytes( values );
	ENDPROC
}


void
SubSlice::copyUint32Bytes(
	uint32 *	values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	DalChild<Slice>::parent() -> copyUint32Bytes( values, from, count );
	ENDPROC
}


void
SubSlice::copyInt32Bytes( int32 * values )
{
	PROC
	DalChild<Slice>::parent() -> copyInt32Bytes( values );
	ENDPROC
}


void
SubSlice::copyInt32Bytes(
	int32 *		values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	DalChild<Slice>::parent() -> copyInt32Bytes( values, from, count );
	ENDPROC
}


void
SubSlice::copyReal32Bytes( real32 * values )
{
	PROC
	DalChild<Slice>::parent() -> copyReal32Bytes( values );
	ENDPROC
}


void
SubSlice::copyReal32Bytes(
	real32 *	values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	DalChild<Slice>::parent() -> copyReal32Bytes( values, from, count );
	ENDPROC
}


void
SubSlice::copyReal64Bytes( real64 * values )
{
	PROC
	DalChild<Slice>::parent() -> copyReal64Bytes( values );
	ENDPROC
}


void
SubSlice::copyReal64Bytes(
	real64 *	values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	DalChild<Slice>::parent() -> copyReal64Bytes( values, from, count );
	ENDPROC
}


void
SubSlice::copyStringBytes( char * values )
{
	PROC
	DalChild<Slice>::parent() -> copyStringBytes( values );
	ENDPROC
}


void
SubSlice::copyStringBytes(
	char *		values,
	unsigned long	from,
	unsigned long	count )
{
	PROC
	DalChild<Slice>::parent() -> copyStringBytes( values, from, count );
	ENDPROC
}




// ------------------------- CLASS MinSlice here -----------------------------//

class MinSlice : public virtual SubSlice
{

	public :
		MinSlice(
			unsigned long	from,
			unsigned long	count,
			Slice *		parentSlice );

		~MinSlice();
		SliceType	type() const;

	private :

		// Forbid
		MinSlice& operator=( const Slicer & );
		MinSlice( const Slice &);

};


MinSlice::MinSlice(
			unsigned long	from,
			unsigned long	count,
			Slice *		parentSlice )
		:
			SubSlice( from, count, parentSlice )
{
	PROC
	ENDPROC
}


MinSlice::~MinSlice()
{
	PROC
	Slice*	parentSlice = DalChild<Slice>::parent();
	Slicer* slicer =  DalChild<Slicer>::parent();

	slicer -> release( parentSlice -> from(), parentSlice -> count() );
	//if( parentSlice -> refs() == 1 )
	//{
	//	parentSlice -> incRefs();
	//	DalChild<Slicer>::parent() -> release( parentSlice -> from(), parentSlice -> count() );

	//}
	//else
	//{
	//	parentSlice -> decRefs();
	//} // if
	ENDPROC
}


Slice::SliceType
MinSlice::type() const
{
	PROC
	RETURN Slice::Min;
}


// ------------------------- CLASS Slicer here -------------------------------//

Slicer::Slicer( HiLowDataStorage * parent )
	: DalChild< HiLowDataStorage >( parent )
{
	PROC
	ENDPROC
}


Slicer::~Slicer()
{
	PROC
	if( _slices.size() > 0 )
	{
		errstr << "Some data has not been flushed to disk. It is unsafe to proceed " << Sas::error( "interrnalError" );
	} // if
	ENDPROC
}


void
Slicer::release(
	unsigned long	from,
	unsigned long	count )
{
	PROC

	const unsigned int numberOfSlicesToCheck = _slices.size();
	unsigned int s = 0;

	for( unsigned int i = 0; i < numberOfSlicesToCheck; ++i )
	{
		Slice* thisSlice = _slices[ s ];

		if(
			thisSlice -> from() == from
			&&
			thisSlice -> count() == count )
		{
			if( thisSlice -> refs() == 1 )
			{
				_slices.erase( _slices.begin() + s );
				delete thisSlice;
			}
			else
			{
				thisSlice -> decRefs();
			} // if
			break;
		}
		else
		{
			++s;
		} // if
	} // for

	ENDPROC
}


// **********************************************************************
// minSliceSize: This value establishes a partitioning of the
// memory into equally sized slices.
// The memory is arranged as a sequence of bytes b0,b1,b2,b3, ...
// Let N = minSliceSize
// The following partition is established:
// P0 = { b0, b1, b2, ... bN-1 }, P1 = { bN, bN+1, bN+1, ... b2N-1 }, ...
// The aim is to choose the minimum set of partitions
// Pi, Pi+1,  Pi + n, which together form a slice, which
// enclose a given slice [from,count]
// In the case if minSliceSize = 1, a partition is simply a single
// byte.
// **********************************************************************
static const unsigned long minSliceSize = getenv( "SAS_MIN_SLICE_SIZE" ) ?  atoi( getenv( "SAS_MIN_SLICE_SIZE" ) ) : 1;


// ********************************************************
// Get routine, calculates the minimum slice which contains
// the given slice [from,count].
// ********************************************************
DalSeekable<unsigned long>
Slicer::getSliceSize(
		const unsigned long 	from,
		unsigned long 		count ) const
{
	PROC
	const unsigned long f = from - from % minSliceSize;
	const unsigned long maxSliceSize = parent() -> size() - f;
	const unsigned long cc = ( from + count  - 1 ) - ( from + count - 1 ) % minSliceSize + minSliceSize - f;
	const unsigned long c = cc < maxSliceSize ? cc : maxSliceSize;
	RETURN DalSeekable<unsigned long>( f, c );
}


Slice *
Slicer::slice(
	unsigned long	from,
	unsigned long	count )
{
	PROC

	// *********************************
	// Search for the slice [from,count]
	// *********************************
	const unsigned int noSlices = _slices.size();

	// Do we already have the slice
	for( unsigned int i = 0; i < noSlices; ++i )
	{
		Slice*			slice = _slices[ i ];
		const unsigned long 	F = slice -> from();
		const unsigned long 	C = slice -> count();

		if( from == F && from + count == F + C )
		{
			//slice -> incRefs();

			ALTRETURNV( slice );
		}
	} // if

	// No, we dont already have it
	for( unsigned int i = 0; i < noSlices; ++i )
	{
		Slice*			slice = _slices[ i ];
		const unsigned long 	F = slice -> from();
		const unsigned long 	C = slice -> count();

		// Do we already have the slice or subslice
		//if( from == F && from + count == F + C )
		//{
			//slice -> incRefs();

		//	ALTRETURNV( slice );
		//}
		// ignore subslices
		if( slice -> type() == Slice::Sub )
		{
			continue;
		}
		// ignore minslices
		else if( slice -> type() == Slice::Min )
		{
			continue;
		}
		// Superslices only now
		// Are we requesting a new subslice
		// of an existing (super)slice.
		else if(
			( from > F && from + count <= F + C )
			||
			( from == F && from + count < F + C ))
		{
			// ???? slice -> incRefs();
			//slice -> incRefs();
			slice = new SubSlice( from, count, slice );
			_slices.push_back( slice );
			RETURN slice;
			//ALTRETURNV( new SubSlice( from, count, _slices[ i ] ));
		}
		// Check for Illegal slice
		else if(
			( from < F && from + count > F )
			||
			( from > F && from < (F + C) ) )
		{
			errstr << "seek[" << from << "," << count << "] is invalid since data corresponding to seek[" << slice -> from() << "," << slice -> count() << "] has already been loaded" << Sas::error( "InvalidSeek" );
		} // if
	} // for

	// *********************************************
	// A new slice is required. Create,
	// insert and return the new slice [from, count]
	// *********************************************
	//Slice * slice = new SuperSlice( from, count, this );
	//_slices.push_back( slice );
	//RETURN _slices.back();
	if( count == 0 )
	{
		ALTRETURNV( new SuperSlice( from, count, this ));
	} // if

	DalSeekable<unsigned long>	sliceSize = getSliceSize( from, count );

	Slice* slice = new SuperSlice( sliceSize.from(),sliceSize.count(), this );
	_slices.push_back( slice );

	if( from == sliceSize.from() && count == sliceSize.count() )
	{
		ALTRETURNV( slice );
	} // if
	slice -> incRefs();
	slice = new MinSlice( from, count, slice );
	_slices.push_back( slice );
	//ALTRETURNV( new SubSlice( from, count, slice ));
	RETURN _slices.back();
}


void
Slicer::insertBytes(
	unsigned long	,
	unsigned long	)
{ 
	PROC
	// Remove all slices.
	// TODO: Implement more efficiently.
	releaseAll();
	ENDPROC
}


void
Slicer::deleteBytes(
	unsigned long	,
	unsigned long	)
{
	PROC
	// Remove all slices.
	// TODO: Implement more efficiently.
	releaseAll();
	ENDPROC
}


void
Slicer::copyBytes(
	unsigned long	from,
	unsigned long	to,
	unsigned long	count )
{
	PROC

	if( from < to )
	{
		// Check for non overlapping segments
		if( from + count <= to )
		{
			// Segments don't overlap
			//Slice * f = initialise( from, count );
			//Slice * t = initialise( to, count );
			Slice * f = slice( from, count );
			f -> incRefs();
			Slice * t = slice( to, count );
			t -> incRefs();
			f -> copyBytes( t );
			release( from, count );
			release( to, count );
		}
		else
		{
			// Segments overlap
			//Slice * s = initialise( from, count + to - from );
			Slice * s = slice( from, count + to - from );
			s -> incRefs();
			s -> copyBytes( from, to, count );
			release( from, count + to - from );

		} // if
	}
	else if( from > to )
	{
		// Check for non overlapping segments
		if( to + count <= from )
		{
			// Segments don't overlap
			//Slice * f = initialise( from, count );
			//Slice * t = initialise( to, count );
			Slice * f = slice( from, count );
			f -> incRefs();
			Slice * t = slice( to, count );
			t -> incRefs();
			f -> copyBytes( t );
			release( from, count );
			release( to, count );
		}
		else
		{
			// Segments overlap
			//Slice * s = initialise( to, count + from - to );
			Slice * s = slice( to, count + from - to );
			s -> incRefs();
			s -> copyBytes( from, to, count );
			release( to, count + from - to );

		} // if
	} // if

	ENDPROC

}


void
Slicer::releaseAll()
{
	PROC
	while( _slices.size() > 0 )
	{
		delete _slices[ 0 ];
		_slices.erase( _slices.begin() );
	} // while
	ENDPROC
}


Slice*
Slicer::initialise(
	const Seekable<unsigned long>& oldSeek,
	const Seekable<unsigned long>& seek )
//Slicer::initialise( unsigned long from, unsigned long count )
{
	PROC
	// Disallow seek.count() == 0
	//assert( seek.count() != 0 );
	// Two cases: (i) construction of new data object
	// (ii) seek existing data object.
	// Check for case (i)
	Slice*	s;

	if( oldSeek.count() == 0 )
	{
		// Construct a new slice
		s = slice( seek.from(), seek.count() );
		s -> incRefs();
	}
	// Must be case (ii)
	else
	{
		// Get existing slice
		s = slice( oldSeek.from(), oldSeek.count() );
		assert( s -> refs() != 0 );
		assert( s -> count() != 0 );

		// check for subslice
		if( s -> type() == Slice::Sub )
		{
			release( s -> from(), s -> count() );
			s = slice( seek.from(), seek.count() );
			s -> incRefs();
		}
		// Check for superslice
		else if( s -> type() == Slice::Super )
		{
			// TODO: allow subslice of this superslice
			// to be constructed. This will probably
			// be a min slice !
			release( s -> from(), s -> count() );
			s = slice( seek.from(), seek.count() );
			s -> incRefs();
		}
		// Check for minslice
		else if( s -> type() == Slice::Min )
		{
			// Only allow the seek to be changed
			// on a data object, which has been referenced
			// exactly once.
			assert( s -> refs() == 1 );

			// Get the parent slice
			Slice* parentSlice = S_TO_S( s );

			const unsigned long F = parentSlice -> from();
			const unsigned long C = parentSlice -> count();
			const unsigned long from = seek.from();
			const unsigned long count = seek.count();

			// Check to see if the new seek is a subset of the
			// parent 
			if(
				( from > F && from + count <= F + C )
				||
				( from == F && from + count < F + C ))
			{
				s -> seek( from, count );
			}
			else
			{
				release( s -> from(), s -> count() );
				s = slice( seek.from(), seek.count() );
				s -> incRefs();
				//assert( s -> type() != Slice::Min );
			} // if
		}
		else
		{
			assert( 1 != 1 );
		} // if
	} // if
	//slice( from, count );
	//Slice * s = slice( from, count );
	//s -> incRefs();
	RETURN s;
}


