/*************************************************************** 
* 
* MemArray.h               & XMM Science Analysis System (SAS) 
* 
* Class representing block arrays.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: MemArray.h,v 1.1.1.1 2002/08/30 13:06:29 gvacanti Exp $ 
* 
*****************************************************************/ 
#ifndef MEM_ARRAY_H
#define MEM_ARRAY_H


#include <Dal.h>

#include "DalArray.h"
#include "DataStorage.h"
#include "DalSeekable.h"


class MemArray : public DalArray, public DalSeekable< longVec >
{
	public :

		// Constructors
		MemArray(
			const std::string &			name,
			DataType 			dataType,
			const longVec &			size,
			const std::string &			units,
			const std::string &			label,
			DalDataSet *			parent );

		MemArray(
			const Array & 	original,
			DalDataSet * 	parent,
			std::string 		newName = "");

	    virtual ~MemArray();
 
		MemArray& operator=( const Array& );

		unsigned long elements() const;
		unsigned int dataSize() const;
		//ArrayData * data(
		//		const longVec &		from=zero,
		//		const longVec &		to=zero,
		//		AccessMode		accessMode=AsParent) const;

		void scaleData(
				double			zero,
				double			scale,
				DataType		toType );

		void unscaleData(
				DataType		toType );

		void printOn( ostream & os ) const;

		//void release();

	private :

		void	setData( const MemArray &	usingDataInThisArray );

		DataStorage *			_data;

		ArrayData * makeArrayData(
				AccessMode		accessMode ) const;

};



#endif
