using namespace std;
/*************************************************************** 
* 
* MemArrayData.cc             & XMM Science Analysis System (SAS) 
* 
* Class representing array data object.
* 
* (c) 1997 ESA - Design by Marco Beijersbergen
*                Written by Mark Thomas, CARA
* 
* $Id: MemArrayData.cc,v 1.2 2002/09/06 14:57:37 gvacanti Exp $ 
* 
*****************************************************************/ 

#include <string>
#include <errstr.h>
#include "Tracer.h"
#include "MemArrayData.h"


static unsigned long
numberOfElements( const longVec & vec )
{
	PROC

	unsigned long	elements = 1;

	for( unsigned int i = 0; i < vec.size(); ++i ) elements *= vec[ i ];

	RETURN elements;
}



MemArrayData::MemArrayData(
			DataStorage *			data,
			const DalArray *		parent,
			const longVec&	from,
			const longVec&	count,
			AccessMode			accessMode )

	:		DalArrayData( parent, from, count ),
			MemData<DalArray>( data, 0, 0, parent, accessMode )
{
	PROC
	
	seek( from, count );

	ENDPROC
}


MemArrayData::~MemArrayData()
{
	PROC

	DalArray* array = this -> DalChild<DalArray>::parent();
	array -> removeDataObject( this );

	ENDPROC
}


unsigned long
MemArrayData::elements() const
{
	PROC
	RETURN this -> DalChild<Array>::parent() -> elements();
}


const longVec &
MemArrayData::size() const
{
	PROC
	RETURN this -> DalChild<Array>::parent() -> size();
}


void
MemArrayData::seek(
	const longVec & from,
	const longVec & count )
{
	PROC

	const Array * array = this -> DalChild<Array>::parent();

	const longVec& size = array->size();
	longVec _from;
	longVec _count;

	if( from.size() == 0 )
	{
		_from = size;
		for (unsigned i = 0; i < _from.size(); ++i)
			_from[i] = 0;
	}
	else if( from.size() == size.size() )
	{
		_from = from;
	}
	else
	{
		errstr << "could not access the data in array " << array->parent()->name() << "(" << array->name() << ") since the dimension " << from.size() << " of the starting point does not match the dimension " << array->dimensions() << " of the column" << Sas::error("invalidFromDimensions");

	} // if

	if ( count.size() == 0 )
	{
		_count = size;
	}
	else if ( count.size() == size.size() )
	{
		_count = count;
	}
	else
	{
		errstr << "could not access the data in array " << array->parent()->name() << "(" << array->name() << ") since the dimension " << count.size() << " of the range of interest does not match the dimension " << array->dimensions() << " of the column" << Sas::error("invalidCountDimensions");
	} // if

	for( unsigned i = 0; i < _from.size(); ++i )
	{
		if (_from[i] + _count[i] > size[i])
		{
			errstr << "could not access the data in array " << array->parent()->name() << "(" << array->name() << ") since the range of interest " << _from[i] << " .. " << _from[i] + _count[i] << " goes outside its size " << size[i] << Sas::error("invalidArrayDataRange");
		} // if
	} // for

	DalSeekable< longVec >::seek( _from, _count );

	seekable().seek(
			position( DalSeekable<longVec>::from() ),
			::numberOfElements( DalSeekable<longVec>::count() ));

	ENDPROC
}


