using namespace std;
#include <Dal.h>
#include <sstream>

int main()
{
  unsigned long nRows = 1000;

  DataSet* set = dataSetServer->open("test.dat",DataSet::Read);
  Table* tab = set->table("some table");
  assert( tab->rows()  == nRows );

  Column *col;
  ColumnData *d;

  col = tab->column("bool");
  assert( col->units().empty() );
  assert( col->label() == "bool data" );
  assert( col->rows() == nRows );
  d = col->data();
  assert( d->count()  == nRows );
  bool8 *b = d->boolData();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( b[i] == (i%2 ? true : false) );
  delete d;

  col = tab->column("int8");
  assert( col->units() == "mm" );
  assert( col->label() == "int8 data" );
  assert( col->rows() == nRows );
  d = col->data();
  assert( d->count()  == nRows );
  int8 *i8 = d->int8Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( i8[i] == int(i) % 100 );
  delete d;

  col = tab->column("uint16");
  assert( col->rows() == nRows );
  assert( col->units() == "cm" );
  assert( col->label() == "uint16 data" );
  d = col->data();
  assert( d->count()  == nRows );
  uint16 *ui16 = d->uint16Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( ui16[i] == 2*(unsigned int)(i) );
  delete d;

  col = tab->column("int16");
  assert( col->rows() == nRows );
  assert( col->units() == "cm" );
  assert( col->label() == "int16 data" );
  d = col->data();
  assert( d->count()  == nRows );
  int16 *i16 = d->int16Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( i16[i] == 2*int(i) );
  delete d;

  col = tab->column("uint32");
  assert( col->units() == "dm" );
  assert( col->rows() == nRows );
  assert( col->label() == "uint32 data" );
  d = col->data();
  assert( d->count()  == nRows );
  uint32 *ui32 = d->uint32Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( ui32[i] == 3*(unsigned int)(i) );
  delete d;

  col = tab->column("int32");
  assert( col->units() == "dm" );
  assert( col->rows() == nRows );
  assert( col->label() == "int32 data" );
  d = col->data();
  assert( d->count()  == nRows );
  int32 *i32 = d->int32Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( i32[i] == 3*int(i) );
  delete d;

  col = tab->column("real32");
  assert( col->units() == "m" );
  assert( col->label() == "real32 data" );
  assert( col->rows() == nRows );
  d = col->data();
  assert( d->count()  == nRows );
  real32 *r32 = d->real32Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( r32[i] == 0.5*i );
  delete d;

  col = tab->column("real64");
  assert( col->units() == "Dm" );
  assert( col->label() == "real64 data" );
  assert( col->rows() == nRows );
  d = col->data();
  assert( d->count()  == nRows );
  real64 *r64 = d->real64Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( r64[i] == 0.25*i );
  delete d;

  col = tab->column("string");
  assert( col->units().empty() );
  assert( col->label() == "string data" );
  assert( col->rows() == nRows );
  d = col->data();
  assert( d->count()  == nRows );
  char* s = d->stringData();
  int len = d->size()[0];
  assert( len == 10 );
  for (unsigned long i = 0; i < nRows; ++i) {
    stringstream ss;
    ss << "string"<<i;
    assert( string(s + d->position(i)) == string(ss.str()) );
  }
  delete d;

  dataSetServer->close(set);

  return 0;
}

