using namespace std;

#include <Dal.h>
#include "StopWatch.h"
#include <iostream>


static void
fill( DataSet* set, unsigned long nRows )
{

  Table* tab = set->addTable("some table",nRows);
  Column *col;
  ColumnData *d;
  uint32* ui32;

  col = tab->addColumn("uint32",Column::Uint32,"uint32 data","dm");

  for (unsigned long i = 0; i < nRows; ++i)
  {
    d = col->data(i,1);
    ui32 = d->uint32Data();
    ui32[ 0 ] = i;
    delete d;
    if( !( i % 100 ))
    {
    	cout << "fill[" << i << "]" << endl;
    } // if
  } // for
}


static void
check( DataSet* set, unsigned long nRows )
{
  Table* tab = set -> table("some table");
  Column *col;
  ColumnData *d;
  uint32* ui32;

  col = tab -> column("uint32");
  for (unsigned long i = 0; i < nRows; ++i)
  {
    d = col->data(i,1);
    ui32 = d->uint32Data();
    assert( ui32[0] == i );
    delete d;
    if( !( i % 100 ))
    {
    	cout << "check[" << i << "]" << endl;
    } // if
  } // for
}


static void
test( unsigned long nRows )
{
  // *********************************
  // Get clock for timing information.
  // *********************************
  StopWatch		cpuWatch( StopWatch::CPU );
  StopWatch		calenderWatch( StopWatch::Calender );

  DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
  fill( set, nRows );
  check( set, nRows );
  dataSetServer -> close( set );

  set = dataSetServer->open("test.dat",DataSet::Read);
  check( set, nRows );
  dataSetServer -> close( set );
  cout << "CPU time: " << cpuWatch.elapsed() << " secs" << endl;
  cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;
}


int
main()
{
  test( 10000 );
  //test( 1000000 );
}

