using namespace std;

#include <sstream>
#include <string>

#include <Dal.h>

int main()
{
  unsigned long nRows = 10;
  string setName = "test.dat";
  string tableName = "ATABLE";
  string columnName = "ACOLUMN";
  string attributeName = "TLMAX";
  Attribute * att;

  string name = setName + ":" + tableName + ":" + columnName + "%" + attributeName;
  assert( name == "test.dat:ATABLE:ACOLUMN%TLMAX" );

  string setAttribute = setName + "%ATT1";
  string tabAttribute = setName + ":" + tableName + "%ATT2";
  string colAttribute = setName + ":" + tableName + ":" + columnName + "%TLMIN";
  int attributeValue = 123;

  DataSet* set = dataSetServer->open(name,DataSet::Create);
  assert( set -> name() == setName );
  *( att = set -> addAttribute( setAttribute, "attribute", "km" )) = attributeValue;
  assert( att -> name() == "ATT1" );
  assert( att -> qualifiedName() == setAttribute );
  assert( set -> hasAttribute( "ATT1" ));
  assert( set -> hasAttribute( setAttribute ));
  assert( !set -> hasAttribute( setName + "%ATT2" ));

  Table* tab = set->addTable(name,nRows);
  assert( tab -> name() == tableName );
  *( att = tab -> addAttribute( tabAttribute, "attribute", "km" )) = attributeValue;
  assert( att -> name() == "ATT2" );
  assert( tab -> hasAttribute( "ATT2" ));
  assert( att -> qualifiedName() == tabAttribute );
  assert( tab -> hasAttribute( tabAttribute ));
  assert( !tab -> hasAttribute( setName + ":" + tableName + "%ATT1" ));

  Column * col = tab->addColumn(name,Column::Int32,"int data");
  assert( col -> name() == columnName );
  *( att = col -> addAttribute( colAttribute, "attribute", "km" )) = attributeValue;
  assert( att -> name() == "TLMIN" );
  assert( att -> qualifiedName() == colAttribute );
  assert( col -> hasAttribute( "TLMIN" ));
  assert( col -> hasAttribute( colAttribute ));
  assert( !col -> hasAttribute( setName + ":" + tableName + ":" + columnName + "%TLMAX" ));

  // Fill data.
  ColumnData * d = col->data();
  int32 *i32 = d->int32Data();
  for (unsigned long i = 0; i < nRows; ++i)
    i32[i] = 3*i;
  delete d;

  dataSetServer->close( set );

  set = dataSetServer->open(name,DataSet::Read);
  assert( set -> attribute( setAttribute ) -> name() == "ATT1" );
  assert( set -> attribute( setName + "%" ) -> name() == "ATT1" );

  assert( set -> hasBlock( name ) );
  tab = set->table(name);
  assert( tab -> attribute( tabAttribute ) -> name() == "ATT2" );
  assert( tab -> attribute( setName + ":" + tableName + "%" ) -> name() == "ATT2" );
  tab = set->table( setName + ":" );
  assert( tab -> name() == tableName );
  tab = set->table( setName + "::" );
  assert( tab -> name() == tableName );
  tab = set->table( setName + "::%stuff" );
  assert( tab -> name() == tableName );

  assert( tab -> hasColumn( name ) );
  col = tab->column(name);
  assert( col -> attribute( colAttribute ) -> name() == "TLMIN" );
  assert( col -> attribute( setName + ":" + tableName + ":" + columnName + "%" ) -> name() == "TLMIN" );
  col = tab->column( setName + ":" + tableName + ":" );
  assert( col -> name() == columnName );
  col = tab->column( setName + "::" );
  assert( col -> name() == columnName );
  col = tab->column( setName + "::%stuff" );
  assert( col -> name() == columnName );

  // Check data
  d = col->data();
  i32 = d->int32Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( i32[i] == (int32)(3*i) );
  delete d;

  dataSetServer->close( set );

  return 0;
}

