using namespace std;
#include <StopWatch.h>
#include <Dal.h>
#include <iostream>


struct Rows
{
	Rows() : _nRows( 0 ), _from( 0 ), _count( 0 ) {}
	unsigned long	_nRows;
	unsigned long	_from;
	unsigned long	_count;

};


static void
deleterows(
	DataSet*		set,
	const Rows&		deleteTheseRows )
{
	Table*			tab = set -> table("some table");
        const unsigned long	nRows = tab -> rows();

	tab -> deleteRows( deleteTheseRows._from, deleteTheseRows._count );
	assert( tab -> rows() == nRows - deleteTheseRows._count );
}


template<class T>
static void
fill(
	DataSet*		set,
	unsigned long		nRows,
	Dal::DataType		dataType,
	T* (Data::*tdata)() const )
{

	Table*		tab = set -> addTable( "some table", nRows );
	Column*		col;
	Data*		cd;
	T*		d;

	col = tab -> addColumn( "col", dataType, "data", "dm" );
	cd = col -> data();
	d = (cd ->* tdata)();

	for( unsigned long i = 0; i < nRows; ++i )
	{
		d[ i ] = i;
	} // for
	delete cd;
}


template<class T>
static void
check(
	DataSet*		set,
	const Rows&		deleteTheseRows,
	T* (Data::*tdata)() const )
{

	Table*			tab = set -> table( "some table" );
	const unsigned long 	nRows = tab -> rows();
	Column*			col;
	Data*			cd;
	T*			d;

	col = tab -> column( "col" );
	cd = col -> data();
	d = (cd ->* tdata)();

	for( unsigned long i = 0, j  = 0; i < nRows; ++i )
	{
		if( i == deleteTheseRows._from )
		{
			j += deleteTheseRows._count;
		} // if
		assert( d[i] == T(j) );
		++j;
	} // for
	delete cd;
}


template<class T>
static void
test(
	unsigned long		nRows,
	const Rows& 		deleteTheseRows,
	Dal::DataType		dataType,
	T* (Data::*tdata)() const )
{
  Rows			none;

  DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
  fill( set, nRows, dataType, tdata );
  check( set, none, tdata );
  deleterows( set, deleteTheseRows );
  check( set, deleteTheseRows, tdata );
  dataSetServer -> close( set );

  set = dataSetServer->open("test.dat",DataSet::Read);
  check( set, deleteTheseRows, tdata );
  dataSetServer -> close( set );

}


static void
doTest1( unsigned long nRows, const Rows& deleteTheseRows )
{

  test( nRows, deleteTheseRows, Dal::Int8, &Data::int8Data );
  test( nRows, deleteTheseRows, Dal::Uint16, &Data::uint16Data );
  test( nRows, deleteTheseRows, Dal::Int16, &Data::int16Data );
  test( nRows, deleteTheseRows, Dal::Uint32, &Data::uint32Data );
  test( nRows, deleteTheseRows, Dal::Int32, &Data::int32Data );
  test( nRows, deleteTheseRows, Dal::Real32, &Data::real32Data );
  test( nRows, deleteTheseRows, Dal::Real64, &Data::real64Data );
 

}



static void
doTest( unsigned long nRows )
{

  // *********************************
  // Get clock for timing information.
  // *********************************
  StopWatch		cpuWatch( StopWatch::CPU );
  StopWatch		calenderWatch( StopWatch::Calender );

  Rows	rows;

  rows._nRows = nRows;

  rows._from = 0;
  rows._count = 0;
  doTest1( rows._nRows, rows );

  rows._from = 0;
  rows._count = 1;
  doTest1( rows._nRows, rows );

  rows._from = nRows - 1;
  rows._count = 0;
  doTest1( rows._nRows, rows );

  rows._from = nRows - 1;
  rows._count = 1;
  doTest1( rows._nRows, rows );

  rows._from = 0;
  rows._count = nRows;
  doTest1( rows._nRows, rows );

  if( nRows > 3 )
  {
  	rows._from = nRows - 4;
  	rows._count = 4;
  	doTest1( rows._nRows, rows );

  	rows._from = 0;
  	rows._count = 4;
  } // if

  cout << "CPU time: " << cpuWatch.elapsed() << " secs" << endl;
  cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;
}


int
main()
{
  doTest( 1 );
  doTest( 10 );
  doTest( 100 );
  doTest( 2589 );

  //rows._nRows = 2589;
  //rows._from = 2585;
  //rows._count = 4;

  //test( 2589, rows, Dal::Int32, &Data::int32Data );
  //test( 2589, rows, Dal::Real32, &Data::real32Data );
  //test( 2589, rows, Dal::Real64, &Data::real64Data );

  //rows._nRows = 2589;
  //rows._from = 2585;
  //rows._count = 2;

  //test( 2589, rows, Dal::Int32, &Data::int32Data );
  //test( 2589, rows, Dal::Real32, &Data::real32Data );
  //test( 2589, rows, Dal::Real64, &Data::real64Data );
}

