using namespace std;
#include <string>
#include <Dal.h>
#include <errstr.h>
#include<sstream>

int main()
{
  unsigned long nRows = 10;
  vector<unsigned long> size(&nRows,&nRows+1);

  DataSet *set = dataSetServer->open("test.dat",DataSet::Modify);
  Table* tab = set->table("someTable");
  if (tab->rows() != nRows) Sas::error("testFailed","wrong nr of rows");

  Attribute* attr;
  attr = tab->attribute("sbool");
  assert( attr->asBool() );
  assert( attr->units().empty() );
  assert( attr->label() == "boolean attribute" );

  attr = tab->attribute("sint");
  assert( attr->asInt() == 3 );
  assert( attr->units() == "int unit" );
  assert( attr->label() == "int attribute" );

  attr = tab->attribute("sreal");
  assert( attr->asReal() == 2.5 );
  assert( attr->units() == "real unit" );
  assert( attr->label() == "real attribute" );


  attr = tab->attribute("sstring");
  assert( attr->asString() == "a string" );
  assert( attr->units().empty() );
  assert( attr->label() == "string attribute" );


  Column *col;
  ColumnData *data;

  col = tab->column("bool");
  assert( col->rows() == nRows );
  data = col->data();
  assert( data->count() == nRows );
  bool8* b = data->boolData();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( b[i] == ( i%2 ? true : false ) );
  delete data;

  col = tab->column("int8");
  assert( col->rows() == nRows );
  assert( col->units() == "mm" );
  data = col->data();
  assert( data->count() == nRows );
  int8* i8 = data->int8Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( i8[i] == int(i) );
  delete data;

  col = tab->column("int16");
  assert( col->rows() == nRows );
  assert( col->units() == "cm" );
  data = col->data();
  assert( data->count() == nRows );
  int16* i16 = data->int16Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( i16[i] == int(2*i) );
  delete data;

  col = tab->column("int32");
  assert( col->rows() == nRows );
  assert( col->units() == "dm" );
  data = col->data();
  assert( data->count() == nRows );
  int32* i32 = data->int32Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( i32[i] == int(3*i) );
  delete data;

  col = tab->column("real32");
  assert( col->rows() == nRows );
  assert( col->units() == "m" );
  data = col->data();
  assert( data->count() == nRows );
  real32* r32 = data->real32Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( r32[i] == 0.5*i );
  delete data;

  col = tab->column("real64");
  assert( col->rows() == nRows );
  assert( col->units() == "Dm" );
  data = col->data();
  assert( data->count() == nRows );
  real64* r64 = data->real64Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( r64[i] == 0.25*i );
  delete data;

  col = tab->column("string");
  assert( col->rows() == nRows );
  data = col->data();
  assert( data->count() == nRows );
  assert( data->size()[0] == 8 );
  char* s = data->stringData();
  for (unsigned long i = 0; i < nRows; ++i) {
    stringstream ss;
    ss << "string" << i;
    assert( string(s+data->position(i)) == ss.str() );
  }
  delete data;

  dataSetServer->close(set);

  return 0;
}

