using namespace std;


#include <Dal.h>
#include <sstream>


static void
renameColumns( Table * tab, const string& c1, const string& c2, const string& t1, const string& t2 )
{
  // table1
  assert( tab -> columns() == 2 );
  assert( tab -> hasColumn( c1 ) );
  assert( tab -> hasColumn( c2 ) );
  assert( !tab -> hasColumn( t1 ) );
  assert( !tab -> hasColumn( t2 ) );

  // rename c1 to t1
  Column * col = tab -> column( c1 );
  assert( col -> name() == c1 );
  col -> rename( t1 );
  assert( col -> name() == t1 );
  assert( tab -> columns() == 2 );
  assert( tab -> hasColumn( t1 ));
  assert( !tab -> hasColumn( c1 ));
  assert( tab -> hasColumn( c2 ));

  // rename c2 to t2
  col = tab -> column( c2 );
  assert( col -> name() == c2 );
  col -> rename( t2 );
  assert( col -> name() == t2 );
  assert( tab -> columns() == 2 );
  assert( !tab -> hasColumn( c2 ));
  assert( tab -> hasColumn( t1 ));
  assert( tab -> hasColumn( t2 ));

}


static void
renameTable( Table * tab, const string& t1, const string& c1, const string& c2 )
{
  assert( tab -> parent() -> hasBlock( tab -> name() ));
  assert( tab -> name() != t1 );
  assert( !tab -> parent() -> hasBlock( t1 ));
  assert( tab -> columns() == 2 );
  assert( tab -> hasColumn( c1 ) );
  assert( tab -> hasColumn( c2 ) );
  tab -> rename( t1 );
  assert( tab -> name() == t1 );
  assert( tab -> parent() -> hasBlock( tab -> name() ));
  assert( tab -> columns() == 2 );
  assert( tab -> hasColumn( c1 ) );
  assert( tab -> hasColumn( c2 ) );

}


static void
testTable( unsigned long nRows )
{
  // Create the set
  DataSet* set = dataSetServer->open("test.dat",DataSet::Create);

  // table1
  Table* tab = set->addTable("table1",nRows);
  Column *col = tab->addColumn("col1",Column::Int32,"int32 data","dm");
  col = tab->addColumn("col2",Column::Int32,"int32 data","dm");
  assert( tab -> columns() == 2 );
  assert( tab -> hasColumn( "col1" ) );
  assert( tab -> hasColumn( "col2" ) );

  // table2
  tab = set->addTable("table2",nRows);
  col = tab->addColumn("col1",Column::Int32,"int32 data","dm");
  col = tab->addColumn("col2",Column::Int32,"int32 data","dm");
  assert( tab -> columns() == 2 );
  assert( tab -> hasColumn( "col1" ));
  assert( tab -> hasColumn( "col2" ));

  assert( set -> blocks() == 2 );
  assert( set -> hasBlock( "table1" ));
  assert( set -> hasBlock( "table2" ));

  dataSetServer->close(set);

  // Modify the set
  set = dataSetServer->open("test.dat",DataSet::Modify);
  assert( set -> blocks() == 2 );
  assert( set -> hasBlock( "table1" ));
  assert( set -> hasBlock( "table2" ));

  // table1
  tab = set->table("table1");
  renameColumns( tab, "col1", "col2", "col3", "col4" );

  // table2
  tab = set->table("table2");
  renameColumns( tab, "col1", "col2", "col5", "col6" );

  // rename table1 to table3
  tab = set->table("table1");
  renameTable( tab, "table3", "col3", "col4" );

  // rename table2 to table4
  tab = set->table("table2");
  renameTable( tab, "table4", "col5", "col6" );

  assert( set -> blocks() == 2 );
  assert( set -> hasBlock( "table3" ));
  assert( set -> hasBlock( "table4" ));

  dataSetServer->close(set);

  // Modify the set
  set = dataSetServer->open("test.dat",DataSet::Modify);
  assert( set -> blocks() == 2 );
  assert( set -> hasBlock( "table3" ));
  assert( set -> hasBlock( "table4" ));

  // table1
  tab = set->table("table3");
  renameColumns( tab, "col3", "col4", "col7", "col8" );

  // table2
  tab = set->table("table4");
  renameColumns( tab, "col5", "col6", "col9", "col10" );

  // rename table3 to table5
  tab = set->table("table3");
  renameTable( tab, "table5", "col7", "col8" );

  // rename table4 to table6
  tab = set->table("table4");
  renameTable( tab, "table6", "col9", "col10" );

  assert( set -> blocks() == 2 );
  assert( set -> hasBlock( "table5" ));
  assert( set -> hasBlock( "table6" ));

  dataSetServer->close(set);

}


int
main()
{

  testTable( 1 );
  testTable( 2 );
  testTable( 100 );

  return 0;
}
