using namespace std;


#include <Dal.h>
#include <iostream>
#include <ctime>
#include "StopWatch.h"



static void
fill( Table * table )
{
	const unsigned long nRows = table -> rows();
	const unsigned long seekSize = nRows / 7;

	for( unsigned int k = 0; k < nRows; k += seekSize )
	{
		const unsigned long seekThisMany =
			(k + seekSize < nRows) ? seekSize : nRows - k;
		clog << "seek[ " << k << "," << seekThisMany << "]" << endl;

		for( unsigned int i = 0; i < table -> columns(); ++i )
		{
			Column * col = table -> column( i );
			ColumnData * coldat = col -> data();
			coldat -> seek( k, seekThisMany );

			switch( col -> dataType() )
			{
			case Column::Bool :
			{
				bool8 * d = coldat -> boolData();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					d[ j ] = ((coldat -> from() + j ) % 2 ? true : false );
				} // for
			}
				break;
			case Column::Int8 :
			{
				int8 * d = coldat -> int8Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					d[ j ] = coldat -> from() + j;
				} // for
			}
				break;
			case Column::Int16 :
			{
				int16 * d = coldat -> int16Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					d[ j ] = coldat -> from() + j;
				} // for
			}
				break;
			case Column::Int32 :
			{
				int32 * d = coldat -> int32Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					d[ j ] = coldat -> from() + j;
				} // for
			}
				break;
			case Column::Real32 :
			{
				real32 * d = coldat -> real32Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					d[ j ] = coldat -> from() + j;
				} // for
			}
				break;
			case Column::Real64 :
			{
				real64 * d = coldat -> real64Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					d[ j ] = coldat -> from() + j;
				} // for
			}
				break;
			case Column::DString :
			{
				char * d = coldat -> stringData();
				for( unsigned long r = 0; r < coldat -> count(); ++r )
				{
					const unsigned long	e = coldat -> parent() -> elements();
					for( unsigned long j = 0; j < e; ++j )
					{
						d[ r * e + j ] = ('a' + ( coldat -> from() + j ) % 26 );
					} // for
				} // for
			}
				break;
			default :
				break;
		} // switch
		delete coldat;
	} // for
	} // for
}


static void
check( Table * table )
{
	const unsigned long nRows = table -> rows();
	const unsigned long seekSize = nRows / 7;

	for( unsigned int k = 0; k < nRows; k += seekSize )
	{
		const unsigned long seekThisMany =
			(k + seekSize < nRows) ? seekSize : nRows - k;
		clog << "seek[ " << k << "," << seekThisMany << "]" << endl;

		for( unsigned int i = 0; i < table -> columns(); ++i )
		{
			Column * col = table -> column( i );
			ColumnData * coldat = col -> data();
			coldat -> seek( k, seekThisMany );

			switch( col -> dataType() )
			{
			case Column::Bool :
			{
				bool8 * booldat = coldat -> boolData();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( booldat[ j ] == ((coldat -> from()
					+ j ) % 2 ? true : false ));
				} // for
			}
				break;
			case Column::Int8 :
			{
				int8 * d = coldat -> int8Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == int8(coldat -> from()
					+ j) );
				} // for
			}
				break;
			case Column::Int16 :
			{
				int16 * d = coldat -> int16Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == int16(coldat ->
					from() + j) );
				} // for
			}
				break;
			case Column::Int32 :
			{
				int32 * d = coldat -> int32Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == int32(coldat ->
					from() + j) );
				} // for
			}
				break;
			case Column::Real32 :
			{
				real32 * d = coldat -> real32Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == coldat -> from() + j );
				} // for
			}
				break;
			case Column::Real64 :
			{
				real64 * d = coldat -> real64Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == coldat -> from() + j );
				} // for
			}
				break;
			case Column::DString :
			{
				char * d = coldat -> stringData();
				for( unsigned long r = 0; r < coldat -> count(); ++r )
				{
					const unsigned long	e = coldat -> parent() -> elements();
					for( unsigned long j = 0; j < e; ++j )
					{
						assert( d[ r * e + j ] = ('a' + ( coldat -> from() + j ) % 26 ) );
					} // for
				} // for
			}
				break;
			default :
				break;
		} // switch
		delete coldat;
	} // for
	} // for
}


static void
testSeek( Table * table )
{
	for( unsigned int i = 0; i < table -> columns(); ++i )
	{
		Column * col = table -> column( i );
		ColumnData * coldat = col -> data();
		coldat -> seek( 1, 2 );

		switch( col -> dataType() )
		{
			case Column::Bool :
			{
				bool8 * booldat = coldat -> boolData();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( booldat[ j ] == ((coldat -> from()
					+ j ) % 2 ? true : false ));
				} // for
			}
				break;
			case Column::Int8 :
			{
				int8 * d = coldat -> int8Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == int8(coldat -> from()
					+ j) );
				} // for
			}
				break;
			case Column::Int16 :
			{
				int16 * d = coldat -> int16Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == int16(coldat ->
					from() + j) );
				} // for
			}
				break;
			case Column::Int32 :
			{
				int32 * d = coldat -> int32Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == int32(coldat ->
					from() + j) );
				} // for
			}
				break;
			case Column::Real32 :
			{
				real32 * d = coldat -> real32Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == coldat -> from() + j );
				} // for
			}
				break;
			case Column::Real64 :
			{
				real64 * d = coldat -> real64Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == coldat -> from() + j );
				} // for
			}
				break;
			case Column::DString :
			{
				char * d = coldat -> stringData();
				for( unsigned long r = 0; r < coldat -> count(); ++r )
				{
					const unsigned long	e = coldat -> parent() -> elements();
					for( unsigned long j = 0; j < e; ++j )
					{
						assert( d[ r * e + j ] = ('a' + ( coldat -> from() + j ) % 26 ));
					} // for
				} // for
			}
				break;
			default :
				break;
		} // switch
		delete coldat;
	} // for
}


int main()
{
	// *********************************
	// Get clock for timing information.
	// *********************************
	StopWatch		cpuWatch( StopWatch::CPU );
	StopWatch		calenderWatch( StopWatch::Calender );

  	unsigned long nRows = 10;

	// Create, fill & check
  	DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
  	Table* tab = set->addTable("some table",nRows);
 	Column* col1; col1 = tab->addColumn("bool",Column::Bool,"bool data");
 	Column* col2; col2 = tab->addColumn("int8",Column::Int8,"int8 data");
 	Column* col3; col3 = tab->addColumn("int16",Column::Int16,"int16 data");
 	Column* col4; col4 = tab->addColumn("int32",Column::Int32,"int32 data");
 	Column* col5; col5 = tab->addColumn("real32",Column::Real32,"real32 data");
 	Column* col6; col6 = tab->addColumn("real64",Column::Real64,"Real64 data");
	vector<unsigned long>	size;
	size.push_back( 500 );
 	Column* col7; col7 = tab->addColumn("string",Column::DString,"string data", "", size );

  	fill( tab );
  	check( tab );
	dataSetServer->close(set);

	// Read & check
  	set = dataSetServer->open("test.dat",DataSet::Read);
  	tab = set->table("some table");
  	check( tab );
  	dataSetServer->close(set);

  	set = dataSetServer->open("test.dat",DataSet::Read);
  	tab = set->table("some table");
	testSeek( tab );
  	dataSetServer->close(set);

	cout << "CPU time: " << cpuWatch.elapsed() / 1000000.0 << " secs" << endl;
	cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;

  	return 0;
}

