using namespace std;
#include <Dal.h>


static struct Cols {
  	char *			name;
	Column::DataType	type;
	char * 			units;
	char *			label;
} cols[] =
{
  	{ "bool",	Column::Bool,		"",	"bool data" },
  	{ "string",	Column::DString,	"",	"string data" },
 	{ "int8",	Column::Int8,		"m",	"int8 data" },
	//{ "uint16",	Column::Uint16,		"m",	"uint16 data" },
	{ "int16",	Column::Int16,		"m",	"int16 data" },
  	//{ "uint32",	Column::Uint32,		"m",	"uint32 data" },
  	{ "int32",	Column::Int32,		"m",	"int32 data" },
  	{ "real32",	Column::Real32,		"m",	"real32 data" },
  	{ "real64",	Column::Real64,		"m",	"real64 data" },
};

static const unsigned numberOfColumns = sizeof( cols ) / sizeof( cols[ 0 ] );
static unsigned long nRows;


template<class T>
static void
fillData( T * data, unsigned long size )
{
     for( unsigned long i = 0; i < size; ++i )
     {
     	data[ i ] = i + size;
     } // for
}


static void
fill( Table * table )
{
	for( unsigned int i = 0; i < table -> columns(); ++i )
	{
		Column * col = table -> column( i );
  		assert(col->elements() == 0);
 		assert(col->size() == Dal::zero );
  		for( unsigned long j = 0; j < col -> rows(); ++j )
 		{
     			CellData * c = col -> cellData( j );
     			assert( c -> parent() -> name() == col -> name() );
     			//assert( c -> size() == 0 );
     			c -> size( 10 + j );
     			assert( c -> size() == 10 + j );
			switch( col -> dataType() )
			{
				case Column::Bool :
     					for( unsigned long k = 0; k < c -> size(); ++k )
    					{
     						c -> boolData()[ k ] = k % 2 ? false : true;
    					} // for
					break;
				case Column::Int8 :
					fillData( c -> int8Data(), c -> size() );
					break;
				case Column::Uint16 :
					fillData( c -> uint16Data(), c -> size() );
					break;
				case Column::Int16 :
					fillData( c -> int16Data(), c -> size() );
					break;
				case Column::Int32 :
					fillData( c -> int32Data(), c -> size() );
					break;
				case Column::Uint32 :
					fillData( c -> uint32Data(), c -> size() );
					break;
				case Column::Real32 :
					fillData( c -> real32Data(), c -> size() );
					break;
				case Column::Real64 :
					fillData( c -> real64Data(), c -> size() );
					break;
				case Column::DString :
					for( unsigned int k = 0; k < c -> size(); ++k ) c -> stringData()[ k ] = 'a' + (j + k ) % 26;
					break;
				default :
					break;
			} // switch
			delete c;
		} // for
	} // for
}


template<class T>
static void
checkData( T * data, unsigned long size )
{
     for( unsigned long i = 0; i < size; ++i )
     {
     	assert( data[ i ] == T(i + size) );
     } // for
}


static void
check( Table * table )
{
	for( unsigned int i = 0; i < table -> columns(); ++i )
	{
		Column * col = table -> column( i );
  		assert(col->elements() == 0);
 		assert(col->size() == Dal::zero );
  		for( unsigned long j = 0; j < col -> rows(); ++j )
 		{
     			CellData * c = col -> cellData( j );
     			assert( c -> parent() -> name() == col -> name() );
     			//assert( c -> size() == 0 );
     			assert( c -> size() == 10 + j );
			switch( col -> dataType() )
			{
				case Column::Bool :
     					for( unsigned long k = 0; k < c -> size(); ++k )
    					{
     						assert( c -> boolData()[ k ] == bool8( k % 2 ? false : true ));
    					} // for
					break;
				case Column::Int8 :
					checkData( c -> int8Data(), c -> size() );
					break;
				case Column::Uint16 :
					checkData( c -> uint16Data(), c -> size() );
					break;
				case Column::Int16 :
					checkData( c -> int16Data(), c -> size() );
					break;
				case Column::Uint32 :
					checkData( c -> uint32Data(), c -> size() );
					break;
				case Column::Int32 :
					checkData( c -> int32Data(), c -> size() );
					break;
				case Column::Real32 :
					checkData( c -> real32Data(), c -> size() );
					break;
				case Column::Real64 :
					checkData( c -> real64Data(), c -> size() );
					break;
				case Column::DString :
					for( unsigned int k = 0; k < c -> size(); ++k )
					{
						assert( c -> stringData()[ k ] == char('a' + (j + k) % 26 ));
					} // for
					break;
				default :
					break;
			} // switch
     			//assert( c -> size() == 10 + j );
			delete c;
		} // for
	} // for
}


static void
basicChecks( Table * table )
{
  assert( table -> rows() == nRows );
  assert( table -> columns() == numberOfColumns );
  for( unsigned int i = 0; i < numberOfColumns; ++i )
  {
  	assert( table -> column( i ) -> dataType() == cols[ i ].type );
  	assert( table -> column( i ) -> name() == cols[ i ].name );
  	//assert( table -> column( i ) -> units() == cols[ i ].units );
  	//assert( table -> column( i ) -> label() == cols[ i ].label );
  } // for
}


static void
test( unsigned long numberOfRows )
{

  nRows = numberOfRows;

  DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
  Table* tab = set->addTable("someTable",nRows);
  for( unsigned int i = 0; i < numberOfColumns; ++i )
  {
  	tab->addColumn(
			cols[ i ].name,
			cols[ i ].type,
			cols[ i ].label,
			cols[ i ].units,
			Dal::zero );
  } // for

  // Basic checks
  basicChecks( tab );
  // Populate the columns
  fill( tab );
  // Checks
  check( tab );

  dataSetServer->close(set);

  set = dataSetServer->open("test.dat",DataSet::Read);
  tab = set -> table( 0 );
  // Basic checks
  basicChecks( tab );
  // Checks
  check( tab );
  dataSetServer->close(set);

  set = dataSetServer->open("test.dat",DataSet::Modify);
  tab = set -> table( 0 );
  // Basic checks
  basicChecks( tab );
  // Checks
  check( tab );
  dataSetServer->close(set);
}


int main()
{

  test( 0 );
  test( 1 );
  test( 10 );
  test( 100 );

  return 0;
}

